@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -32,11 +32,6 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "menu_searchedit.h"
 #include "menu_searchresults.h"
 #include <math.h>
-#ifdef HAVE_PCREPOSIX
-#include <pcreposix.h>
-#else
-#include <regex.h>
-#endif
 
 cBlacklists Blacklists;
 
@@ -66,6 +61,7 @@ cBlacklist::cBlacklist(void)
     useDayOfWeek = false;
     DayOfWeek = 0;
     buffer = NULL;
+    isGlobal = 0;
 
     useExtEPGInfo = false;
     catvalues = (char**) malloc(SearchExtCats.Count() * sizeof(char*));
@@ -92,7 +88,7 @@ cBlacklist::~cBlacklist(void)
   {
       cSearchExtCat *SearchExtCat = SearchExtCats.First();
       int index = 0;
-      while (SearchExtCat && index < (int)(sizeof(catvalues)/sizeof(char*))) 
+      while (SearchExtCat) 
       {
 	  free(catvalues[index]);
 	  SearchExtCat = SearchExtCats.Next(SearchExtCat);
@@ -106,12 +102,10 @@ cBlacklist::~cBlacklist(void)
 cBlacklist& cBlacklist::operator= (const cBlacklist &Blacklist)
 {
   memcpy(this, &Blacklist, sizeof(*this));
-  catvalues = (char**) malloc(SearchExtCats.Count() * sizeof(char*));
   cSearchExtCat *SearchExtCat = SearchExtCats.First();
   int index = 0;
   while (SearchExtCat) 
   {
-      catvalues[index] = (char*)malloc(MaxFileName);
       *catvalues[index] = 0;
       strcpy(catvalues[index], Blacklist.catvalues[index]);
       SearchExtCat = SearchExtCats.Next(SearchExtCat);
@@ -138,6 +132,7 @@ void cBlacklist::CopyFromTemplate(const cSearchExt* templ)
   useDayOfWeek = templ->useDayOfWeek;
   DayOfWeek = templ->DayOfWeek;
   useExtEPGInfo = templ->useExtEPGInfo;
+  isGlobal = 0;
 
   cSearchExtCat *SearchExtCat = SearchExtCats.First();
   int index = 0;
@@ -238,7 +233,7 @@ const char *cBlacklist::ToText(void)
       }
   }
 
-  msprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%s:%d:%d",
+  msprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%s:%d:%d:%d",
 	   ID,
 	   tmp_search,
 	   useTime,
@@ -259,7 +254,8 @@ const char *cBlacklist::ToText(void)
 	   useExtEPGInfo,
 	   useExtEPGInfo?tmp_catvalues:"",
 	   fuzzyTolerance,
-	   ignoreMissingEPGCats);
+	   ignoreMissingEPGCats,
+	   isGlobal);
 
 
   if (tmp_chanSel)
@@ -321,9 +317,15 @@ bool cBlacklist::Parse(const char *s)
 		    int fields = sscanf(value, "%d-%d", &minNum, &maxNum);
 		    if (fields == 0) // stored with ID
 		    {
+#ifdef __FreeBSD__
+			char *channelMinbuffer = MALLOC(char, 32);
+			char *channelMaxbuffer = MALLOC(char, 32);
+			int channels = sscanf(value, "%31[^|]|%31[^|]", channelMinbuffer, channelMaxbuffer);
+#else
 			char *channelMinbuffer = NULL;
 			char *channelMaxbuffer = NULL;
 			int channels = sscanf(value, "%a[^|]|%a[^|]", &channelMinbuffer, &channelMaxbuffer);
+#endif
 			channelMin = Channels.GetByChannelID(tChannelID::FromString(channelMinbuffer), true, true);
 			if (!channelMin)
 			{
@@ -384,6 +386,10 @@ bool cBlacklist::Parse(const char *s)
 		break;
             case 21: ignoreMissingEPGCats = atoi(value);
          	break;
+            case 22: isGlobal = atoi(value);
+         	break;
+	    default:
+	        break;
         } //switch
       }
       parameter++;
@@ -483,6 +489,8 @@ bool cBlacklist::ParseExtEPGEntry(const char *s)
 		    }
 		}
 	      break;
+	    default:
+              break;
         } //switch
       }
       parameter++;
