@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -29,13 +29,14 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "conflictcheck_thread.h"
 #include "recstatus.h"
 #include "timerstatus.h"
+#include "uservars.h"
 
 #define FULLMATCH 1000
 #define EPGLIMITBEFORE   (1 * 3600) // Time in seconds before a timer's start time and
 #define EPGLIMITAFTER    (1 * 3600) // after its stop time within which EPG events will be taken into consideration.
 
 // --- cConflictCheckTimerObj --------------------------------------------------------
-cConflictCheckTimerObj::cConflictCheckTimerObj(cTimer* Timer, time_t Start, time_t Stop, int Device) : cTimerObj(Timer), start(Start), stop(Stop), device(Device), conflCheckTime(NULL), concurrentTimers(NULL), ignore(false)
+cConflictCheckTimerObj::cConflictCheckTimerObj(cTimer* Timer, time_t Start, time_t Stop, int Device, int OrigIndex) : cTimerObj(Timer), start(Start), stop(Stop), device(Device), origIndex(OrigIndex), conflCheckTime(NULL), concurrentTimers(NULL), ignore(false)
 {
     event = Timer->Event();
     recDuration = 0;
@@ -43,6 +44,12 @@ cConflictCheckTimerObj::cConflictCheckTimerObj(cTimer* Timer, time_t Start, time
     lastRecStop = 0;
 }
 
+cConflictCheckTimerObj::~cConflictCheckTimerObj()
+{
+  // conflict checks works on a copy of a timer, so delete it again
+  DELETENULL(timer);
+}
+
 int cConflictCheckTimerObj::Compare(const cListObject &ListObject) const
 {
     cConflictCheckTimerObj *p = (cConflictCheckTimerObj *)&ListObject;
@@ -50,7 +57,7 @@ int cConflictCheckTimerObj::Compare(const cListObject &ListObject) const
     if (diff == 0)
 	diff = p->timer->Priority() - timer->Priority();
     if (diff == 0)
-	diff = timer->Index() - p->timer->Index();
+	diff = origIndex - p->origIndex;
     return diff;
 }
 
@@ -222,6 +229,7 @@ void cConflictCheck::Check()
 
 cList<cConflictCheckTimerObj>* cConflictCheck::CreateCurrentTimerList()
 {
+    LogFile.Log(3,"current timer list creation started");
     cList<cConflictCheckTimerObj>* CurrentTimerList = NULL;
 
     // collect single event timers
@@ -233,7 +241,12 @@ cList<cConflictCheckTimerObj>* cConflictCheck::CreateCurrentTimerList()
 	if (!ti->IsSingleEvent()) continue;
         // already recording?
 	int deviceNr = gl_recStatusMonitor->TimerRecDevice(ti)-1;
-	cConflictCheckTimerObj* timerObj = new cConflictCheckTimerObj(ti, ti->StartTime(), ti->StopTime(), deviceNr);
+
+        // create a copy of this timer 
+        cTimer* clone = new cTimer(*ti);
+        clone->SetEvent(ti->Event());
+
+	cConflictCheckTimerObj* timerObj = new cConflictCheckTimerObj(clone, ti->StartTime(), ti->StopTime(), deviceNr, ti->Index());
 	if (deviceNr >= 0)
 	{
 	    devices[deviceNr].recTimers.insert(timerObj);
@@ -248,7 +261,7 @@ cList<cConflictCheckTimerObj>* cConflictCheck::CreateCurrentTimerList()
     }
 
     // collect repeating timers from now until the date of the timer with tMax
-    time_t maxCheck = time(NULL) + EPGSearchConfig.checkMaxDays * SECSINDAY;
+    time_t maxCheck = time(NULL) + min(14,EPGSearchConfig.checkMaxDays) * SECSINDAY;
     tMax = max(tMax, maxCheck);
     for (ti = Timers.First(); ti; ti = Timers.Next(ti))
     {
@@ -280,7 +293,12 @@ cList<cConflictCheckTimerObj>* cConflictCheck::CreateCurrentTimerList()
 		    day += SECSINDAY;
 		    continue;
 		}
-		cConflictCheckTimerObj* timerObj = new cConflictCheckTimerObj(ti, Start, Start + ti->StopTime() - ti->StartTime(), deviceNr);
+
+		// create a copy of this timer 
+		cTimer* clone = new cTimer(*ti);
+		clone->SetEvent(ti->Event());
+
+		cConflictCheckTimerObj* timerObj = new cConflictCheckTimerObj(clone, Start, Start + ti->StopTime() - ti->StartTime(), deviceNr, ti->Index());
 		LogFile.Log(3,"add timer '%s' (%s, channel %s) for conflict check", ti->File(), DAYDATETIME(Start), CHANNELNAME(ti->Channel()));
 		if (deviceNr >= 0)
 		{
@@ -296,6 +314,7 @@ cList<cConflictCheckTimerObj>* cConflictCheck::CreateCurrentTimerList()
     }
 
     if (CurrentTimerList) CurrentTimerList->Sort();
+    LogFile.Log(3,"current timer list created");
     return CurrentTimerList;
 }
 
@@ -359,7 +378,7 @@ cList<cConflictCheckTime>* cConflictCheck::CreateConflictList(cList<cConflictChe
     LogFile.Log(3,"create conflict list");
     relevantConflicts = 0;
     numConflicts = 0;
-    maxCheck = time(NULL) + EPGSearchConfig.checkMaxDays * SECSINDAY;
+    maxCheck = time(NULL) + min(14, EPGSearchConfig.checkMaxDays) * SECSINDAY;
 
     // check each time
     for(cConflictCheckTime* checkTime = EvalTimeList->First(); checkTime; checkTime = EvalTimeList->Next(checkTime))
@@ -423,6 +442,7 @@ cList<cConflictCheckTime>* cConflictCheck::CreateConflictList(cList<cConflictChe
 int cConflictCheck::ProcessCheckTime(cConflictCheckTime* checkTime)
 {
     if (!checkTime) return 0;
+
     LogFile.Log(3,"check time %s", DAYDATETIME(checkTime->evaltime));
 
     LogFile.Log(3,"detach stopping timers");
@@ -491,6 +511,19 @@ int cConflictCheck::ProcessCheckTime(cConflictCheckTime* checkTime)
 }
 
 #if APIVERSNUM >= 10500
+
+eModuleStatus cConflictCheck::CamSlotModuleStatus(cCamSlot *CamSlot)
+{
+  if (!CamSlot) return msNone;
+  if ((int)camSlotStatusArray.size() != CamSlots.Count())
+    for (cCamSlot *CamSlot = CamSlots.First(); CamSlot; CamSlot = CamSlots.Next(CamSlot)) 
+      camSlotStatusArray.push_back(CamSlot->ModuleStatus());
+  if (CamSlot->Index() < (int)camSlotStatusArray.size())
+    return camSlotStatusArray[CamSlot->Index()];
+  else
+    return msNone;
+}
+
 int cConflictCheck::GetDevice(cConflictCheckTimerObj* TimerObj, bool* NeedsDetachReceivers)
 {
     int Priority = TimerObj->timer->Priority();
@@ -504,7 +537,7 @@ int cConflictCheck::GetDevice(cConflictCheckTimerObj* TimerObj, bool* NeedsDetac
     if (Channel->Ca() >= CA_ENCRYPTED_MIN) {
 	for (cCamSlot *CamSlot = CamSlots.First(); CamSlot; CamSlot = CamSlots.Next(CamSlot)) {
 	    SlotPriority[CamSlot->Index()] = MAXPRIORITY + 1; // assumes it can't be used
-	    if (CamSlot->ModuleStatus() == msReady) {
+	    if (CamSlotModuleStatus(CamSlot) == msReady) {
 		if (CamSlot->ProvidesCa(Channel->Caids())) {
 		    if (!ChannelCamRelations.CamChecked(Channel->GetChannelID(), CamSlot->SlotNumber())) {
 			SlotPriority[CamSlot->Index()] = CamSlot->Priority();
@@ -656,7 +689,7 @@ bool cConflictCheck::TimerInConflict(cTimer* timer)
 		{
 		    for (it2 = (*it)->concurrentTimers->begin(); it2 != (*it)->concurrentTimers->end(); it2++)
 		    {
-			if ((*it2)->timer == timer)
+			if ((*it2)->OrigTimer() == timer)
 			    return true;
 		    }
 		}
@@ -665,3 +698,38 @@ bool cConflictCheck::TimerInConflict(cTimer* timer)
     }
     return false;
 }
+
+void cConflictCheck::EvaluateConflCheckCmd()
+{
+  if (strlen(EPGSearchConfig.conflCheckCmd) > 0)
+    {
+      LogFile.Log(2,"evaluating conflict check command '%s'", EPGSearchConfig.conflCheckCmd);
+      for(cConflictCheckTime* ct = failedList->First(); ct; ct = failedList->Next(ct))
+	{
+	  if (ct->ignore) continue;
+	  std::set<cConflictCheckTimerObj*,TimerObjSort>::iterator it;
+	  for (it = ct->failedTimers.begin(); it != ct->failedTimers.end(); it++) 
+	    if ((*it) && !(*it)->ignore)
+	      {
+		string result = EPGSearchConfig.conflCheckCmd;
+		if (!(*it)->OrigTimer()) 
+		  {
+		    LogFile.Log(3,"timer has disappeared meanwhile");
+		    continue;
+		  }
+		else
+		  LogFile.Log(3,"evaluating conflict check command for timer '%s' (%s, channel %s)", (*it)->timer->File(), DAYDATETIME((*it)->start), CHANNELNAME((*it)->timer->Channel()));
+		
+		if ((*it)->Event())
+		  {
+		    cVarExpr varExprEvent(result);
+		    result = varExprEvent.Evaluate((*it)->Event());
+		  }
+		cVarExpr varExprTimer(result);
+		result =  varExprTimer.Evaluate((*it)->timer);
+		cVarExpr varExpr(result);
+		varExpr.Evaluate();
+	      }
+	}
+    }
+}
