@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -43,6 +43,7 @@ class cConflictCheckTimerObj : public cTimerObj
     time_t start;
     time_t stop;
     int device;
+    int origIndex;
     int recDuration;
     time_t lastRecStart;
     time_t lastRecStop;
@@ -50,11 +51,14 @@ class cConflictCheckTimerObj : public cTimerObj
     std::set<cConflictCheckTimerObj*,TimerObjSort>* concurrentTimers;
     bool ignore;
 
-    cConflictCheckTimerObj(cTimer* Timer, time_t Start, time_t Stop, int Device = -1);
+
+    cConflictCheckTimerObj(cTimer* Timer, time_t Start, time_t Stop, int Device = -1, int OrigIndex=-1);
+    ~cConflictCheckTimerObj();
     int Compare(const cListObject &ListObject) const;
     const cEvent* Event();
     const cEvent* SetEventFromSchedule();
     int Matches(const cEvent *Event, int *Overlap) const;
+    cTimer* OrigTimer() {return Timers.GetTimer(timer); }
 };
 
 class TimerObjSort
@@ -285,6 +289,7 @@ class cConflictCheck
 
     int numDevices;   
     time_t maxCheck;
+    std::vector<eModuleStatus> camSlotStatusArray;
  public:
     int relevantConflicts;
     int numConflicts;
@@ -303,6 +308,10 @@ class cConflictCheck
     void AddConflict(cConflictCheckTimerObj* TimerObj, cConflictCheckTime* Checktime, std::set<cConflictCheckTimerObj*>& pendingTimers);
     int ProcessCheckTime(cConflictCheckTime* checkTime);
     bool TimerInConflict(cTimer*);
+    void EvaluateConflCheckCmd();
+#if APIVERSNUM >= 10500
+    eModuleStatus CamSlotModuleStatus(cCamSlot *CamSlot);
+#endif
 };
 
 #endif
