@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -25,9 +25,9 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "conflictcheck_thread.h"
 #include "epgsearchcfg.h"
 #include "mail.h"
+#include "epgsearch.h"
 
 #define CONFLICTCHECK_NICE 19
-#define CONFLCHECK_WAIT 20
 
 cConflictCheckThread *cConflictCheckThread::m_Instance = NULL;
 time_t cConflictCheckThread::m_cacheNextConflict = 0;
@@ -77,6 +77,7 @@ void cConflictCheckThread::Exit(void) {
 
 void cConflictCheckThread::Stop(void) {
     m_Active = false;
+    Wait.Signal();
     Cancel(6);
 }
 
@@ -86,9 +87,13 @@ void cConflictCheckThread::Action(void)
 
     m_Active = true;
     // let VDR do its startup
-    if (!m_runOnce)   
-      for(int wait = 0; wait < CONFLCHECK_WAIT && m_Active; wait++)
-	sleepSec(1);
+    if (!m_runOnce)
+      {
+	if (!cPluginEpgsearch::VDR_readyafterStartup)
+	  LogFile.Log(2, "ConflictCheckThread: waiting for VDR to become ready...");
+	while(m_Active && !cPluginEpgsearch::VDR_readyafterStartup)
+	  Wait.Wait(1000);
+      }
 
     time_t nextUpdate = time(NULL);
     while (m_Active) 
@@ -96,16 +101,16 @@ void cConflictCheckThread::Action(void)
 	time_t now = time(NULL);
 	if (now >= nextUpdate || m_forceUpdate)
 	{
-        m_forceUpdate = false;
-	    if (Timers.BeingEdited()) 		
+	    m_forceUpdate = false;
+	    if (Timers.BeingEdited())
 	    {
-		sleepSec(1);
+		Wait.Wait(1000);
 		continue;
 	    }
 	    LogFile.iSysLog("timer conflict check started");
 
-	    cConflictCheck conflictCheck;   
-	    conflictCheck.Check(); 
+	    cConflictCheck conflictCheck;
+	    conflictCheck.Check();
 
 	    time_t nextConflict = 0;
 	    if (conflictCheck.relevantConflicts > 0)
@@ -127,8 +132,8 @@ void cConflictCheckThread::Action(void)
 		  cMailConflictNotifier mailNotifier;
 		  mailNotifier.SendConflictNotifications(conflictCheck);
 		}
-	    }
-	    
+	      conflictCheck.EvaluateConflCheckCmd();
+	    }	    
 	    // store for external access
 	    cConflictCheckThread::m_cacheNextConflict = conflictCheck.nextRelevantConflictDate;
 	    cConflictCheckThread::m_cacheRelevantConflicts = conflictCheck.relevantConflicts;
@@ -138,18 +143,20 @@ void cConflictCheckThread::Action(void)
 
 	    m_lastUpdate = time(NULL);
 	    int Intervall = EPGSearchConfig.conflictCheckIntervall;
-	    if (nextConflict > 0 && EPGSearchConfig.conflictCheckWithinLimit > 0 && 
+	    if (nextConflict > 0 && EPGSearchConfig.conflictCheckWithinLimit > 0 &&
 		nextConflict - time(NULL) < EPGSearchConfig.conflictCheckWithinLimit * 60)
 		Intervall = EPGSearchConfig.conflictCheckIntervall2;
 
 	    nextUpdate = long(m_lastUpdate/60)*60 + (Intervall * 60);
 	}
-	sleepSec(2); // to avoid high system load if time%30==0 
+	if (m_Active)
+	    Wait.Wait(2000); // to avoid high system load if time%30==0 ?????????????????????
+	// no waiting in the while loop if m_runOnce is true
 	while (m_Active && time(NULL)%30 != 0 && !m_runOnce) // sync heart beat to a multiple of 5secs
-	    sleepSec(1);
+	    Wait.Wait(1000);
     };
 
-    m_Active = false;
+    m_Active = false;	// always false at this point
     LogFile.iSysLog("Leaving conflict check thread");
 }
 
