@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,6 +30,9 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include <vdr/plugin.h>
 #include <vdr/status.h>
 #include <vdr/epg.h>
+#ifdef __FreeBSD__
+#include <netinet/in.h>
+#endif
 #include <getopt.h>
 #include "menu_event.h"
 #include "menu_main.h"
@@ -65,8 +68,9 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "menu_quicksearch.h"
 #include "menu_announcelist.h"
 #include "confdloader.h"
+#include "pending_notifications.h"
 
-static const char VERSION[]        = "0.9.24";
+static const char VERSION[]        = "1.0.0";
 static const char DESCRIPTION[]    =  trNOOP("search the EPG for repeats and more");
 
 // globals
@@ -79,6 +83,7 @@ bool isUTF8 = false;
 cLogFile LogFile;
 char *cLogFile::LogFileName = NULL;
 int cLogFile::loglevellimit = 0;
+bool cPluginEpgsearch::VDR_readyafterStartup = false;
 
 // external SVDRPCommand
 const char *cSVDRPClient::SVDRPSendCmd = "svdrpsend.pl";
@@ -171,7 +176,7 @@ bool cPluginEpgsearch::ProcessArgs(int argc, char *argv[])
       { "verbose",        required_argument, NULL, 'v' },
       { "mailcmd",        required_argument, NULL, 'm' },
       { "reloadmenuconf", no_argument, NULL, 'r' },
-      { NULL }
+      { NULL,             no_argument, NULL, 0 }
    };
 
    int c=0,i=0;
@@ -245,7 +250,7 @@ bool cPluginEpgsearch::Service(const char *Id, void *Data)
 
       return true;
    }
-   if (strcmp(Id, "Epgsearch-exttimeredit-v1.0") == 0) {
+   if (strcmp(Id, "Epgsearch-exttimeredit-v1.0") == 0 && !EPGSearchConfig.useVDRTimerEditMenu) {
       if (Data == NULL)
          return true;
       
@@ -260,7 +265,9 @@ bool cPluginEpgsearch::Service(const char *Id, void *Data)
       else
       {
          Epgsearch_updatesearchtimers_v1_0* serviceData = (Epgsearch_updatesearchtimers_v1_0*) Data;
-         updateForced = serviceData->showMessage?2:1;
+         if (!EPGSearchConfig.useSearchTimers) // enable search timer thread if necessary
+            cSearchTimerThread::Init((cPluginEpgsearch*) cPluginManager::GetPlugin("epgsearch"), true);
+         updateForced = serviceData->showMessage?3:1;
       }
       return true;
    }
@@ -350,7 +357,7 @@ bool cPluginEpgsearch::Service(const char *Id, void *Data)
                cSwitchTimer *lTimer = SwitchTimers.InSwitchList(serviceData->event);
                if (lTimer) {
                   serviceData->switchMinsBefore = lTimer->switchMinsBefore;
-                  serviceData->announceOnly     = lTimer->announceOnly;
+                  serviceData->announceOnly     = lTimer->mode;
                } // if
                serviceData->success=lTimer!=NULL;
                break;
@@ -359,7 +366,7 @@ bool cPluginEpgsearch::Service(const char *Id, void *Data)
                cSwitchTimer *lTimer = SwitchTimers.InSwitchList(serviceData->event);
                if (lTimer) {
                   lTimer->switchMinsBefore = serviceData->switchMinsBefore;
-                  lTimer->announceOnly     = serviceData->announceOnly;
+                  lTimer->mode             = serviceData->announceOnly;
                } else {
                   cMutexLock SwitchTimersLock(&SwitchTimers);	
                   SwitchTimers.Add(new cSwitchTimer(serviceData->event,serviceData->switchMinsBefore,serviceData->announceOnly));  
@@ -452,6 +459,7 @@ bool cPluginEpgsearch::Start(void)
    NoAnnounces.Load(AddDirectory(CONFIGDIR, "noannounce.conf"));
    DefTimerCheckModes.Load(AddDirectory(CONFIGDIR, "deftimerchkmodes.conf"));
    TimersDone.Load(AddDirectory(CONFIGDIR, "timersdone.conf"));
+   PendingNotifications.Load(AddDirectory(CONFIGDIR, "pendingnotifications.conf"));
 
    cSearchTimerThread::Init(this);    
    cSwitchTimerThread::Init();    
@@ -464,23 +472,8 @@ bool cPluginEpgsearch::Start(void)
 
 void cPluginEpgsearch::CheckUTF8()
 {
-#if APIVERSNUM >= 10503 
-   // Taken from VDR's vdr.c
-   char *CodeSet = NULL;
-   if (setlocale(LC_CTYPE, ""))
-     CodeSet = nl_langinfo(CODESET);
-   else {
-     char *LangEnv = getenv("LANG"); // last resort in case locale stuff isn't installed
-     if (LangEnv) {
-       CodeSet = strchr(LangEnv, '.');
-       if (CodeSet)
-	 CodeSet++; // skip the dot
-     }
-   }
-   
-   if (CodeSet && strcasestr(CodeSet, "UTF-8") != 0)
-     isUTF8=true;
-#endif
+  std::string CodeSet = GetCodeset();
+  isUTF8 = EqualsNoCase(CodeSet, "UTF-8");
 }
 
 void cPluginEpgsearch::Stop(void)
@@ -489,9 +482,14 @@ void cPluginEpgsearch::Stop(void)
    cSwitchTimerThread::Exit();
 }
 
-void cPluginEpgsearch::Housekeeping(void)
+void cPluginEpgsearch::MainThreadHook(void)
 {
-   // Perform any cleanup or other regular tasks.
+  if (!VDR_readyafterStartup)
+    {
+      // signal VDR is ready, otherwise the search timer thread could use SVDRP before it works
+      LogFile.Log(2, "VDR ready");
+      VDR_readyafterStartup = true;
+    }
 }
 
 cOsdObject *cPluginEpgsearch::DoInitialSearch(char* rcFilename)
@@ -643,9 +641,15 @@ bool cPluginEpgsearch::SetupParse(const char *Name, const char *Value)
    if      (!strcasecmp(Name, "CheckConflictsMinDuration"))  EPGSearchConfig.checkMinDuration = atoi(Value);
    if      (!strcasecmp(Name, "CheckConflictsAfterTimerProg"))  EPGSearchConfig.checkTimerConflAfterTimerProg = atoi(Value);
    if      (!strcasecmp(Name, "CheckConflictsOnRecording"))  EPGSearchConfig.checkTimerConflOnRecording = atoi(Value);
+   if      (!strcasecmp(Name, "ConflCheckCmd"))  strcpy(EPGSearchConfig.conflCheckCmd, Value);
+
    if      (!strcasecmp(Name, "NoConflMsgWhileReplay"))  EPGSearchConfig.noConflMsgWhileReplay = atoi(Value);
    if      (!strcasecmp(Name, "NoAnnounceWhileReplay"))  EPGSearchConfig.noAnnounceWhileReplay = atoi(Value);
    if      (!strcasecmp(Name, "TimerProgRepeat"))  EPGSearchConfig.TimerProgRepeat = atoi(Value);
+   if      (!strcasecmp(Name, "CheckEPGHours"))  EPGSearchConfig.checkEPGHours = atoi(Value);
+   if      (!strcasecmp(Name, "CheckEPGWarnByOSD"))  EPGSearchConfig.checkEPGWarnByOSD = atoi(Value);
+   if      (!strcasecmp(Name, "CheckEPGWarnByMail"))  EPGSearchConfig.checkEPGWarnByMail = atoi(Value);
+   if      (!strcasecmp(Name, "CheckEPGChannelgroup"))  EPGSearchConfig.checkEPGchannelGroupNr = atoi(Value);
 
    if      (!strcasecmp(Name, "TimeIntervallFRFF"))  EPGSearchConfig.timeShiftValue = atoi(Value);
    if      (!strcasecmp(Name, "ToggleGreenYellow"))  EPGSearchConfig.toggleGreenYellow = atoi(Value);
@@ -671,6 +675,8 @@ bool cPluginEpgsearch::SetupParse(const char *Name, const char *Value)
   
    if      (!strcasecmp(Name, "MailViaScript"))  EPGSearchConfig.mailViaScript = atoi(Value); 
    if      (!strcasecmp(Name, "MailNotificationSearchtimers"))  EPGSearchConfig.sendMailOnSearchtimers = atoi(Value); 
+   if      (!strcasecmp(Name, "MailNotificationSearchtimersHours")) EPGSearchConfig.sendMailOnSearchtimerHours = atoi(Value);
+   if      (!strcasecmp(Name, "MailNotificationSearchtimersLastAt")) EPGSearchConfig.lastMailOnSearchtimerAt = atol(Value);
    if      (!strcasecmp(Name, "MailNotificationConflicts"))  EPGSearchConfig.sendMailOnConflicts = atoi(Value); 
    if      (!strcasecmp(Name, "MailAddress")) strcpy(EPGSearchConfig.MailAddress, Value);
    if      (!strcasecmp(Name, "MailAddressTo")) strcpy(EPGSearchConfig.MailAddressTo, Value);
@@ -685,8 +691,14 @@ bool cPluginEpgsearch::SetupParse(const char *Name, const char *Value)
 
    if      (!strcasecmp(Name, "LastMailConflicts")) strcpy(EPGSearchConfig.LastMailConflicts, Value);
 
+
    return true;
 }
 
+cString cPluginEpgsearch::Active(void)
+{
+  return cSearchTimerThread::justRunning?tr("search timer update running"):NULL;
+}
+
 VDRPLUGINCREATOR(cPluginEpgsearch); // Don't touch this!
 
