@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -102,9 +102,10 @@ bool cSearchExtCat::Parse(const char *s)
 		break;
 	    }
 	    case 5:  
-           searchmode = atoi(value);
+	      searchmode = atoi(value);
+	      break;
+	    default:
 	      break;
-	    break;
         } //switch
       }
       parameter++;
