@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -60,15 +60,15 @@ class cShowMode: public cListObject
 	    else
 		sprintf(description, "%02d:%02d", iTime/100, iTime%100);
 	}
-    char* GetDescription() { return description; }
-    int GetTime() { return itime; }
-    bool GetUsage() { return useIt; }
+    const char* GetDescription() { return description; }
+    int GetTime() const { return itime; }
+    bool GetUsage() const { return useIt; }
 
-    void SetDescription(const char* szD) { if (szD)  strcpy(description, szD); }
+    void SetDescription(const char* szD) { if (szD)  strncpy(description, szD, sizeof(description)); }
     void SetTime(int iT) { itime = iT; }
     void SetUsage(bool bU) { useIt = bU; }
     int Compare(const cListObject &ListObject) const;
-    showMode GetMode() { return mode; }
+    showMode GetMode() const { return mode; }
 };
 
 
@@ -107,7 +107,7 @@ cEPGSearchConfig(void);
     int useExternalSVDRP;
     int ignorePrimary;
     char defrecdir[MaxFileName];
-    cShowMode ShowModes[6];
+    cShowMode ShowModes[showModeMax];
     int useVDRTimerEditMenu;
     int showChannelGroups;
     int showDaySeparators;
@@ -141,6 +141,13 @@ cEPGSearchConfig(void);
     int TimerProgRepeat;
     int maxChannelMenuNow;
     int noConflMsgWhileReplay;
+    int sendMailOnSearchtimerHours;
+    int checkEPGHours;
+    int checkEPGWarnByOSD;
+    int checkEPGWarnByMail;
+    int checkEPGchannelGroupNr;
+    time_t lastMailOnSearchtimerAt;
+    char conflCheckCmd[MaxFileName*10];    
 };
 
 extern cEPGSearchConfig EPGSearchConfig;
