@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -38,15 +38,9 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "uservars.h"
 #include "blacklist.h"
 #include <math.h>
-#ifdef HAVE_PCREPOSIX
-#include <pcreposix.h>
-#else
-#include <regex.h>
-#endif
 
 cSearchExts SearchExts;
 cSearchExts SearchTemplates;
-extern bool updateForced;
 
 #define MAX_SUBTITLE_LENGTH 40
 
@@ -86,6 +80,7 @@ cSearchExt::cSearchExt(void)
    useVPS = false;
    action = searchTimerActionRecord;
    useExtEPGInfo = false;
+   contentsFilter = "";
    catvalues = (char**) malloc(SearchExtCats.Count() * sizeof(char*));
    cSearchExtCat *SearchExtCat = SearchExtCats.First();
    int index = 0;
@@ -100,6 +95,8 @@ cSearchExt::cSearchExt(void)
    compareTitle = 1;
    compareSubtitle = 1;
    compareSummary = 1;
+   compareSummaryMatchInPercent = 90;
+   compareDate = 0;
    allowedRepeats = 0;
    catvaluesAvoidRepeat = 0;
    repeatsWithinDays = 0;
@@ -107,7 +104,7 @@ cSearchExt::cSearchExt(void)
    recordingsKeep = 0;
    switchMinsBefore = 1;
    pauseOnNrRecordings = 0;
-   blacklistMode = blacklistsNo; // no blacklists
+   blacklistMode = blacklistsOnlyGlobal; // no blacklists
    blacklists.Clear();
    fuzzyTolerance = 1;
    useInFavorites = 0;
@@ -119,6 +116,7 @@ cSearchExt::cSearchExt(void)
    useAsSearchTimerTil = 0;
    ignoreMissingEPGCats = 0;
    unmuteSoundOnSwitch = 0;
+   skipRunningEvents = false;
 }
 
 cSearchExt::~cSearchExt(void)
@@ -132,11 +130,11 @@ cSearchExt::~cSearchExt(void)
    {
       cSearchExtCat *SearchExtCat = SearchExtCats.First();
       int index = 0;
-      while (SearchExtCat && index < (int)(sizeof(catvalues)/sizeof(char*))) 
+      while (SearchExtCat) 
       {
-         free(catvalues[index]);
-         SearchExtCat = SearchExtCats.Next(SearchExtCat);
-         index++;
+	free(catvalues[index]);
+	SearchExtCat = SearchExtCats.Next(SearchExtCat);
+	index++;
       }
       free(catvalues);
       catvalues = NULL;
@@ -148,12 +146,11 @@ cSearchExt& cSearchExt::operator= (const cSearchExt &SearchExt)
    CopyFromTemplate(&SearchExt);
    ID = SearchExt.ID;
    strcpy(search, SearchExt.search);
-   catvalues = (char**) malloc(SearchExtCats.Count() * sizeof(char*));
+
    cSearchExtCat *SearchExtCat = SearchExtCats.First();
    int index = 0;
    while (SearchExtCat) 
    {
-      catvalues[index] = (char*)malloc(MaxFileName);
       *catvalues[index] = 0;
       strcpy(catvalues[index], SearchExt.catvalues[index]);
       SearchExtCat = SearchExtCats.Next(SearchExtCat);
@@ -163,13 +160,14 @@ cSearchExt& cSearchExt::operator= (const cSearchExt &SearchExt)
    return *this;
 }
 
-void cSearchExt::CopyFromTemplate(const cSearchExt* templ)
+  void cSearchExt::CopyFromTemplate(const cSearchExt* templ, bool ignoreChannelSettings)
 {
    options = templ->options;
    useTime = templ->useTime;
    startTime = templ->startTime;
    stopTime = templ->stopTime;
-   useChannel = templ->useChannel;
+   if (!ignoreChannelSettings)
+     useChannel = templ->useChannel;
    useCase = templ->useCase;
    mode = templ->mode;
    useTitle = templ->useTitle;
@@ -190,6 +188,7 @@ void cSearchExt::CopyFromTemplate(const cSearchExt* templ)
    useVPS = templ->useVPS;
    action = templ->action;
    useExtEPGInfo = templ->useExtEPGInfo;
+   contentsFilter = templ->contentsFilter;
    switchMinsBefore = templ->switchMinsBefore;
    pauseOnNrRecordings = templ->pauseOnNrRecordings;
 
@@ -202,19 +201,24 @@ void cSearchExt::CopyFromTemplate(const cSearchExt* templ)
       index++;
    }
   
-   channelMin = templ->channelMin;
-   channelMax = templ->channelMax;
-   if (channelGroup)
-   {
-      free(channelGroup);
-      channelGroup = NULL;
-   }
-   if (templ->channelGroup)
-      channelGroup = strdup(templ->channelGroup);
+   if (!ignoreChannelSettings)
+     {
+       channelMin = templ->channelMin;
+       channelMax = templ->channelMax;
+       if (channelGroup)
+	 {
+	   free(channelGroup);
+	   channelGroup = NULL;
+	 }
+       if (templ->channelGroup)
+	 channelGroup = strdup(templ->channelGroup);
+     }
    avoidRepeats = templ->avoidRepeats;
    compareTitle = templ->compareTitle;
    compareSubtitle = templ->compareSubtitle;
    compareSummary = templ->compareSummary;
+   compareSummaryMatchInPercent = templ->compareSummaryMatchInPercent;
+   compareDate = templ->compareDate;
    allowedRepeats = templ->allowedRepeats;
    catvaluesAvoidRepeat = templ->catvaluesAvoidRepeat;
    repeatsWithinDays = templ->repeatsWithinDays;
@@ -246,6 +250,15 @@ bool cSearchExt::operator< (const cListObject &ListObject)
    return strcasecmp(search, SE->search) < 0;
 }
 
+char* replaceSpecialChars(const char* in)
+{
+   char* tmp_in = strdup(in);
+   while(strstr(tmp_in, "|"))
+      tmp_in = strreplace(tmp_in, "|", "!^pipe^!"); // ugly: replace a pipe with something, 
+   strreplace(tmp_in, ':', '|');
+   return tmp_in;
+}
+
 const char *cSearchExt::ToText()
 {
    char tmp_Start[5] = "";
@@ -253,23 +266,13 @@ const char *cSearchExt::ToText()
    char tmp_minDuration[5] = "";
    char tmp_maxDuration[5] = "";
    cString tmp_chanSel;
-   char* tmp_search = NULL;
-   char* tmp_directory = NULL;
    char* tmp_catvalues = NULL;
    char* tmp_blacklists = NULL;
 
    free(buffer);
-   tmp_search = strdup(search);
-   while(strstr(tmp_search, "|"))
-      tmp_search = strreplace(tmp_search, "|", "!^pipe^!"); // ugly: replace a pipe with something, 
-                                                            // that should not happen to be part of a search string
-   tmp_directory = strdup(directory);
-   while(strstr(tmp_directory, "|"))
-      tmp_directory = strreplace(tmp_directory, "|", "!^pipe^!"); // ugly: replace a pipe with something, 
-   // that should not happen to be part of a search string
-
-   strreplace(tmp_search, ':', '|');
-   strreplace(tmp_directory, ':', '|');
+   char* tmp_search = replaceSpecialChars(search);
+   char* tmp_directory = replaceSpecialChars(directory);
+   char* tmp_contentsFilter = replaceSpecialChars(contentsFilter.c_str());
 
    if (useTime)
    {
@@ -347,7 +350,7 @@ const char *cSearchExt::ToText()
       }      
    }
 
-   msprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%ld:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%d:%ld:%ld:%d:%d",
+   msprintf(&buffer, "%d:%s:%d:%s:%s:%d:%s:%d:%d:%d:%d:%d:%d:%s:%s:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%ld:%d:%d:%d:%d:%d:%d:%s:%d:%d:%d:%d:%d:%d:%ld:%ld:%d:%d:%d:%s:%d",
             ID,
             tmp_search,
             useTime,
@@ -398,12 +401,16 @@ const char *cSearchExt::ToText()
 	    useAsSearchTimerFrom,
 	    useAsSearchTimerTil,
 	    ignoreMissingEPGCats, 
-	    unmuteSoundOnSwitch);
+	    unmuteSoundOnSwitch,
+	    compareSummaryMatchInPercent,
+	    contentsFilter.c_str(),
+	    compareDate);
 
    if (tmp_search) free(tmp_search);
    if (tmp_directory) free(tmp_directory);
    if (tmp_catvalues) free(tmp_catvalues);
    if (tmp_blacklists) free(tmp_blacklists);
+   if (tmp_contentsFilter) free(tmp_contentsFilter);
 
    return buffer;
 }
@@ -462,9 +469,15 @@ bool cSearchExt::Parse(const char *s)
                      int fields = sscanf(value, "%d-%d", &minNum, &maxNum);
                      if (fields == 0) // stored with ID
                      {
+#ifdef __FreeBSD__
+                        char *channelMinbuffer = MALLOC(char, 32);
+                        char *channelMaxbuffer = MALLOC(char, 32);
+                        int channels = sscanf(value, "%31[^|]|%31[^|]", channelMinbuffer, channelMaxbuffer);
+#else
                         char *channelMinbuffer = NULL;
                         char *channelMaxbuffer = NULL;
                         int channels = sscanf(value, "%a[^|]|%a[^|]", &channelMinbuffer, &channelMaxbuffer);
+#endif
                         channelMin = Channels.GetByChannelID(tChannelID::FromString(channelMinbuffer), true, true);
                         if (!channelMin) 
                         {
@@ -547,7 +560,7 @@ bool cSearchExt::Parse(const char *s)
                   break;
                case 31: compareTitle = atoi(value);
                   break;
-               case 32: compareSubtitle = atoi(value);
+	       case 32: compareSubtitle = atoi(value)>0?1:0;
                   break;
                case 33: compareSummary = atoi(value);
                   break;
@@ -597,6 +610,17 @@ bool cSearchExt::Parse(const char *s)
 	    case 51:
 	      unmuteSoundOnSwitch = atoi(value);
 	      break;
+	    case 52:
+	      compareSummaryMatchInPercent = atoi(value);
+	      break;
+	    case 53:
+	      contentsFilter = value;
+	      break;
+	    case 54:
+	      compareDate = atoi(value);
+	      break;
+	    default:
+	      break;
             } //switch
          }
          parameter++;
@@ -606,10 +630,13 @@ bool cSearchExt::Parse(const char *s)
 
    strreplace(directory, '|', ':');
    strreplace(search, '|', ':');
+   strreplace(contentsFilter, "|", ":");
+
    while(strstr(search, "!^pipe^!"))
       strreplace(search, "!^pipe^!", "|");
    while(strstr(directory, "!^pipe^!"))
       strreplace(directory, "!^pipe^!", "|");
+   strreplace(contentsFilter, "!^pipe^!", "|");
 
    if (disableSearchtimer && useAsSearchTimer)
    {
@@ -629,15 +656,18 @@ char* cSearchExt::BuildFile(const cEvent* pEvent) const
       return file;
     
    const char *Subtitle = pEvent ? pEvent->ShortText() : NULL;
-   char SubtitleBuffer[MAX_SUBTITLE_LENGTH];
+   char SubtitleBuffer[Utf8BufSize(MAX_SUBTITLE_LENGTH)];
    if (isempty(Subtitle))
    {
-      sprintf(SubtitleBuffer, "%s-%s", GETDATESTRING(pEvent), GETTIMESTRING(pEvent));
-      Subtitle = SubtitleBuffer;
+     time_t Start = pEvent->StartTime();
+     struct tm tm_r;
+     strftime(SubtitleBuffer, sizeof(SubtitleBuffer), "%Y.%m.%d-%R-%a", localtime_r(&Start, &tm_r));      
+     Subtitle = SubtitleBuffer;
    }
-   else if (strlen(Subtitle) > MAX_SUBTITLE_LENGTH) 
+   else if (Utf8StrLen(Subtitle) > MAX_SUBTITLE_LENGTH) 
    {
-      strn0cpy(SubtitleBuffer, Subtitle, MAX_SUBTITLE_LENGTH);
+      Utf8Strn0Cpy(SubtitleBuffer, Subtitle, sizeof(SubtitleBuffer));
+      SubtitleBuffer[Utf8SymChars(SubtitleBuffer, MAX_SUBTITLE_LENGTH)] = 0;
       Subtitle = SubtitleBuffer;
    }
 
@@ -808,6 +838,8 @@ bool cSearchExt::ParseExtEPGEntry(const char *s)
                      }
                   }
                   break;
+	    default:
+	      break;
             } //switch
          }
          parameter++;
@@ -838,7 +870,6 @@ cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEven
       p1 = Events->First();
 
    time_t tNow=time(NULL);
-   char* szTest = NULL;
    char* searchText = strdup(search);
 
    int searchStart = 0, searchStop = 0;
@@ -867,25 +898,15 @@ cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEven
          break;
       }
 
-      if (szTest)
-      {
-         free(szTest);
-         szTest = NULL;
-      }
+      if (skipRunningEvents && tNow > p->StartTime())
+	continue;
 
       // ignore events without title
       if (!p->Title() || !*p->Title())
          continue;
         
-      msprintf(&szTest, "%s%s%s%s%s", (useTitle?(p->Title()?p->Title():""):""), (useSubtitle||useDescription)?"~":"",
-               (useSubtitle?(p->ShortText()?p->ShortText():""):""),useDescription?"~":"",
-               (useDescription?(p->Description()?p->Description():""):""));     
-	
       if (tNow < p->EndTime() + (inspectTimerMargin?(MarginStop * 60):0)) 
       {	    
-         if (!useCase)
-            ToLower(szTest);
-	
          if (useTime)
          {
             time_t tEvent = p->StartTime();
@@ -951,11 +972,27 @@ cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEven
             }
          }
 
-         if (*szTest)
+	 char* szTest = NULL;
+	 msprintf(&szTest, "%s%s%s%s%s", (useTitle?(p->Title()?p->Title():""):""), (useSubtitle||useDescription)?"~":"",
+               (useSubtitle?(p->ShortText()?p->ShortText():""):""),useDescription?"~":"",
+               (useDescription?(p->Description()?p->Description():""):""));     
+	
+         if (!useCase)
+            ToLower(szTest);
+	
+         if (szTest && *szTest)
          {
             if (!MatchesSearchMode(szTest, searchText, mode," ,;|~", fuzzyTolerance))
+	    {
+	       free(szTest);
                continue;
+	    }
          }
+	 if (szTest)
+	   free(szTest);
+
+	 if (contentsFilter.size() > 0 && !MatchesContentsFilter(p))
+	    continue;
 
          if (useExtEPGInfo && !MatchesExtEPGInfo(p))
             continue;
@@ -963,8 +1000,6 @@ cEvent * cSearchExt::GetEventBySearchExt(const cSchedule *schedules, const cEven
          break;
       }
    }
-   if (szTest)
-      free(szTest);
    free(searchText);
    return pe;
 }
@@ -1071,15 +1106,26 @@ cSearchResults* cSearchExt::Run(int PayTVMode, bool inspectTimerMargin, int eval
       CheckRepeatTimers(pSearchResults);
    }
 
+   skipRunningEvents = false;
    return pSearchResults;
 }
 
 cSearchResults* cSearchExt::GetBlacklistEvents(int MarginStop)
 {
-   if (blacklistMode == blacklistsNo) return NULL;
+   if (blacklistMode == blacklistsNone) return NULL;
 
    cMutexLock BlacklistLock(&Blacklists);
    cSearchResults* blacklistEvents = NULL; 
+   if (blacklistMode == blacklistsOnlyGlobal)
+   {
+      cBlacklist* tmpblacklist = Blacklists.First();
+      while(tmpblacklist)
+      {
+	if (tmpblacklist->isGlobal)
+	    blacklistEvents = tmpblacklist->Run(blacklistEvents, MarginStop);
+	tmpblacklist = Blacklists.Next(tmpblacklist);
+      }
+   }
    if (blacklistMode == blacklistsAll)
    {
       cBlacklist* tmpblacklist = Blacklists.First();
@@ -1110,16 +1156,16 @@ void cSearchExt::CheckRepeatTimers(cSearchResults* pResults)
       return;
 
    LogFile.Log(2,"analysing repeats for search timer '%s'...", search);
-   if (action == searchTimerActionAnnounceOnly)
+   if (action != searchTimerActionRecord)
    {
-      LogFile.Log(3,"search timer set to 'announce only', so skip all");
+      LogFile.Log(3,"search timer not set to 'record', so skip all");
       return;
    }
    
    cSearchResult* pResultObj = NULL;
    for (pResultObj = pResults->First(); pResultObj; pResultObj = pResults->Next(pResultObj))
    {
-      if (action == searchTimerActionAnnounceOnly) // only announce if there is no timer for the event
+      if (action != searchTimerActionRecord) // only announce if there is no timer for the event
       {
          pResultObj->needsTimer = false; 		
          continue;
@@ -1129,8 +1175,8 @@ void cSearchExt::CheckRepeatTimers(cSearchResults* pResults)
       // check if this event was already recorded
       int records = 0;
       cRecDone* firstRec = NULL;
-      LogFile.Log(3,"get count recordings");
-      records = RecsDone.GetCountRecordings(pEvent, this, &firstRec);
+      LogFile.Log(3,"get count recordings with %d%% match", compareSummaryMatchInPercent);
+      records = RecsDone.GetCountRecordings(pEvent, this, &firstRec, compareSummaryMatchInPercent);
       LogFile.Log(3,"recordings: %d", records);	
       
       if (records > allowedRepeats) // already recorded
@@ -1153,7 +1199,7 @@ void cSearchExt::CheckRepeatTimers(cSearchResults* pResults)
          
          if (!pResultObjP->needsTimer) continue;
          
-         if (EventsMatch(pEvent, pEventP, compareTitle, compareSubtitle, compareSummary, catvaluesAvoidRepeat))
+         if (EventsMatch(pEvent, pEventP, compareTitle, compareSubtitle, compareSummary, compareDate, catvaluesAvoidRepeat))
          {
             if (!pFirstResultMatching) pFirstResultMatching = pResultObjP;
             plannedTimers++;
@@ -1373,6 +1419,64 @@ bool cSearchExt::IsActiveAt(time_t t)
   return true;
 }
 
+bool cSearchExt::HasContent(int contentID)
+{
+  for(unsigned int i=0; i<contentsFilter.size();i+=2)
+  {
+    std::string hexContentID = contentsFilter.substr(i,2);
+    if(hexContentID.size()!=2) return false;
+    std::istringstream iss(hexContentID);
+    int tmpContentID =0;
+    if(!(iss>>std::noshowbase>>std::hex>>tmpContentID)) return false;
+    if (contentID == tmpContentID) return true;
+  }
+  return false;
+}
+
+void cSearchExt::SetContentFilter(int* contentStringsFlags)
+{
+  // create the hex array of content descriptor IDs
+  string tmp;
+  contentsFilter = "";
+  for(unsigned int i=0; contentStringsFlags && i<=CONTENT_DESCRIPTOR_MAX; i++)
+    {
+      if (contentStringsFlags[i])
+	{
+	  std::ostringstream oss;
+	  oss<<std::hex<<std::noshowbase<<i;
+	  contentsFilter += oss.str();
+	}
+    }
+}
+
+bool cSearchExt::MatchesContentsFilter(const cEvent* e)
+{
+#if APIVERSNUM < 10711 
+  return true;
+#else
+  if (!e) return false;
+  // check if each content filter ID is contained in the events descriptors
+  for(unsigned int i=0; i<contentsFilter.size();i+=2)
+  {
+    std::string hexContentID = contentsFilter.substr(i,2);
+    if(hexContentID.size()!=2) return false;
+    std::istringstream iss(hexContentID);
+    int searchContentID =0;
+    if(!(iss>>std::hex>>searchContentID)) return false;
+    int c=0, eventContentID=0;
+    bool found = false;
+    while((eventContentID=e->Contents(c++)) > 0)
+      if (eventContentID == searchContentID) 
+	{
+	  found = true;
+	  break;
+	}
+    if (!found) return false;
+  }
+  return true;
+#endif
+}
+
 // -- cSearchExts ----------------------------------------------------------------
 bool cSearchExts::Load(const char *FileName)
 {
@@ -1558,3 +1662,8 @@ bool cSearchExts::CheckForAutoDelete(cSearchExt* SearchExt)
    }
    return delSearch;
 }
+
+cSearchResult::cSearchResult(const cEvent* Event, int searchID) : event(Event), blacklist(NULL), needsTimer(true) 
+{
+  search = SearchExts.GetSearchFromID(searchID);
+}
