@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include <vdr/plugin.h>
 #include <vdr/interface.h>
 #include <vdr/skins.h>
+#include <string>
 
 #include "log.h"
 #include "epgsearchtools.h"
@@ -40,16 +41,19 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 
 typedef enum
 {
-    blacklistsNo = 0,
+    blacklistsOnlyGlobal = 0,
     blacklistsSelection,
-    blacklistsAll
+    blacklistsAll,
+    blacklistsNone
 } blacklistModes;
 
 typedef enum
 {
     searchTimerActionRecord=0,
-    searchTimerActionAnnounceOnly,
-    searchTimerActionSwitchOnly
+    searchTimerActionAnnounceViaOSD,
+    searchTimerActionSwitchOnly,
+    searchTimerActionAnnounceAndSwitch,
+    searchTimerActionAnnounceViaMail
 } searchTimerAction;
 
 class cSearchExt;
@@ -62,6 +66,7 @@ class cSearchResult : public cListObject {
     const cBlacklist* blacklist;
     bool needsTimer;
     cSearchResult(const cEvent* Event, const cSearchExt* Search) : event(Event), search(Search), blacklist(NULL), needsTimer(true) {}
+    cSearchResult(const cEvent* Event, int searchID);
     cSearchResult(const cEvent* Event, const cBlacklist* Blacklist) : event(Event), search(NULL), blacklist(Blacklist), needsTimer(true) {}
 };
 
@@ -134,6 +139,7 @@ public:
   int      MarginStop;
   int      useVPS;
   int      action;
+  std::string contentsFilter;
   int      useExtEPGInfo;
   char**   catvalues;
   cChannel *channelMin;
@@ -143,6 +149,8 @@ public:
   int      compareTitle;
   int      compareSubtitle;
   int      compareSummary;
+  int      compareSummaryMatchInPercent;
+  int      compareDate;
   int      allowedRepeats;
   unsigned long catvaluesAvoidRepeat;
   int      repeatsWithinDays;
@@ -162,7 +170,7 @@ public:
   time_t   useAsSearchTimerTil;
   int      ignoreMissingEPGCats;
   int      unmuteSoundOnSwitch;
-
+  bool     skipRunningEvents;
   static char *buffer;
 public:
   cSearchExt(void);
@@ -189,20 +197,23 @@ public:
   cSearchResults* Run(int PayTVMode = -1, bool inspectTimerMargin = false, int evalLimitMins = 0, cSearchResults* pPrevResults = NULL, bool suppressRepeatCheck = false); 
   void CheckRepeatTimers(cSearchResults* pResults);
   void CheckExistingRecordings(cSearchResults* pResults);
-  void CopyFromTemplate(const cSearchExt* templ);
+  void CopyFromTemplate(const cSearchExt* templ, bool ignoreChannelSettings = false);
   cSearchResults* GetBlacklistEvents(int MarginStop = 0);
   void OnOffTimers(bool);
   void DeleteAllTimers();
   cTimerObjList* GetTimerList(cTimerObjList* timerList);
   int GetCountRecordings();
   bool IsActiveAt(time_t t);
+  bool HasContent(int contentID);
+  void SetContentFilter(int* contentStringsFlags);
+  bool MatchesContentsFilter(const cEvent* e);
 };
 
 class cSearchExts : public cList<cSearchExt>, public cMutex {
  private:
     char *fileName;
     bool allowComments;
-    void Clear(void)
+    virtual void Clear(void)
 	{
 	    cMutexLock SearchExtsLock(this);
 	    free(fileName);
@@ -211,7 +222,7 @@ class cSearchExts : public cList<cSearchExt>, public cMutex {
 	}
  public:
   cSearchExts(void) { fileName = NULL; }
-  virtual ~cSearchExts() { free(fileName); }
+  virtual ~cSearchExts() { Clear(); free(fileName); }
 
 public:
     bool Load(const char *FileName = NULL);
