@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -49,7 +49,7 @@ const tI18nPhrase PhrasesLocal[] = {
    "",				// TODO 
    "Suoratoiminto EPGSearch-laajennoksen haulle",
    "",				// TODO Polski
-   "Acceso directo al men de bsqueda EPGSearch",	// Espaol
+   "Acceso directo al men de bsquedas en EPG",	// Espaol
    "",				// TODO  (Greek)
    "",				// TODO Svenska
    "",				// TODO Romaneste
@@ -72,7 +72,7 @@ const tI18nPhrase PhrasesLocal[] = {
    "",				// TODO
    "Etsi ohjelmaoppaasta",
    "",				// TODO
-   "Buscar",
+   "Bsquedas en EPG",
    "",				// TODO
    "",				// TODO
    "",				// TODO
@@ -95,7 +95,7 @@ const tI18nPhrase PhrasesLocal[] = {
    "",				// TODO 
    "EpgSearch-laajennoksen hakutoiminto pvalikossa",
    "",				// TODO Polski
-   "Buscar EPG en men principal",
+   "Bsquedas en EPG en men principal",
    "",				// TODO  (Greek)
    "",				// TODO Svenska
    "",				// TODO Romaneste
