@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -38,6 +38,7 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "menu_dirselect.h"
 #include "menu_deftimercheckmethod.h"
 #include "conflictcheck.h"
+#include "menu_main.h"
 
 using std::string;
 using std::set;
@@ -45,6 +46,7 @@ using std::set;
 template< class Iter > Iter advance_copy( Iter it, std::size_t count = 1) { using std::advance; advance( it, count ); return it; }
 
 extern int updateForced;
+extern int exitToMainMenu;
 
 const char **cPluginEpgsearch::SVDRPHelpPages(void)
 {
@@ -61,10 +63,11 @@ const char **cPluginEpgsearch::SVDRPHelpPages(void)
       "    Edit an existing search\n",
       "MODS ID ON|OFF\n"
       "    Turn on/off 'Use as search timer'\n",
-      "UPDS [ OSD ]\n"
+      "UPDS [ OSD ] [ SCAN ]\n"
       "    Update search timers.\n"
       "    If the optional keyword 'OSD' is passed, an OSD message\n"
-      "    will inform about update completion.",  
+      "    will inform about update completion. With 'SCAN' you can\n"
+      "    trigger an EPG scan before the searchtimer udpate.",
       "UPDD\n"
       "    Reload epgsearchdone.data",
       "SETS <ON|OFF>\n"
@@ -127,6 +130,11 @@ const char **cPluginEpgsearch::SVDRPHelpPages(void)
       "LSCC [ REL ]\n"
       "    Returns the current timer conflicts. With the option\n"
       "    'REL' only relevant conflicts are listed",
+      "MENU [ NOW|PRG|SUM ]\n"
+      "    Calls one of the main menus of epgsearch or the summary\n"
+      "    of the current event\n",
+      "UPDT\n"
+      "    Reload search timers from epgsearch.conf",
       NULL
    };
    return HelpPages;
@@ -136,8 +144,25 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
 {
    if (strcasecmp(Command, "UPDS") == 0) 
    {
-      updateForced = (Option && strcasecmp(Option, "OSD") == 0)?2:1;
-      return cString("update triggered");
+     if (!EPGSearchConfig.useSearchTimers) // enable search timer thread if necessary
+       cSearchTimerThread::Init((cPluginEpgsearch*) cPluginManager::GetPlugin("epgsearch"), true);
+     updateForced = 1;
+     if (Option)
+       {
+	 char *pstrOptionToken, *pptr;
+	 char *pstrOptions=strdup(Option);
+	 pstrOptionToken=strtok_r(pstrOptions, " ", &pptr);     
+	 while(pstrOptionToken) 
+	   {
+	     if (strcasecmp(Option, "OSD")==0)
+	       updateForced |= UPDS_WITH_OSD;
+	     if (strcasecmp(Option, "SCAN")==0)
+	       updateForced |= UPDS_WITH_EPGSCAN;
+	     pstrOptionToken=strtok_r(NULL, "|", &pptr);
+	   }
+	 free(pstrOptions);
+       }
+     return cString("update triggered");
    }
    else if (strcasecmp(Command, "UPDD") == 0) 
    {
@@ -583,6 +608,13 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
          return cString("no results");	
       }
    }
+   else if (strcasecmp(Command, "UPDT") == 0)
+   {
+     if (SearchExts.Load(AddDirectory(CONFIGDIR, "epgsearch.conf")))
+       return cString("reload epgsearch.conf successful");
+     else
+       return cString("reload epgsearch.conf failed");
+   }
    // ---------------------
    // recording directories
    else if (strcasecmp(Command, "LSRD") == 0) 
@@ -740,45 +772,50 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
    }
    else if (strcasecmp(Command, "RENC") == 0) 
    {
-      if (*Option) 
-      {
-         char* pipePos = strchr(Option, '|');
+     if (*Option) 
+       {
+         const char* pipePos = strchr(Option, '|');
          if (pipePos)
-         {
-            *pipePos = 0;
-            const char* oldName = Option;
-            const char* newName = pipePos+1;
-            if (strlen(oldName) > 0 && strlen(newName) > 0)
-            {
-               cChannelGroup *changrp = ChannelGroups.GetGroupByName(Option);
-               if (changrp)
-               {
-                  strcpy(changrp->name, newName);
-                  cMutexLock SearchExtsLock(&SearchExts);
-                  cSearchExt *SearchExt = SearchExts.First();
-                  while (SearchExt) 
-                  {
-                     if (SearchExt->useChannel == 2 && 
-                         SearchExt->channelGroup && 
-                         strcmp(SearchExt->channelGroup, oldName) == 0)
-                     {
-                        free(SearchExt->channelGroup);
-                        SearchExt->channelGroup = strdup(newName);
-                     }
-                     SearchExt = SearchExts.Next(SearchExt);
-                  }
-                  ChannelGroups.Save();
-                  SearchExts.Save();
-                  return cString::sprintf("renamed channel group '%s' to '%s'", oldName, newName);
-
-               }		    
-               else
-               {
-                  ReplyCode = 901;
-                  return cString::sprintf("channel group '%s' not defined", Option);
-               }
-            }
-         }
+	   {
+	     int index = pipePos - Option;
+	     char* oldName = strdup(Option);
+	     *(oldName + index) = 0;
+	     const char* newName = oldName + index + 1;
+	     if (strlen(oldName) > 0 && strlen(newName) > 0)
+	       {
+		 cChannelGroup *changrp = ChannelGroups.GetGroupByName(oldName);
+		 if (changrp)
+		   {
+		     strcpy(changrp->name, newName);
+		     cMutexLock SearchExtsLock(&SearchExts);
+		     cSearchExt *SearchExt = SearchExts.First();
+		     while (SearchExt) 
+		       {
+			 if (SearchExt->useChannel == 2 && 
+			     SearchExt->channelGroup && 
+			     strcmp(SearchExt->channelGroup, oldName) == 0)
+			   {
+			     free(SearchExt->channelGroup);
+			     SearchExt->channelGroup = strdup(newName);
+			   }
+			 SearchExt = SearchExts.Next(SearchExt);
+		       }
+		     ChannelGroups.Save();
+		     SearchExts.Save();
+		     cString strReturn = cString::sprintf("renamed channel group '%s' to '%s'", oldName, newName);
+		     free(oldName);
+		     return strReturn;
+		     
+		   }		    
+		 else
+		   {
+		     free(oldName);
+		     ReplyCode = 901;
+		     return cString::sprintf("channel group '%s' not defined", Option);
+		   }
+	       }
+	     free(oldName);
+	   }
          ReplyCode = 901;
          return cString("Error in channel group parameters");	
       }
@@ -1082,7 +1119,7 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
          {
             cSearchExt* search = SearchTemplates.Get(i);
             if (search)
-               sBuffer += string(search->ToText()) + string((i<SearchExts.Count()-1)?"\n":"");
+               sBuffer += string(search->ToText()) + string((i<SearchTemplates.Count()-1)?"\n":"");
          }
          return sBuffer.c_str();
       }
@@ -1240,13 +1277,13 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
 		 if (relOnly && (*it)->ignore) continue;
 		 std::ostringstream timerpart;
 		 int recPart = (*it)->recDuration * 100 / ((*it)->stop - (*it)->start);
-		 timerpart << (*it)->timer->Index()+1 << "|" << recPart << "|";
+		 timerpart << (*it)->origIndex+1 << "|" << recPart << "|";
 		 std::set<cConflictCheckTimerObj*,TimerObjSort>::iterator itcc;
 		 if ((*it)->concurrentTimers)
 		   {
 		   std::ostringstream cctimers;
 		   for (itcc = (*it)->concurrentTimers->begin(); itcc != (*it)->concurrentTimers->end(); itcc++) 
-		     cctimers << (cctimers.str().empty()?"":"#") << (*itcc)->timer->Index()+1;
+		     cctimers << (cctimers.str().empty()?"":"#") << (*itcc)->origIndex+1;
 		   timerpart << cctimers.str();
 		   }
 		 timerparts << (timerparts.str().empty()?"":":") << timerpart.str();
@@ -1262,6 +1299,34 @@ cString cPluginEpgsearch::SVDRPCommand(const char *Command, const char *Option,
          return cString("no conflicts found");	    	
        }
    }
-
+   else if (strcasecmp(Command, "MENU") == 0) 
+     {
+       if (*Option)
+	 {
+	   if (cMenuSearchMain::forceMenu == 0)
+	     {
+	       if (strcasecmp(Option, "PRG") == 0)
+		 cMenuSearchMain::forceMenu = 2;
+	       else if (strcasecmp(Option, "NOW") == 0)
+		 cMenuSearchMain::forceMenu = 1;
+	       else if (strcasecmp(Option, "SUM") == 0)
+		 cMenuSearchMain::forceMenu = 3;
+	       else
+		 {
+		   ReplyCode = 901;
+		   return cString::sprintf("unknown option '%s'", Option);
+		 }
+	       cRemote::CallPlugin("epgsearch");
+	       return "menu called";
+	     }
+	   else
+	     {
+	       cRemote::Put(kBack);
+	       exitToMainMenu = 1;
+	       return "menu closed";
+	     }
+	 } 
+     }
+   
    return NULL;
 }
