@@ -1,9 +1,36 @@
+/*
+Copyright (C) 2004-2011 Christian Wieninger
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation; either version 2
+of the License, or (at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+Or, point your browser to http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
+
+The author can be reached at cwieninger@gmx.de
+
+The project's page is at http://winni.vdr-developer.org/epgsearch
+*/
+
 #include <algorithm>
 #include <string>
 #include <fstream>
 #include <iostream>
 #include <ctype.h>
-
+#include <netdb.h>
+#include <arpa/inet.h>
+#ifdef __FreeBSD__
+#include <netinet/in.h>
+#endif
 #include "uservars.h"
 #include "epgsearchtools.h"
 #include "epgsearchext.h"
@@ -13,16 +40,22 @@
 #include "distance.h"
 #include "md5.h"
 #include "afuzzy.h"
+#include "i18n.h"
+#include "timerstatus.h"
+#if APIVERSNUM >= 10503 
+#include <langinfo.h>
+#endif
 
 #ifdef HAVE_PCREPOSIX
 #include <pcreposix.h>
+#elif defined(HAVE_LIBTRE)
+#include <tre/regex.h>
 #else
 #include <regex.h>
 #endif
 
-const char AllowedChars[] = "$ abcdefghijklmnopqrstuvwxyz0123456789-.#~\\^$[]|()*+?{}/:%@&";
-
-#define MATCHLIMIT 0.9
+const char AllowedChars[] = trNOOP("$ abcdefghijklmnopqrstuvwxyz0123456789-.,#~\\^$[]|()*+?{}/:%@&_");
+extern bool isUTF8;
 
 int CompareEventTime(const void *p1, const void *p2)
 {
@@ -46,16 +79,17 @@ int CompareEventChannel(const void *p1, const void *p2)
       return ch1 - ch2;
 }
 
-char* IndentMenuItem(const char* szString)
+char* IndentMenuItem(const char* szString, int indentions)
 {
    char* szIndented = NULL;
-   asprintf(&szIndented, "  %s", szString);
+   msprintf(&szIndented, "%*s", strlen(szString)+indentions*2, szString);
    return szIndented;
 }
 
 // based on Nordlicht's EPG-Plugin
 void PatchFont(eDvbFont DvbFont)
 {
+#if VDRVERSNUM < 10503
    const cFont *font = cFont::GetFont(DvbFont);
    int num_rows_global = font->Height();
    int num_rows = num_rows_global+2;
@@ -267,11 +301,13 @@ void PatchFont(eDvbFont DvbFont)
    }
  
    cFont::SetFont(DvbFont, font_data);
+#endif
 }
 
+
 bool MatchesSearchMode(const char* szTest, const char* searchText, int mode, const char* delim, int tolerance)
 {
-   if (strlen(szTest) > 0)
+   if (szTest && *szTest)
    {
       if (mode == 0) // substring
          return (strstr(szTest, searchText) != NULL); 
@@ -319,11 +355,11 @@ bool MatchesSearchMode(const char* szTest, const char* searchText, int mode, con
          else
             return false;
       } 
-      else if (mode == 4)
+      else if (mode == 4) // regexp
       {
          regex_t re;
 
-         if ( 0 == regcomp(&re, searchText, REG_EXTENDED) )
+         if ( 0 == regcomp(&re, searchText, REG_EXTENDED | REG_NOSUB) )
          {
             int status = regexec( &re, szTest, 0, NULL, 0);
             regfree(&re);
@@ -331,9 +367,9 @@ bool MatchesSearchMode(const char* szTest, const char* searchText, int mode, con
          }
          return false;
       }	
-      else if (mode == 5)
+      else if (mode == 5) // fuzzy
       {
-         AFUZZY af = { NULL, NULL, NULL, NULL, NULL, NULL };
+	 AFUZZY af = { NULL, NULL, NULL, NULL, NULL, NULL, { 0 }, { 0 }, 0, 0, 0, 0, 0, 0 };
          string query = searchText?searchText:"";
          if (query.size() > 32) query = query.substr(0, 32);
          afuzzy_init(query.c_str(), tolerance, 0, &af);
@@ -342,6 +378,25 @@ bool MatchesSearchMode(const char* szTest, const char* searchText, int mode, con
          afuzzy_free(&af);
          return (res > 0);
       }
+      else if (mode >= 10 && mode <= 15) 
+      {
+         int testvalue = atoi(szTest);
+         int value = atoi(searchText);
+	 if (value == 0) return true;
+
+         if (mode == 10) // less 
+            return testvalue < value;
+         else if (mode == 11) // less or equal
+            return testvalue <= value;
+         else if (mode == 12) // greater
+            return testvalue > value;
+         else if (mode == 13) // greater or equal
+            return testvalue >= value;
+         else if (mode == 14) // equal
+            return testvalue == value;
+         else if (mode == 15) // not equal
+            return testvalue != value;
+      }
    }
    return false;
 }
@@ -349,9 +404,24 @@ bool MatchesSearchMode(const char* szTest, const char* searchText, int mode, con
 void ToLower(char* szText)
 {
    if (!szText)
-      return;
-   for (int loop = 0; szText[loop] !=0; loop++)
-      szText[loop] = tolower(szText[loop]);
+     return;
+
+   if (!isUTF8)
+   {
+     for (int loop = 0; szText[loop] !=0; loop++)
+       szText[loop] = tolower(szText[loop]);
+     return;
+   }
+   else
+   {
+     int length = strlen(szText)+1;
+     uint* valueUtf8 = new uint[length];
+     int lengthUtf8 = Utf8ToArray(szText, valueUtf8, length);
+     for(int i=0; i<lengthUtf8; i++)
+       valueUtf8[i] = Utf8to(lower, valueUtf8[i]);
+     Utf8FromArray(valueUtf8, szText, length);
+     delete [] valueUtf8;
+   }
 }
 
 char* GetExtEPGValue(const cEvent* e, cSearchExtCat* SearchExtCat)
@@ -368,7 +438,7 @@ char* GetExtEPGValue(const char* description, const char* catname)
    char* tmp = NULL;
     
    // search the category, must be at beginnig of a line
-   asprintf(&tmp, "\n%s: ", catname);
+   msprintf(&tmp, "\n%s: ", catname);
    char* descr = strdup(description);
    char* cat = NULL;
    if ((cat = strstr(descr, tmp)) == NULL)
@@ -388,7 +458,7 @@ char* GetExtEPGValue(const char* description, const char* catname)
    cat[endpos] = 0;
 
    char* value = NULL;
-   asprintf(&value, "%s", cat + strlen(tmp)-1);
+   msprintf(&value, "%s", cat + strlen(tmp)-1);
    free(descr);
    free(tmp);
 
@@ -403,33 +473,44 @@ char* GetAuxValue(const char* aux, const char* name)
    char* descr = strdup(aux);
    char* beginaux = strstr(descr, "<epgsearch>");
    char* endaux = strstr(descr, "</epgsearch>");
-   if (!beginaux || !endaux) return NULL;
-   strn0cpy(descr, beginaux + strlen("<epgsearch>"), endaux - (beginaux + strlen("<epgsearch>")) + 1);
-   if (strcmp(name, "epgsearch") == 0) return descr; // full aux
+   if (!beginaux || !endaux) {
+      free(descr);
+      return NULL;
+      }
+   
+   beginaux +=  11;  // strlen("<epgsearch>");
+   endaux[0] = 0;
+   memmove(descr, beginaux, endaux - beginaux + 1);
+   
+   if (strcmp(name, "epgsearch") == 0) 
+      return descr; // full aux
 
+   int namelen = strlen(name);
    char catname[100] = "";
-   sprintf(catname, "<%s>", name);
-
-   char* cat = NULL;
-   int iPosCat = FindIgnoreCase(descr, catname);
-   if (iPosCat < 0) return NULL;
-   cat = descr + iPosCat;
+   catname[0] = '<';
+   memcpy(catname + 1, name, namelen);
+   catname[1 + namelen] = '>';
+   catname[2 + namelen] = 0;
 
+   char* cat = strcasestr(descr, catname);
+   if (!cat) {
+      free(descr);
+      return NULL;
+      }
+      
+   cat += namelen + 2;
    char* end = strstr(cat, "</");
-   int endpos = strlen(cat);
-   if (end)
-      endpos = end - cat;
-   else
-   {
+   if (!end) {
       free(descr);	
       return NULL;
-   }
-   cat[endpos] = 0;
-	
-   char* value = NULL;
-   asprintf(&value, "%s", cat + strlen(name)+2);
-   free(descr);
+      }
+   end[0] = 0;
    
+   int catlen = end - cat + 1;
+   char* value = (char *) malloc(catlen);
+   memcpy(value, cat, catlen);
+   
+   free(descr);
    return value;
 }
 
@@ -469,54 +550,34 @@ char *strreplacei(char *s, const char *s1, const char *s2)
    char *p = strcasestr(s, s1);
    if (p) {
       int of = p - s;
-      int l  = strlen(s);
+      int l = strlen(s);
       int l1 = strlen(s1);
-      int l2 = strlen(s2);
+      int l2 = 0;
+      if (s2)
+        l2 = strlen(s2);
       if (l2 > l1)
-         s = (char *)realloc(s, strlen(s) + l2 - l1 + 1);
+         s = (char *)realloc(s, l + l2 - l1 + 1);
       if (l2 != l1)
          memmove(s + of + l2, s + of + l1, l - of - l1 + 1);
-      strncpy(s + of, s2, l2);
+      memcpy(s + of, s2, l2);
    }
    return s;
 }
 
-unsigned int strcharcount(char* source, char ch)
+std::string strreplace(
+  std::string& result, 
+  const std::string& replaceWhat, 
+  const std::string& replaceWithWhat)
 {
-   unsigned int count;
-    
-   for (count = 0; *source; source++)
-      if (*source == ch)
-         count++;
-   return (count);
+  while(1)
+  {
+    const int pos = result.find(replaceWhat);
+    if (pos==-1) break;
+    result.replace(pos,replaceWhat.size(),replaceWithWhat);
+  }
+  return result;
 }
 
-char *strreplaceall(char* source, char ch, char* replacement)
-{
-   int		replen;
-   unsigned int	chnum;
-   register char	*sptr, *nptr;
-   char		*newval;
-    
-   chnum = strcharcount(source, ch);
-    
-   if (chnum == 0)
-      return (NULL);
-    
-   replen = strlen(replacement);
-   newval = (char*) malloc(strlen(source) + (chnum * replen) + 1);
-   nptr = newval;
-   for (sptr = source; *sptr; sptr++)
-      if (*sptr == ch) {
-         strcpy(nptr, replacement);
-         nptr += replen;
-      }
-      else
-         *nptr++ = *sptr;
-   *nptr = '\0';
-    
-   return (newval);
-}
 
 void sleepMSec(long ms)
 {
@@ -528,42 +589,41 @@ void sleepSec(long s)
    sleepMSec(s * 1000);
 }
 
-bool SendViaSVDRP(const char* SVDRPcmd) 
+bool SendViaSVDRP(cString SVDRPcmd) 
 {
    bool bSuccess = true;
-   char *cmdbuf = NULL;
+   cString cmdbuf;
    if (EPGSearchConfig.useExternalSVDRP)
    {
-      asprintf(&cmdbuf, "%s -p %d \"%s\"", 
+     cmdbuf = cString::sprintf("%s -p %d \"%s\"", 
                cSVDRPClient::SVDRPSendCmd,
                EPGSearchConfig.SVDRPPort,
-               SVDRPcmd);
+               *SVDRPcmd);
 
       FILE *p = popen(cmdbuf, "r");
       if (p)
          pclose(p);
       else
       {
-         LogFile.eSysLog("can't open pipe for command '%s'", cmdbuf);
+         LogFile.eSysLog("can't open pipe for command '%s'", *cmdbuf);
          bSuccess = false;
       }
    }
    else
    {
-      asprintf(&cmdbuf, "%s", SVDRPcmd);
-      cSVDRPClient client(EPGSearchConfig.SVDRPPort);
-      if (!client.SendCmd(cmdbuf))
+      cmdbuf = SVDRPcmd;
+      cSVDRPClient client;
+      if (!client.SendCmd(*cmdbuf))
       {
-         LogFile.eSysLog("command '%s' failed", cmdbuf);
+         LogFile.eSysLog("command '%s' failed", *cmdbuf);
          bSuccess = false;
       }
    }
 	    
-   free(cmdbuf);
    return bSuccess;
 }
 
-int SendMsg(const char* Message, bool confirm, int seconds) 
+int SendMsg(cString Message, bool confirm, int seconds) 
 {
    int Keys = Skins.QueueMessage(mtInfo, Message, seconds, confirm?seconds+2:0);
    return Keys;
@@ -606,48 +666,6 @@ int TriggeredFromSearchTimerID(const cTimer* timer)
       return -1;
 }
 
-bool CopyFile(const char* file_path_from,  const char* file_path_to)
-{
-   FILE* f_from;		/* stream of source file. */
-   FILE* f_to;			/* stream of target file. */
-   char buf[MAXPARSEBUFFER+1];   /* input buffer.          */
-
-   /* open the source and the target files. */
-   f_from = fopen(file_path_from, "r");
-   if (!f_from) {
-      LogFile.eSysLog("Cannot open source file: %s", file_path_from);
-      return false;
-   }
-   f_to = fopen(file_path_to, "w+");
-   if (!f_to) {
-      LogFile.eSysLog("Cannot open target file: %s", file_path_to);
-      return false;
-   }
-
-   /* copy source file to target file, line by line. */
-   while (fgets(buf, MAXPARSEBUFFER+1, f_from)) {
-      if (fputs(buf, f_to) == EOF) {  /* error writing data */
-         LogFile.eSysLog("Error writing to target file: %s", file_path_to);
-         return false;
-      }
-   }
-   if (!feof(f_from)) { /* fgets failed _not_ due to encountering EOF */
-      LogFile.eSysLog("Error reading from source file: %s", file_path_from);
-      return false;
-   }
-
-   /* close source and target file streams. */
-   if (fclose(f_from) == EOF) {
-      LogFile.eSysLog("Error when closing source file: %s", file_path_from);
-      return false;
-   }
-   if (fclose(f_to) == EOF) {
-      LogFile.eSysLog("Error when closing target file: %s", file_path_to);
-      return false;
-   }
-   return true;
-}
-
 double FuzzyMatch(const char* s1, const char* s2, int maxLength)
 {
    Distance D;
@@ -656,7 +674,7 @@ double FuzzyMatch(const char* s1, const char* s2, int maxLength)
    return (fMaxLength - dist)/fMaxLength;
 }
 
-bool DescriptionMatches(const char* eDescr, const char* rDescr)
+bool DescriptionMatches(const char* eDescr, const char* rDescr, int matchLimit)
 {
    if (eDescr == NULL && rDescr == NULL) return true;
    if (eDescr == NULL && rDescr != NULL) return false;
@@ -666,15 +684,21 @@ bool DescriptionMatches(const char* eDescr, const char* rDescr)
    if (l_eDescr == l_rDescr && strcmp(eDescr, rDescr) == 0) return true;
 
    // partial match:
-   // first check the length, should only be different at MATCHLIMIT
+   // first check the length, should only be different at match limit
    int minLength = min(l_eDescr, l_rDescr);
    int maxLength = max(l_eDescr, l_rDescr);
-   if (double(minLength)/double(maxLength) < MATCHLIMIT)
+   if (100*double(minLength)/double(maxLength) < matchLimit)
       return false;
 
    // last try with Levenshtein Distance, only compare the first 1000 chars 
    double fMatch = FuzzyMatch(eDescr, rDescr, 1000);
-   if (fMatch > MATCHLIMIT)
+   double tmp_matchlimit = matchLimit/100.0;
+   if(maxLength - minLength < 5)
+   {
+      tmp_matchlimit = 0.95;
+      LogFile.Log(2,"difference between both descriptions is < 5 setting matchlimit to: %.2f %%", tmp_matchlimit*100);
+   }
+   if (fMatch > tmp_matchlimit)
    {
       LogFile.Log(2,"match is: %.2f %%", fMatch*100);
       return true;
@@ -702,7 +726,7 @@ const cEvent* GetEvent(cTimer* timer)
       }
       if (seconds == 0)
          LogFile.Log(2,"waiting for EPG info...");
-      sleep(1);
+      sleepSec(1);
    }
    LogFile.Log(1,"no EPG info available");
    return NULL;
@@ -714,35 +738,30 @@ const cEvent* GetEvent(cTimer* timer)
 // value are separated with ': '
 #define MAXCATNAMELENGTH 40
 #define MAXCATVALUELENGTH 60
+
 char* GetRawDescription(const char* descr)
 {
-   if (!descr) return NULL;
+   if (!descr || !*descr) return NULL;
 
-   char* rawDescr = NULL;
+   char* rawDescr = (char*) calloc(strlen(descr)+1, sizeof(char));
+   
    const char* tmp = descr;    
    while(tmp)
    {
       // extract a single line
-      char* line = strdup(tmp);
-      char* lf = strchr(line, '\n');
-      if (lf) *lf = 0;
+      const char* lf = strchr(tmp, '\n');
+      char* line = NULL;
+      if (lf)
+	line = strndup(tmp, lf-tmp);
+      else
+	line = strdup(tmp);
+      //      if (lf) *lf = 0;
       // check for category
       char* delim = strstr(line, ": ");
       if (!delim || 
           (delim && (delim - line > MAXCATNAMELENGTH || strlen(line) - (delim - line) + 2 > MAXCATVALUELENGTH)))
-         if (strlen(line) > 0)
-         {
-            if (rawDescr)
-            {
-               char* tmp = strdup(rawDescr);
-               free(rawDescr); 
-               rawDescr = NULL;
-               asprintf(&rawDescr, "%s%s", tmp, line);
-               free(tmp);
-            }
-            else
-               rawDescr = strdup(line);
-         }
+         if (*line)
+	   strcat(rawDescr, line);
 	
       if (lf) tmp += strlen(line)+1; else tmp = NULL;
       free(line);
@@ -765,7 +784,7 @@ void PrepareTimerFile(const cEvent* event, cTimer* timer)
       if (addSubtitle)
       {
          char tmp[MaxFileName] = "";
-         sprintf(tmp, "%s~%s", event->Title(), event->ShortText());
+         snprintf(tmp, MaxFileName, "%s~%s", event->Title(), event->ShortText());
          timer->SetFile(tmp);
       }
    }
@@ -787,7 +806,7 @@ void PrepareTimerFile(const cEvent* event, cTimer* timer)
    }
 }
 
-bool EventsMatch(const cEvent* event1, const cEvent* event2, bool compareTitle, bool compareSubtitle, bool compareSummary, unsigned long catvaluesAvoidRepeat)
+bool EventsMatch(const cEvent* event1, const cEvent* event2, bool compareTitle, int compareSubtitle, bool compareSummary, int compareDate, unsigned long catvaluesAvoidRepeat, int matchLimit)
 {
    if (!event1 || !event2) return false;
    if (event1 == event2) return true;
@@ -815,10 +834,14 @@ bool EventsMatch(const cEvent* event1, const cEvent* event2, bool compareTitle,
       std::transform(Subtitle1.begin(), Subtitle1.end(), Subtitle1.begin(), tolower);
       std::transform(Subtitle2.begin(), Subtitle2.end(), Subtitle2.begin(), tolower);
    }
+   string compareExpression = "";
+   if (compareDate == 1) compareExpression = "%date%";
+   if (compareDate == 2) compareExpression = "%year%-%week%";
+   if (compareDate == 3) compareExpression = "%year%-%month%";
 
    bool match = false;
    if ((!compareTitle || Title1 == Title2) &&
-       (!compareSubtitle || Subtitle1 == Subtitle2))
+       (!compareSubtitle || (Subtitle1 == Subtitle2 && Subtitle1!="")))
    {	
       const char* Descr1    = event1->Description();
       const char* Descr2    = event2->Description();
@@ -826,11 +849,19 @@ bool EventsMatch(const cEvent* event1, const cEvent* event2, bool compareTitle,
       {
          char* rawDescr1 = GetRawDescription(Descr1);
          char* rawDescr2 = GetRawDescription(Descr2);
-         match = DescriptionMatches(rawDescr1, rawDescr2);
+         match = DescriptionMatches(rawDescr1, rawDescr2, matchLimit);
          free(rawDescr1);
          free(rawDescr2);
          if (!match) return false;
       }
+      if (compareExpression.size() > 0)
+      {
+	   cVarExpr varExpr(compareExpression); 
+	   string resEvent1 = varExpr.Evaluate(event1);
+	   string resEvent2 = varExpr.Evaluate(event2);
+	   if (resEvent1 != resEvent2)
+	     return false;
+      }
       if (catvaluesAvoidRepeat != 0) // check categories
       {
          bool bCatMatch = ((Descr1 && Descr2) || (!Descr1 && !Descr2));
@@ -874,32 +905,51 @@ int ChannelNrFromEvent(const cEvent* pEvent)
 
 void DelTimer(int index)
 {
-   char *cmdbuf = NULL;    
-   asprintf(&cmdbuf, "DELT %d", index);    
-   LogFile.Log(2, "delete timer %d", index);
-   SendViaSVDRP(cmdbuf);
-	      
-   free(cmdbuf); 
+  cString cmdbuf = cString::sprintf("DELT %d", index);    
+  LogFile.Log(2, "delete timer %d", index);
+  SendViaSVDRP(cmdbuf);
+  gl_timerStatusMonitor->SetConflictCheckAdvised(); 
 }
 
 char* FixSeparators(char* buffer, char sep)
 {
-   char tmp[5];
-   sprintf(tmp, "%c\t", sep);
-   buffer = strreplacei(buffer, tmp, "\t");
-   sprintf(tmp, "%c \t", sep);
-   buffer = strreplacei(buffer, tmp, "\t");
-   sprintf(tmp, "%c ", sep);
-   if (strstr(buffer, tmp) == buffer + strlen(buffer) - 2) buffer[strlen(buffer) - 2] = 0;
-   if (buffer[strlen(buffer)-1] == sep) buffer[strlen(buffer) - 1] = 0;
-
-   sprintf(tmp, "\t%c", sep);
-   buffer = strreplacei(buffer, tmp, "\t");
-   sprintf(tmp, "\t %c", sep);
-   buffer = strreplacei(buffer, tmp, "\t");
-   sprintf(tmp, "\t  %c", sep);
-   buffer = strreplacei(buffer, tmp, "\t");
-   return buffer;
+  int l = strlen(buffer);
+  char *dest = buffer;
+  for (int i = 0; i < l; i ++) {
+    char c = buffer[i];
+    int j = i;
+    if (c == sep) {
+      for (j = i + 1; (j < l) & (buffer[j] == ' '); j++)
+	;
+      
+      if ((j <= l) | (i + 1 < j)) {
+	switch (buffer[j]) {
+	case '\t':
+	  i = j;
+	  c = '\t';
+	  break;
+	case 0:
+	  i = j;
+	  c = 0;
+	  break;
+	default:
+	  break;
+	}
+      }
+    }
+    if (c == '\t') {
+      for (; (j < l) & (buffer[j] == ' '); j++)
+	;
+      if (j < l && buffer[j] == sep) {
+	buffer[j] = '\t';
+	i = j - 1;
+	continue;
+      }
+    }
+    *dest++ = c;
+  }
+  *dest = 0;
+  return buffer;
 }
 
 cString DateTime(time_t t)
@@ -922,25 +972,30 @@ string NumToString(long num)
 
 int FindIgnoreCase(const string& expr, const string& query)
 {
-   string exprLower = expr;
-   std::transform(exprLower.begin(), exprLower.end(), exprLower.begin(), tolower);
-   string queryLower = query;
-   std::transform(queryLower.begin(), queryLower.end(), queryLower.begin(), tolower);
-   int pos = exprLower.find(queryLower);
-   return pos;
+  const char *p = expr.c_str();
+  const char *r = strcasestr(p, query.c_str());
+
+  if (!r)
+     return -1;
+  return r - p;
 }
 
 bool EqualsNoCase(const string& a, const string& b)
 {
-   return (FindIgnoreCase(a,b) == 0 && a.size() == b.size());
+   return strcasecmp(a.c_str(), b.c_str()) == 0;
 }
 
 string Strip(const string& input)
 {
-   string str = input;
-   while(str[0] == ' ') str.replace(0,1, "");
-   while(str[str.size()-1] == ' ') str.replace(str.size()-1,1, "");
-   return str;
+  string str = input;
+  string::size_type pos = str.find_last_not_of(' ');
+  if(pos != string::npos) {
+    str.erase(pos + 1);
+    pos = str.find_first_not_of(' ');
+    if(pos != string::npos) str.erase(0, pos);
+  }
+  else str.erase(str.begin(), str.end());
+  return str;
 }
 
 string GetAlNum(const string& s)
@@ -996,19 +1051,6 @@ string MD5(const string& input)
    return res;
 }
 
-bool movefile(const string& from, const string& to)
-{
-   if (!CopyFile(from.c_str(), to.c_str()))
-   {
-      LogFile.Log(1, "error moving %s to %s", from.c_str(), to.c_str());
-      return false;
-   }
-   bool success = (remove(from.c_str()) == 0);
-   if (!success)
-      LogFile.Log(1, "error removing %s", from.c_str());
-   return success;
-}
-
 void SetAux(cTimer* timer, string aux)
 {
    if (!timer) return;
@@ -1020,3 +1062,182 @@ void SetAux(cTimer* timer, string aux)
    timerText.replace(auxPos+1, timerText.size()-auxPos+1, aux);
    timer->Parse(timerText.c_str());
 }
+
+int msprintf(char **strp, const char *fmt, ...)
+{
+  va_list ap;
+  va_start (ap, fmt);
+  int res=vasprintf (strp, fmt, ap);
+  va_end (ap);
+  return res;
+}
+
+std::string GetCodeset()
+{
+#if APIVERSNUM >= 10503 
+   // Taken from VDR's vdr.c
+   char *CodeSet = NULL;
+   if (setlocale(LC_CTYPE, ""))
+     CodeSet = nl_langinfo(CODESET);
+   else {
+     char *LangEnv = getenv("LANG"); // last resort in case locale stuff isn't installed
+     if (LangEnv) {
+       CodeSet = strchr(LangEnv, '.');
+       if (CodeSet)
+	 CodeSet++; // skip the dot
+     }
+   }
+   if (CodeSet)
+     return std::string(CodeSet);
+   else
+     return "ISO-8859-15";
+#else
+   return "ISO-8859-15";
+#endif
+}
+
+/*  Read a line from a socket  */
+ssize_t Readline(int sockd, char *vptr, size_t maxlen) {
+    size_t n, rc;
+    char    c, *buffer;
+
+    buffer = vptr;
+
+    for ( n = 1; n < maxlen; n++ ) {
+	
+	if ( (rc = read(sockd, &c, 1)) == 1 ) {
+	    if ( c == '\n' )
+		break;
+	    *buffer++ = c;
+	}
+	else if ( rc == 0 ) {
+	    if ( n == 1 )
+		return 0;
+	    else
+		break;
+	}
+	else {
+	    if ( errno == EINTR )
+		continue;
+	    return -1;
+	}
+    }
+
+    *buffer = 0;
+    return n;
+}
+
+/*  Write a line to a socket  */
+ssize_t Writeline(int sockd, const char *vptr, size_t n) {
+    size_t      nleft;
+    size_t     nwritten;
+    const char *buffer;
+
+    buffer = vptr;
+    nleft  = n;
+
+    while ( nleft > 0 ) {
+	if ( (nwritten = write(sockd, buffer, nleft)) <= 0 ) {
+	    if ( errno == EINTR )
+		nwritten = 0;
+	    else
+		return -1;
+	}
+	nleft  -= nwritten;
+	buffer += nwritten;
+    }
+
+    return n;
+}
+
+long getAddrFromString(const char* hostnameOrIp, struct sockaddr_in* addr)
+{
+  unsigned long ip;
+  
+  struct hostent * he;
+  
+  if(hostnameOrIp==NULL || addr==NULL)
+    return -1;
+  
+  ip=inet_addr(hostnameOrIp);
+  
+  if(ip!=INADDR_NONE)
+    {
+      addr->sin_addr.s_addr=ip;		
+      return 0;
+    }
+  else
+    {
+      he=gethostbyname(hostnameOrIp);
+      if(he==NULL)
+	return -1;
+      else
+	memcpy(&(addr->sin_addr),he->h_addr_list[0],4);
+      return 0;
+    }
+}
+
+#if VDRVERSNUM >= 10712
+char *cCommand::result = NULL;
+
+cCommand::cCommand(void)
+{
+  title = command = NULL;
+  confirm = false;
+}
+
+cCommand::~cCommand()
+{
+  free(title);
+  free(command);
+}
+
+bool cCommand::Parse(const char *s)
+{
+  const char *p = strchr(s, ':');
+  if (p) {
+     int l = p - s;
+     if (l > 0) {
+        title = MALLOC(char, l + 1);
+        stripspace(strn0cpy(title, s, l + 1));
+        if (!isempty(title)) {
+           int l = strlen(title);
+           if (l > 1 && title[l - 1] == '?') {
+              confirm = true;
+              title[l - 1] = 0;
+              }
+           command = stripspace(strdup(skipspace(p + 1)));
+           return !isempty(command);
+           }
+        }
+     }
+  return false;
+}
+
+const char *cCommand::Execute(const char *Parameters)
+{
+  free(result);
+  result = NULL;
+  cString cmdbuf;
+  if (Parameters)
+     cmdbuf = cString::sprintf("%s %s", command, Parameters);
+  const char *cmd = *cmdbuf ? *cmdbuf : command;
+  dsyslog("executing command '%s'", cmd);
+  cPipe p;
+  if (p.Open(cmd, "r")) {
+     int l = 0;
+     int c;
+     while ((c = fgetc(p)) != EOF) {
+           if (l % 20 == 0)
+              result = (char *)realloc(result, l + 21);
+           result[l++] = char(c);
+           }
+     if (result)
+        result[l] = 0;
+     p.Close();
+     }
+  else
+     esyslog("ERROR: can't open pipe for command '%s'", cmd);
+  return result;
+}
+#endif
