@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -76,6 +76,8 @@ using std::string;
 
 #define ISRADIO(x) ((x)->Vpid()==0||(x)->Vpid()==1||(x)->Vpid()==0x1fff)
 
+#define UPDS_WITH_OSD (1<<1)
+#define UPDS_WITH_EPGSCAN (1<<2)
 
 #if APIVERSNUM >= 10503 
 // Icons used in VDRSymbols-Font
@@ -116,6 +118,8 @@ using std::string;
 #define Utf8BufSize(s) ((s) * 4)
 #endif
 
+#define CONTENT_DESCRIPTOR_MAX 255
+
 extern const char AllowedChars[];
 
 extern char* ConfigDir;
@@ -125,7 +129,7 @@ class cSearchExt;
 class cSearchExtCat;
 class cEvent;
 
-char* IndentMenuItem(const char*);
+char* IndentMenuItem(const char*, int indentions=1);
 void PatchFont(eDvbFont);
 bool MatchesSearchMode(const char* test, const char* values, int searchmode, const char* delim, int tolerance);
 char* GetExtEPGValue(const cEvent* e, cSearchExtCat* SearchExtCat);
@@ -137,7 +141,22 @@ string UpdateAuxValue(string aux, string section, string value);
 string UpdateAuxValue(string aux, string section, long num);
 void ToLower(char* szText);
 char *strreplacei(char *s, const char *s1, const char *s2);
-char *strreplaceall(char* source, char ch, const char* replacement);
+std::string strreplace(std::string& result, const std::string& replaceWhat, const std::string& replaceWithWhat);
+
+// replace s1 with s2 in s ignoring the case of s1
+inline char *strreplacei(char *s, const char *s1, const char s2)
+{
+   char *p = strcasestr(s, s1);
+   if (p) {
+      int offset = p - s;
+      int l  = strlen(s);
+      int l1 = strlen(s1);
+      memmove(s + offset + 1, s + offset + l1, l - offset - l1 + 1);
+      s[offset] =  s2;
+   }
+   return s;
+}
+
 void sleepMSec(long ms);
 void sleepSec(long s);
 bool SendViaSVDRP(cString SVDRPcmd);
@@ -146,13 +165,13 @@ bool InEditMode(const char* ItemText, const char* ItemName, const char* ItemValu
 cSearchExt* TriggeredFromSearchTimer(const cTimer* timer);
 int TriggeredFromSearchTimerID(const cTimer* timer);
 double FuzzyMatch(const char* s1, const char* s2, int maxLength);
-bool DescriptionMatches(const char* eDescr, const char* rDescr);
+bool DescriptionMatches(const char* eDescr, const char* rDescr, int matchLimit = 90);
 const cEvent* GetEvent(cTimer* timer);
 char* GetRawDescription(const char* descr);
 void PrepareTimerFile(const cEvent* event, cTimer* timer);
 int CompareEventTime(const void *p1, const void *p2);
 int CompareEventChannel(const void *p1, const void *p2);
-bool EventsMatch(const cEvent* event1, const cEvent* event2, bool compareTitle, bool compareSubtitle, bool compareSummary, unsigned long catvaluesAvoidRepeat);
+bool EventsMatch(const cEvent* event1, const cEvent* event2, bool compareTitle, int compareSubtitle, bool compareSummary, int compareDate, unsigned long catvaluesAvoidRepeat, int matchLimit=90);
 int ChannelNrFromEvent(const cEvent* pEvent);
 void DelTimer(int index);
 char* FixSeparators(char* buffer, char sep);
@@ -169,13 +188,17 @@ string MD5(const string& input);
 time_t GetDateTime(time_t day, int start);
 void SetAux(cTimer* timer, string aux);
 int msprintf(char **strp, const char *fmt, ...);
+std::string GetCodeset();
+ssize_t Readline(int sockd, char *vptr, size_t maxlen);
+ssize_t Writeline(int sockd, const char *vptr, size_t n);
+long getAddrFromString(const char* hostnameOrIp, struct sockaddr_in* addr);
 
 // --- cTimerObj --------------------------------------------------------
 class cTimerObj : public cListObject {
 public:
     cTimer* timer;
     cTimerObj(cTimer* Timer) : timer(Timer) {}
-    ~cTimerObj() { timer = NULL;  } // do not delete anything!
+    virtual ~cTimerObj() { timer = NULL;  } // do not delete anything!
 };
 
 // --- cTimerObjList --------------------------------------------------------
@@ -238,4 +261,24 @@ typedef std::basic_string<char,ignorecase_traits> icstring;
 // --- eTimerMod -------------------------------------------------------------
 enum eTimerMod { tmNoChange=0, tmStartStop=1, tmFile=2, tmAuxEventID=4 };
 
+#if VDRVERSNUM >= 10712
+// --- cCommands -------------------------------------------------------------------
+class cCommand : public cListObject {
+private:
+  char *title;
+  char *command;
+  bool confirm;
+  static char *result;
+public:
+  cCommand(void);
+  virtual ~cCommand();
+  bool Parse(const char *s);
+  const char *Title(void) { return title; }
+  bool Confirm(void) { return confirm; }
+  const char *Execute(const char *Parameters = NULL);
+  };
+
+class cCommands : public cConfig<cCommand> {};
+#endif
+
 #endif
