@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -48,7 +48,8 @@ class cLogFile: public cFile
 		char* buffer = NULL;
 		va_list Arg; 
 		va_start(Arg,text); 
-		vasprintf(&buffer, text, Arg); 
+		if (vasprintf(&buffer, text, Arg) < 0)
+		   esyslog("EPGSearch: vasprintf error");
 		va_end(Arg);
 		time_t now = time(NULL);
 
@@ -73,7 +74,8 @@ class cLogFile: public cFile
 	    char* buffer = NULL;
 	    va_list Arg; 
 	    va_start(Arg,text); 
-	    vasprintf(&buffer, text, Arg); 
+	    if (vasprintf(&buffer, text, Arg) < 0)
+	       esyslog("EPGSearch: vasprintf error");
 	    va_end(Arg);
 	    esyslog("EPGSearch: %s", buffer);
 	    Log(1, "%s", buffer);
@@ -84,7 +86,8 @@ class cLogFile: public cFile
 	    char* buffer = NULL;
 	    va_list Arg; 
 	    va_start(Arg,text); 
-	    vasprintf(&buffer, text, Arg); 
+	    if (vasprintf(&buffer, text, Arg) < 0)
+	       esyslog("EPGSearch: vasprintf error");
 	    va_end(Arg);
 	    isyslog("EPGSearch: %s", buffer);
 	    Log(1, "%s", buffer);
