@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,7 +30,14 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "log.h"
 #include "epgsearchtools.h"
 #include "uservars.h"
+#include "noannounce.h"
+#include "pending_notifications.h"
 
+#ifndef SENDMAIL
+#define SENDMAIL "/usr/sbin/sendmail"
+#endif
+
+extern bool isUTF8;
 using namespace std;
 
 string cMailNotifier::MailCmd = "sendEmail.pl";
@@ -67,7 +74,7 @@ const cEvent* cMailTimerNotification::GetEvent() const
     const cSchedules *schedules = cSchedules::Schedules(schedulesLock);
     if (!schedules) return NULL;
     const cSchedule *schedule = schedules->GetSchedule(channelID);
-    if (!schedule) return NULL;
+    if (!schedule) return NULL;    
     return schedule->GetEvent(eventID);
 }
 
@@ -105,6 +112,13 @@ cMailDelTimerNotification::cMailDelTimerNotification(cTimer* pTimer, const cEven
     formatted =  varExprTimer.Evaluate(pTimer);
 }
 
+cMailDelTimerNotification::cMailDelTimerNotification(const string& Formatted, tChannelID ChannelID, time_t Start)
+{
+    formatted = Formatted;
+    channelID = ChannelID;
+    start = Start;
+}
+
 bool cMailDelTimerNotification::operator< (const cMailDelTimerNotification &N) const
 {
     const cChannel* channel = Channels.GetByChannelID(channelID,true,true);
@@ -117,19 +131,38 @@ bool cMailDelTimerNotification::operator< (const cMailDelTimerNotification &N) c
 	return (start < N.start);
 }
 
+// ----------------------
+// cMailAnnounceEventNotification
+string cMailAnnounceEventNotification::Format(const string& templ) const
+{
+    const cEvent* pEvent = GetEvent();
+    if (!pEvent) return "";
+
+    string result = templ;
+    cVarExpr varExprEvent(result);
+    result =  varExprEvent.Evaluate(pEvent);
+
+    result = ReplaceAll(result, "%searchid%", NumToString(searchextID));
+    cSearchExt* search = SearchExts.GetSearchFromID(searchextID);
+    if (search)
+	result = ReplaceAll(result, "%search%", search->search);
+    return result;
+}
+
 // -------------
 // cMailNotifier
 cMailNotifier::cMailNotifier(string Subject, string Body)
 : subject(Subject), body(Body) 
 {
-    SendMail();
+  if (subject.size() > 0)
+    SendMail(true);
 }
 
 bool cMailNotifier::SendMailViaSendmail() 
 {
     char mailcmd[256];
     const char* mailargs = "%s -i -FVDR -oem  %s";
-    const char* mailproc = "/usr/lib/sendmail";
+    const char* mailproc = SENDMAIL;
     FILE* mail;
     
     string to = EPGSearchConfig.MailAddressTo; 
@@ -143,9 +176,9 @@ bool cMailNotifier::SendMailViaSendmail()
     fprintf(mail, "To: %s\n", to.c_str());
     fprintf(mail, "Subject: %s\n", subject.c_str());
     if (FindIgnoreCase(body, "<html>") >= 0)
-	fprintf(mail, "Content-Type: text/html; charset=ISO-8859-15\n");
+      fprintf(mail, "Content-Type: text/html; charset=%s\n", GetCodeset().c_str());
     else
-	fprintf(mail, "Content-Type: text/plain; charset=ISO-8859-15\n");
+      fprintf(mail, "Content-Type: text/plain; charset=%s\n", GetCodeset().c_str());
 
     fprintf(mail, "\n");
     
@@ -172,15 +205,16 @@ bool cMailNotifier::SendMailViaScript()
     string AuthUser = EPGSearchConfig.MailAuthUser;
     string AuthPass = EPGSearchConfig.MailAuthPass;
     string cmdArgs =  
-	string(" -f \"VDR <") + EPGSearchConfig.MailAddress + ">\"" +
-	" -t " + EPGSearchConfig.MailAddressTo +
-	" -s " + EPGSearchConfig.MailServer +
-	" -u \"" + subject + "\""+ 
-	(EPGSearchConfig.MailUseAuth?
-	 (AuthUser != "" ?(" -xu " + AuthUser):"") +
-	 (AuthPass != "" ?(" -xp " + AuthPass):"")
-	 :"") +
-        " -o message-file=" + filename;
+      string(" -f \"VDR <") + EPGSearchConfig.MailAddress + ">\"" +
+      " -t " + EPGSearchConfig.MailAddressTo +
+      " -s " + EPGSearchConfig.MailServer +
+      " -u \"" + subject + "\""+ 
+      (EPGSearchConfig.MailUseAuth?
+       (AuthUser != "" ?(" -xu " + AuthUser):"") +
+       (AuthPass != "" ?(" -xp " + AuthPass):"")
+       :"") +
+      " -o message-charset=" + GetCodeset() + 
+      " -o message-file=" + filename;
 
     bool success = ExecuteMailScript(cmdArgs);
 
@@ -190,17 +224,28 @@ bool cMailNotifier::SendMailViaScript()
     return success;
 }
 
-bool cMailNotifier::SendMail()
+bool cMailNotifier::SendMail(bool force)
 {
-    if (!EPGSearchConfig.mailViaScript)
+  time_t nextMailDelivery = EPGSearchConfig.lastMailOnSearchtimerAt + EPGSearchConfig.sendMailOnSearchtimerHours*60*60;
+  if (time(NULL) > nextMailDelivery || force)
+    {
+      if (!EPGSearchConfig.mailViaScript)
 	return SendMailViaSendmail();
-    else
+      else
 	return SendMailViaScript();
+    }
+  else
+    {
+      
+      LogFile.Log(2, "mail delivery delayed until %s", DAYDATETIME(nextMailDelivery));
+      return false;
+    }
 }
 
 bool cMailNotifier::ExecuteMailScript(string ScriptArgs)
 {
     string mailCmd = MailCmd;
+    LogFile.Log(3, "starting mail script: %s with parameters: %s", mailCmd.c_str(), ScriptArgs.c_str());
     if (mailCmd == "sendEmail.pl") // beautify output for standard script
 	ScriptArgs += " | cut -d\" \" -f 6-";
 
@@ -300,11 +345,39 @@ void cMailUpdateNotifier::AddRemoveTimerNotification(cTimer* t, const cEvent* e)
     delTimers.insert(N);
 }
 
+void cMailUpdateNotifier::AddRemoveTimerNotification(const string& Formatted, tChannelID ChannelID, time_t Start)
+{
+    cMailDelTimerNotification N(Formatted, ChannelID, Start);
+    delTimers.insert(N);
+}
+
+void cMailUpdateNotifier::AddAnnounceEventNotification(tEventID EventID, tChannelID ChannelID, int SearchExtID)
+{
+  cMailAnnounceEventNotification N(EventID, ChannelID, SearchExtID);
+  announceEvents.insert(N);
+}
+
 void cMailUpdateNotifier::SendUpdateNotifications()
 {
+    // insert pending notifications
+    cPendingNotification* p = PendingNotifications.First();
+    while (p) 
+    {
+      if (p->type == 0) 
+	AddNewTimerNotification(p->eventID, p->channelID);
+      else if (p->type == 1)
+	AddModTimerNotification(p->eventID, p->channelID, p->timerMod);
+      else if (p->type == 2)
+	AddRemoveTimerNotification(p->formatted, p->channelID, p->start);
+      else if (p->type == 3)
+	AddAnnounceEventNotification(p->eventID, p->channelID, p->searchID);
+      p = PendingNotifications.Next(p);
+    }	    
+
     if (newTimers.size() == 0 && 
 	modTimers.size() == 0 && 
-	delTimers.size() == 0)
+	delTimers.size() == 0 &&
+	announceEvents.size() == 0)
 	return;
 
     // extract single templates
@@ -316,6 +389,7 @@ void cMailUpdateNotifier::SendUpdateNotifications()
     string templSubject = GetTemplValue(mailTemplate, "subject");
     string templBody = GetTemplValue(mailTemplate, "mailbody");
     string templTimer = GetTemplValue(mailTemplate, "timer");
+    string templEvent = GetTemplValue(mailTemplate, "event");
 
     // create the timer list for new timers
     string newtimers;
@@ -346,35 +420,85 @@ void cMailUpdateNotifier::SendUpdateNotifications()
     std::set<cMailDelTimerNotification>::iterator itdt;
     for (itdt = delTimers.begin(); itdt != delTimers.end(); itdt++) 
     {
-	string message = (*itdt).formatted;
+      string message = (*itdt).Format("");
 	if (message != "") deltimers += message;
     }
 
+    // create the list of events to announce
+    string announceevents;
+    if (announceEvents.size() == 0)
+	announceevents = tr("No new events to announce.");
+    std::set<cMailAnnounceEventNotification>::iterator itae;
+    for (itae = announceEvents.begin(); itae != announceEvents.end(); itae++) 
+    {
+	string message = (*itae).Format(templEvent);
+	if (message != "") announceevents += message;
+    }
+
     // evaluate variables
     cVarExpr varExprSubj(templSubject);
     subject =  varExprSubj.Evaluate();
     subject = ReplaceAll(subject, "%update.countnewtimers%", NumToString((long)newTimers.size()));
     subject = ReplaceAll(subject, "%update.countmodtimers%", NumToString((long)modTimers.size()));
     subject = ReplaceAll(subject, "%update.countdeltimers%", NumToString((long)delTimers.size()));
+    subject = ReplaceAll(subject, "%update.countnewevents%", NumToString((long)announceEvents.size()));
 
     newtimers = ReplaceAll(newtimers, "%update.countnewtimers%", NumToString((long)newTimers.size()));
     modtimers = ReplaceAll(modtimers, "%update.countmodtimers%", NumToString((long)modTimers.size()));
     deltimers = ReplaceAll(deltimers, "%update.countdeltimers%", NumToString((long)delTimers.size()));
+    announceevents = ReplaceAll(announceevents, "%update.countnewevents%", NumToString((long)announceEvents.size()));
 
     cVarExpr varExprBody(templBody);
     body =  varExprBody.Evaluate();
     body = ReplaceAll(body, "%update.countnewtimers%", NumToString((long)newTimers.size()));
     body = ReplaceAll(body, "%update.countmodtimers%", NumToString((long)modTimers.size()));
     body = ReplaceAll(body, "%update.countdeltimers%", NumToString((long)delTimers.size()));
+    body = ReplaceAll(body, "%update.countnewevents%", NumToString((long)announceEvents.size()));
     body = ReplaceAll(body, "%update.newtimers%", newtimers);
     body = ReplaceAll(body, "%update.modtimers%", modtimers);
     body = ReplaceAll(body, "%update.deltimers%", deltimers);
+    body = ReplaceAll(body, "%update.newevents%", announceevents);
 
-    SendMail();
-
+    if (SendMail())
+    {
+      EPGSearchConfig.lastMailOnSearchtimerAt = time(NULL);
+      cPluginManager::GetPlugin("epgsearch")->SetupStore("MailNotificationSearchtimersLastAt",  
+							 EPGSearchConfig.lastMailOnSearchtimerAt);
+      // remove pending notifications
+      while((p = PendingNotifications.First()) != NULL)
+	PendingNotifications.Del(p);
+    }
+    else
+    {
+      // add current notifications to pending ones
+      for (itnt = newTimers.begin(); itnt != newTimers.end(); itnt++) 
+	PendingNotifications.Add(new cPendingNotification(0, itnt->eventID, itnt->channelID, -1));
+      for (itmt = modTimers.begin(); itmt != modTimers.end(); itmt++) 
+	PendingNotifications.Add(new cPendingNotification(1, itmt->eventID, itmt->channelID, -1, itmt->timerMod));
+      for (itdt = delTimers.begin(); itdt != delTimers.end(); itdt++) 
+	PendingNotifications.Add(new cPendingNotification(2, -1, itdt->channelID, itdt->start, -1, -1, itdt->formatted));
+      for (itae = announceEvents.begin(); itae != announceEvents.end(); itae++) 
+ 	PendingNotifications.Add(new cPendingNotification(3, itae->eventID, itae->channelID, -1, -1, itae->searchextID));     
+    } 
+    PendingNotifications.Save();
+    
     newTimers.clear();
     modTimers.clear();
     delTimers.clear();
+    // Add announced events to the "no announce" list
+    for (itae = announceEvents.begin(); itae != announceEvents.end(); itae++) 
+    {
+      cNoAnnounce* noAnnounce = new cNoAnnounce(itae->GetEvent());             
+      if (noAnnounce && noAnnounce->Valid())
+	NoAnnounces.Add(noAnnounce);
+    }
+    if (announceEvents.size() > 0)
+    {
+      NoAnnounces.ClearOutdated();
+      NoAnnounces.Save();
+    }
+
+    announceEvents.clear();
 }
 
 // ---------------------
