@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -43,6 +43,8 @@ int toggleKeys=0;
 int exitToMainMenu = 0;
 extern int gl_InfoConflict;
 
+int cMenuSearchMain::forceMenu = 0; // 1 = now, 2 = schedule, 3 = summary
+
 // --- cMenuSearchMain ---------------------------------------------------------
 cMenuSearchMain::cMenuSearchMain(void)
 :cOsdMenu("", GetTab(1), GetTab(2), GetTab(3), GetTab(4), GetTab(5))
@@ -57,7 +59,7 @@ cMenuSearchMain::cMenuSearchMain(void)
   schedules = cSchedules::Schedules(schedulesLock);
   if (channel) {
     cMenuWhatsOnSearch::SetCurrentChannel(channel->Number());
-    if (EPGSearchConfig.StartMenu == 0)
+    if (EPGSearchConfig.StartMenu == 0 || forceMenu != 0)
       PrepareSchedule(channel);
     SetHelpKeys();
     cMenuWhatsOnSearch::currentShowMode = showNow;
@@ -66,16 +68,36 @@ cMenuSearchMain::cMenuSearchMain(void)
     //isyslog("duration epgs sched:  %d", tnow.millitm - gl_time.millitm + ((tnow.millitm - gl_time.millitm<0)?1000:0));
 
   }
-  if (EPGSearchConfig.StartMenu == 1)
+  if ((EPGSearchConfig.StartMenu == 1 || forceMenu == 1) && forceMenu != 2)
     {
       InWhatsOnMenu = true;
       AddSubMenu(new cMenuWhatsOnSearch(schedules, cDevice::CurrentChannel()));
     }
+  if (forceMenu == 3)
+      ShowSummary();
 }
 
+#ifdef USE_GRAPHTFT
+void cMenuSearchMain::Display(void)
+{
+   cOsdMenu::Display();
+   
+   if (Count() > 0) 
+   {
+      int i = 0;
+
+      for (cOsdItem *item = First(); item; item = Next(item)) 
+         cStatus::MsgOsdEventItem(!item->Selectable() ? 0 : 
+                                  ((cMenuMyScheduleItem*)item)->event, 
+                                  item->Text(), i++, Count());
+   }
+}
+#endif /* GRAPHTFT */
+
 cMenuSearchMain::~cMenuSearchMain()
 {
     cMenuWhatsOnSearch::ScheduleChannel(); // makes sure any posted data is cleared
+    forceMenu = 0;
 }
 
 int cMenuSearchMain::GetTab(int Tab)
