@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -74,7 +74,8 @@ cMenuSearchActions::~cMenuSearchActions()
 eOSState cMenuSearchActions::Search(void)
 {
     cMenuTemplate* MenuTemplate = NULL;
-    if (search && search->menuTemplate > 0)
+    if (!search) return osContinue;
+    if (search->menuTemplate > 0)
 	MenuTemplate = cTemplFile::GetSearchTemplateByPos(search->menuTemplate);
     if (!MenuTemplate)
 	MenuTemplate = cTemplFile::GetTemplateByName("MenuSearchResults");
@@ -121,14 +122,14 @@ eOSState cMenuSearchActions::Execute()
             Skins.Message(mtInfo, tr("Search timers activated in setup."));			
          }
          if (Interface->Confirm(tr("Run search timer update?")))
-            updateForced = 2; // with message about completion
+            updateForced = 3; // with message about completion
          return osBack;
       }	
-      if (current == 3)
+      if (current == 3 && search)
          return AddSubMenu(new cMenuRecsDone(search));
-      if (current == 4)
+      if (current == 4 && search)
          return AddSubMenu(new cMenuTimersDone(search));
-      if (current == 5)
+      if (current == 5 && search)
       {
          if (!Interface->Confirm(tr("Copy this entry?")))
             return osBack;
@@ -142,7 +143,7 @@ eOSState cMenuSearchActions::Execute()
 	 SearchExts.Save();
 	 return AddSubMenu(new cMenuEditSearchExt(copy));
       }
-      if (current == 6)
+      if (current == 6 && search)
       {
          if (!Interface->Confirm(tr("Copy this entry to templates?")))
             return osBack;
@@ -167,7 +168,7 @@ eOSState cMenuSearchActions::Execute()
 	 return osBack;
      }
      if (current == 10)	 
-	 return AddSubMenu(new cMenuConflictCheck());
+       return AddSubMenu(new cMenuConflictCheck());
   }
   return osContinue;
 }
