@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,6 +23,7 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 
 #include <vector>
 #include <string>
+#include <map>
 
 #include "menu_searchedit.h"
 #include "changrp.h"
@@ -45,7 +46,7 @@ cChannelGroups ChannelGroups;
 cSearchExtCats SearchExtCats;
 
 // --- cMenuEditSearchExt --------------------------------------------------------
-cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New, bool Template)
+cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New, bool Template, bool FromEPG)
    :cOsdMenu(tr("Edit search"),32)
 {
    templateMode = Template;
@@ -72,12 +73,15 @@ cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New, bool Tem
    UseChannelSel[3] = strdup(tr("only FTA"));
 
    SearchTimerModes[0] = strdup(tr("Record"));
-   SearchTimerModes[1] = strdup(tr("Announce only"));
+   SearchTimerModes[1] = strdup(tr("Announce by OSD"));
    SearchTimerModes[2] = strdup(tr("Switch only"));
+   SearchTimerModes[3] = strdup(tr("Announce and switch"));
+   SearchTimerModes[4] = strdup(tr("Announce by mail"));
 
-   BlacklistModes[0] = strdup(trVDR("no"));
+   BlacklistModes[0] = strdup(tr("only globals"));
    BlacklistModes[1] = strdup(tr("Selection"));
    BlacklistModes[2] = strdup(tr("all"));
+   BlacklistModes[3] = strdup(trVDR("none"));
 
    DelModes[0] = strdup(trVDR("no"));
    DelModes[1] = strdup(tr("count recordings"));
@@ -87,6 +91,29 @@ cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New, bool Tem
    SearchActiveModes[1] = strdup(trVDR("yes"));
    SearchActiveModes[2] = strdup(tr("user-defined"));
 
+   CompareSubtitleModes[0] = strdup(trVDR("no"));
+   CompareSubtitleModes[1] = strdup(tr("if present"));
+
+   CompareDateModes[0] = strdup(trVDR("no"));
+   CompareDateModes[1] = strdup(tr("same day"));
+   CompareDateModes[2] = strdup(tr("same week"));
+   CompareDateModes[3] = strdup(tr("same month"));
+
+#if APIVERSNUM > 10710 
+   // collect content string IDs
+   std::set<std::string> contentStrings;
+   for(unsigned int i=0; i<CONTENT_DESCRIPTOR_MAX;i++)
+     {
+       const string contentDescr = cEvent::ContentToString(i);
+       if (!contentDescr.empty() && contentStrings.find(contentDescr) == contentStrings.end())
+	 {
+	   contentStrings.insert(contentDescr);
+	   contentStringIDs.push_back(i);
+	 }
+     }
+#endif
+   useContentDescriptors = false;
+
    if (!templateMode && New)
    {
       cSearchExt* SearchTempl = NULL; // copy the default settings, if we have a default template	
@@ -99,7 +126,7 @@ cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New, bool Tem
          SearchExtTempl = SearchTemplates.Next(SearchExtTempl);
       }    
       if (SearchTempl)
-         SearchExt->CopyFromTemplate(SearchTempl);
+	SearchExt->CopyFromTemplate(SearchTempl, FromEPG);
    }
 
    searchExt = SearchExt;
@@ -122,11 +149,11 @@ cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New, bool Tem
 
       channelMin = channelMax = cDevice::CurrentChannel();
       channelGroupNr = 0;  
-      if (data.useChannel==1)
-      {
+      if (data.channelMin)
          channelMin = data.channelMin->Number();
+      if (data.channelMax)
          channelMax = data.channelMax->Number();
-      }
+
       if (data.useChannel==2)
       {
          channelGroupNr = ChannelGroups.GetIndex(data.channelGroup);
@@ -143,6 +170,15 @@ cMenuEditSearchExt::cMenuEditSearchExt(cSearchExt *SearchExt, bool New, bool Tem
          }
       }
 
+      contentStringsFlags = NULL;
+#if APIVERSNUM > 10710 
+      // set the flags for the content descriptors
+      contentStringsFlags = (int*) malloc((CONTENT_DESCRIPTOR_MAX+1) * sizeof(int));
+      for(unsigned int i=0; i<=CONTENT_DESCRIPTOR_MAX;i++)
+	contentStringsFlags[i] = data.HasContent(i);
+      useContentDescriptors = (data.contentsFilter.size() > 0);
+#endif
+
       catarrayAvoidRepeats = NULL;
       catvaluesNumeric = NULL;
       if (SearchExtCats.Count() > 0)
@@ -215,6 +251,20 @@ void cMenuEditSearchExt::Set()
    Add(new cMenuEditBoolItem( tr("Use description"), &data.useDescription, trVDR("no"), trVDR("yes")));
    AddHelp(tr("Help$Set this to 'Yes' if you like to search in the summary of an event."));    
     
+#if APIVERSNUM > 10710 
+   Add(new cMenuEditBoolItem( tr("Use content descriptor"), &useContentDescriptors, trVDR("no"), trVDR("yes")));
+   AddHelp(tr("Help$Set this to 'Yes' if you want to search the contents by a descriptor."));    
+   if (useContentDescriptors)
+     {
+       vector<int>::const_iterator it;
+       for(unsigned int i=0; i< contentStringIDs.size(); i++)
+	 {
+	   int level = (contentStringIDs[i] % 0x10==0?1:2);
+	   Add(new cMenuEditBoolItem(IndentMenuItem(tr(cEvent::ContentToString(contentStringIDs[i])), level), &contentStringsFlags[contentStringIDs[i]], trVDR("no"), trVDR("yes")));
+	 }
+     }
+#endif
+
    // show Categories only if we have them
    if (SearchExtCats.Count() > 0)
      {
@@ -286,7 +336,7 @@ void cMenuEditSearchExt::Set()
       }	
       Add(new cMenuEditStraItem(IndentMenuItem(tr("Day of week")),     &data.DayOfWeek, 8, DaysOfWeek));
    }
-   Add(new cMenuEditStraItem(tr("Use blacklists"), &data.blacklistMode, 3, BlacklistModes));
+   Add(new cMenuEditStraItem(tr("Use blacklists"), &data.blacklistMode, 4, BlacklistModes));
 
    if (EPGSearchConfig.showFavoritesMenu)
       Add(new cMenuEditBoolItem( tr("Use in favorites menu"), &data.useInFavorites, trVDR("no"), trVDR("yes")));
@@ -300,12 +350,17 @@ void cMenuEditSearchExt::Set()
    Add(new cMenuEditStraItem( tr("Use as search timer"), &data.useAsSearchTimer, 3, SearchActiveModes));
    if (data.useAsSearchTimer)
    {
-      Add(new cMenuEditStraItem(IndentMenuItem(tr("Action")), &data.action, 3, SearchTimerModes));
+      Add(new cMenuEditStraItem(IndentMenuItem(tr("Action")), &data.action, 5, SearchTimerModes));
       if (data.action == searchTimerActionSwitchOnly)
       {
          Add(new cMenuEditIntItem(IndentMenuItem(tr("Switch ... minutes before start")), &data.switchMinsBefore, 0, 99));
          Add(new cMenuEditBoolItem(IndentMenuItem(tr("Unmute sound")), &data.unmuteSoundOnSwitch, trVDR("no"), trVDR("yes")));
       }
+      if (data.action ==  searchTimerActionAnnounceAndSwitch)
+      {
+         Add(new cMenuEditIntItem(IndentMenuItem(tr("Ask ... minutes before start")), &data.switchMinsBefore, 0, 99));
+         Add(new cMenuEditBoolItem(IndentMenuItem(tr("Unmute sound")), &data.unmuteSoundOnSwitch, trVDR("no"), trVDR("yes")));
+      }
       if (data.action == searchTimerActionRecord)
       {
          Add(new cMenuEditBoolItem( tr("  Series recording"), &data.useEpisode, trVDR("no"), trVDR("yes")));
@@ -317,29 +372,32 @@ void cMenuEditSearchExt::Set()
          Add(new cMenuEditBoolItem(IndentMenuItem(tr("Avoid repeats")), &data.avoidRepeats, trVDR("no"), trVDR("yes")));
          if (data.avoidRepeats)
          {
-            Add(new cMenuEditIntItem(IndentMenuItem(IndentMenuItem(tr("Allowed repeats"))), &data.allowedRepeats, 0, 99));
-            if (data.allowedRepeats > 0)
-               Add(new cMenuEditIntItem(IndentMenuItem(IndentMenuItem(tr("Only repeats within ... days"))), &data.repeatsWithinDays, 0, 999));
-            Add(new cMenuEditBoolItem(IndentMenuItem(IndentMenuItem(tr("Compare title"))), &data.compareTitle, trVDR("no"), trVDR("yes")));
-            Add(new cMenuEditBoolItem(IndentMenuItem(IndentMenuItem(tr("Compare subtitle"))), &data.compareSubtitle, trVDR("no"), trVDR("yes")));
-            Add(new cMenuEditBoolItem(IndentMenuItem(IndentMenuItem(tr("Compare summary"))), &data.compareSummary, trVDR("no"), trVDR("yes")));
-            // show 'Compare categories' only if we have them
-            if (SearchExtCats.Count() > 0)
-            {
+	   Add(new cMenuEditIntItem(IndentMenuItem(tr("Allowed repeats"),2), &data.allowedRepeats, 0, 99));
+	   if (data.allowedRepeats > 0)
+	     Add(new cMenuEditIntItem(IndentMenuItem(tr("Only repeats within ... days"),2), &data.repeatsWithinDays, 0, 999));
+	   Add(new cMenuEditBoolItem(IndentMenuItem(tr("Compare title"),2), &data.compareTitle, trVDR("no"), trVDR("yes")));
+	   Add(new cMenuEditStraItem(IndentMenuItem(tr("Compare subtitle"),2), &data.compareSubtitle, 2, CompareSubtitleModes));
+	   Add(new cMenuEditBoolItem(IndentMenuItem(tr("Compare summary"),2), &data.compareSummary, trVDR("no"), trVDR("yes")));
+	   if (data.compareSummary)
+	     Add(new cMenuEditIntItem(IndentMenuItem(tr("Min. match in %"),3), &data.compareSummaryMatchInPercent, 1, 100));
+	   Add(new cMenuEditStraItem(IndentMenuItem(tr("Compare date"),2), &data.compareDate, 4, CompareDateModes));
+	   // show 'Compare categories' only if we have them
+	   if (SearchExtCats.Count() > 0)
+	     {
                cSearchExtCat *SearchExtCat = SearchExtCats.First();
                int iUsed = 0;
                int index = 0;
                while (SearchExtCat) 
-               {
-                  if (catarrayAvoidRepeats[index])
+		 {
+		   if (catarrayAvoidRepeats[index])
                      iUsed++;
-                  SearchExtCat = SearchExtCats.Next(SearchExtCat);
-                  index++;
-               }
-		    
+		   SearchExtCat = SearchExtCats.Next(SearchExtCat);
+		   index++;
+		 }
+	       
                cString itemtext = cString::sprintf("%s (%d/%d)", tr("Compare categories"), iUsed, SearchExtCats.Count());
                Add(new cOsdItem(IndentMenuItem(IndentMenuItem(itemtext))));
-            }
+	     }
          }
 	    
          Add(new cMenuEditIntItem(IndentMenuItem(trVDR("Priority")), &data.Priority, 0, MAXPRIORITY));
@@ -372,6 +430,8 @@ cMenuEditSearchExt::~cMenuEditSearchExt()
       free(catarrayAvoidRepeats);
    if (catvaluesNumeric)
      free(catvaluesNumeric);
+   if (contentStringsFlags)
+     free(contentStringsFlags);
 
    int i;
    for(i=0; i<=5; i++)
@@ -382,12 +442,16 @@ cMenuEditSearchExt::~cMenuEditSearchExt()
       free(UseChannelSel[i]);
    for(i=0; i<=2; i++)
       free(SearchTimerModes[i]);
-   for(i=0; i<=2; i++)
+   for(i=0; i<=3; i++)
       free(BlacklistModes[i]);
    for(i=0; i<=2; i++)
       free(DelModes[i]);
    for(i=0; i<=2; i++)
       free(SearchActiveModes[i]);
+   for(i=0; i<=1; i++)
+      free(CompareSubtitleModes[i]);
+   for(i=0; i<=3; i++)
+      free(CompareDateModes[i]);
 }
 
 eOSState cMenuEditSearchExt::Help()
@@ -417,11 +481,13 @@ eOSState cMenuEditSearchExt::ProcessKey(eKeys Key)
    int iTemp_useDayOfWeek = data.useDayOfWeek;
    int iTemp_useAsSearchTimer = data.useAsSearchTimer;
    int iTemp_useExtEPGInfo = data.useExtEPGInfo;
+   int iTemp_useContentDescriptor = useContentDescriptors;
    int iTemp_avoidRepeats = data.avoidRepeats;
    int iTemp_allowedRepeats = data.allowedRepeats;
    int iTemp_delAfterDays = data.delAfterDays;
    int iTemp_action = data.action;
    int iTemp_delMode = data.delMode;
+   int iTemp_compareSummary = data.compareSummary;
     
    eOSState state = cOsdMenu::ProcessKey(Key);
     
@@ -432,11 +498,13 @@ eOSState cMenuEditSearchExt::ProcessKey(eKeys Key)
        iTemp_useDayOfWeek != data.useDayOfWeek ||
        iTemp_useAsSearchTimer != data.useAsSearchTimer ||
        iTemp_useExtEPGInfo != data.useExtEPGInfo ||
+       iTemp_useContentDescriptor != useContentDescriptors ||
        iTemp_avoidRepeats != data.avoidRepeats ||
        iTemp_allowedRepeats != data.allowedRepeats ||
        iTemp_delAfterDays != data.delAfterDays ||
        iTemp_action != data.action || 
-       iTemp_delMode != data.delMode)
+       iTemp_delMode != data.delMode || 
+       iTemp_compareSummary != data.compareSummary)
    {
       Set();
       Display();
@@ -632,6 +700,8 @@ eOSState cMenuEditSearchExt::ProcessKey(eKeys Key)
 		 index++;
                }
 	       
+	       searchExt->SetContentFilter(useContentDescriptors?contentStringsFlags:NULL);
+
                if (data.blacklistMode == blacklistsSelection)
                {
                   searchExt->blacklists.Clear();
