@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -39,6 +39,7 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "menu_deftimercheckmethod.h"
 #include "afuzzy.h"
 #include "timerstatus.h"
+#include "switchtimer.h"
 
 const char* ButtonBlue[3] = {NULL, NULL, NULL};
 extern int gl_InfoConflict;
@@ -46,7 +47,11 @@ extern bool isUTF8;
 
 static int CompareRecording(const void *p1, const void *p2)
 {
+#if APIVERSNUM < 10721 
    return (int)((*(cRecording **)p1)->start - (*(cRecording **)p2)->start);
+#else
+   return (int)((*(cRecording **)p1)->Start() - (*(cRecording **)p2)->Start());
+#endif
 }
 
 // --- cMenuSearchResultsItem -------------------------------------------------------
@@ -61,6 +66,7 @@ cMenuSearchResultsItem::cMenuSearchResultsItem(const cEvent *EventInfo, bool Epi
    previewTimer = PreviewTimer;
    menuTemplate = MenuTemplate?MenuTemplate:cTemplFile::GetTemplateByName("MenuSearchResults");
    search = Search;
+   inSwitchList = false;
    Update(true);
 }
 
@@ -72,12 +78,14 @@ bool cMenuSearchResultsItem::Update(bool Force)
    bool result = false;
 
    int OldTimerMatch = timerMatch;
+   bool OldInSwitchList = inSwitchList;
    bool hasMatch = false; 
    cTimer* timer = NULL;
    if (event) timer = Timers.GetMatch(event, &timerMatch);
+   if (event) inSwitchList = (SwitchTimers.InSwitchList(event)!=NULL);
    if (timer) hasMatch = true;
     
-   if (Force || timerMatch != OldTimerMatch) 
+   if (Force || timerMatch != OldTimerMatch || inSwitchList != OldInSwitchList)  
    {
      char t[Utf8BufSize(2)]="",v[Utf8BufSize(2)]="",r[Utf8BufSize(2)]="";
      char szStatus[Utf8BufSize(4)] = "";
@@ -85,9 +93,13 @@ bool cMenuSearchResultsItem::Update(bool Force)
       {
 	if (!isUTF8)
 	  {
-	    sprintf(t, "%c", event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?ICON_REC:ICON_CLOCK) : ICON_CLOCK_HALF : ' ');
-	    sprintf(v, "%c", event && event->Vps() && (event->Vps() - event->StartTime()) ? ICON_VPS : ' ');
-	    sprintf(r, "%c", event && event->IsRunning() ? ICON_RUNNING : ' ');
+	    t[0] = event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?ICON_REC:ICON_CLOCK) : ICON_CLOCK_HALF : ' ';
+	    t[1] = '\0';
+	    v[0] = event && event->Vps() && (event->Vps() - event->StartTime()) ? ICON_VPS : ' ';
+	    v[1] = '\0';
+	    r[0] = event && event->IsRunning() ? ICON_RUNNING : ' ';
+	    r[1] = '\0';
+
 	  }
 	else
 	  {
@@ -101,19 +113,27 @@ bool cMenuSearchResultsItem::Update(bool Force)
       }
       else
       {
-         sprintf(t, "%c", event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?'R':'T') : 't' : ' ');
-         sprintf(v, "%c", event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ');
-         sprintf(r, "%c", event && event->IsRunning() ? '*' : ' ');
+         t[0] = event && hasMatch ? (timerMatch == tmFull) ? ((timer && timer->Recording())?'R':'T') : 't' : ' ';
+         t[1] = '\0';
+         v[0] = event && event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
+         v[1] = '\0';
+         r[0] = event && event->IsRunning() ? '*' : ' ';
+         r[1] = '\0';
+      }
+      if (event && inSwitchList)
+      {
+         cSwitchTimer* s = SwitchTimers.InSwitchList(event);
+	 t[0] = (s && s->mode==1)?'s':'S';
       }
       if (t[0] != 'T' && previewTimer)
          t[0] = 'P';
 	
-      sprintf(szStatus, "%s%s%s", t,v,r);
+      strcpy(szStatus, t);
+      strcat(szStatus, v);
+      strcat(szStatus, r);
 	
       char* buffer = strdup(menuTemplate->MenuTemplate());
-      char* tmp = strreplaceall(buffer, '|', "\t");
-      free(buffer);
-      buffer = tmp;
+      strreplace(buffer, '|', '\t');
 
       if (!strcasestr(buffer, "%subtitle%") && cTemplFile::GetTemplateByName("MenuFavorites") != menuTemplate) 
 	// make sure, there is a subtitle
@@ -123,7 +143,7 @@ bool cMenuSearchResultsItem::Update(bool Force)
 	    
       // parse the epxression and evaluate it
       cVarExpr varExpr(buffer);
-      tmp = strdup(varExpr.Evaluate(event).c_str());
+      char* tmp = strdup(varExpr.Evaluate(event).c_str());
       free(buffer);
       buffer = tmp;
 
@@ -243,7 +263,7 @@ eOSState cMenuSearchResults::Record(void)
 
 #ifdef USE_PINPLUGIN
          aux = "";
-         aux = UpdateAuxValue(aux, "protected", timer->FskProtection() ? "yes" : "no");
+	 aux = UpdateAuxValue(aux, "protected", timer->FskProtection() ? "yes" : "no");
          fullaux = UpdateAuxValue(fullaux, "pin-plugin", aux);
 #endif
 
@@ -674,6 +694,7 @@ cMenuSearchResultsForQuery::cMenuSearchResultsForQuery(const char *query, bool I
       searchExt->useTitle = 1;
       searchExt->useSubtitle = 0;
       searchExt->useDescription = 0;
+      searchExt->blacklistMode = blacklistsNone;
       BuildList();
    }
 }
@@ -734,7 +755,7 @@ bool cMenuSearchResultsForRecs::BuildList()
      string s2 = searchExt->search;
      if (s1.empty() || s2.empty()) continue;
 
-     // tolerance for fuzzy searching: 90% of the shorter text lenght, but atleast 1
+     // tolerance for fuzzy searching: 90% of the shorter text length, but at least 1
      int tolerance = std::max(1, (int)std::min(s1.size(), s2.size()) / 10); 
 
      bool match = FindIgnoreCase(s1, s2) >= 0 ||
@@ -742,7 +763,7 @@ bool cMenuSearchResultsForRecs::BuildList()
 
      if (!match)
        {
-	 AFUZZY af = { NULL, NULL, NULL, NULL, NULL, NULL };
+	 AFUZZY af = { NULL, NULL, NULL, NULL, NULL, NULL, { 0 }, { 0 }, 0, 0, 0, 0, 0, 0 };
 	 if (s1.size() > 32) s1 = s1.substr(0, 32);
 	 afuzzy_init(s1.c_str(), tolerance, 0, &af);
 	 /* Checking substring */
@@ -752,7 +773,7 @@ bool cMenuSearchResultsForRecs::BuildList()
        }
      if (!match)
        {
-	 AFUZZY af = { NULL, NULL, NULL, NULL, NULL, NULL };
+	 AFUZZY af = { NULL, NULL, NULL, NULL, NULL, NULL, { 0 }, { 0 }, 0, 0, 0, 0, 0, 0 };
 	 if (s2.size() > 32) s2 = s2.substr(0, 32);
 	 afuzzy_init(s2.c_str(), tolerance, 0, &af);
 	 /* Checking substring */
