@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 // --- cMenuEditSwitchTimer --------------------------------------------------------
 class cMenuEditSwitchTimer : public cOsdMenu {
 private:
+  char *SwitchModes[3];
   cSwitchTimer *switchTimer;
   cSwitchTimer data;
   bool addIfConfirmed;
@@ -39,12 +40,16 @@ public:
 cMenuEditSwitchTimer::cMenuEditSwitchTimer(cSwitchTimer* SwitchTimer, bool New)
 :cOsdMenu(tr("Edit entry"),30)
 {
-    switchTimer = SwitchTimer;
-    addIfConfirmed = New;
-    if (switchTimer) 
+  SwitchModes[0] = strdup(tr("Switch"));
+  SwitchModes[1] = strdup(tr("Announce only"));
+  SwitchModes[2] = strdup(tr("Announce and switch"));
+
+  switchTimer = SwitchTimer;
+  addIfConfirmed = New;
+  if (switchTimer) 
     {
-	data = *switchTimer;
-	Set();
+      data = *switchTimer;
+      Set();
     }
 }
 
@@ -53,13 +58,19 @@ void cMenuEditSwitchTimer::Set()
     int current = Current();
     Clear();
 
-    Add(new cMenuEditIntItem(tr("Switch ... minutes before start"), &data.switchMinsBefore, 0, 99));
+    Add(new cMenuEditStraItem(tr("Action"), &data.mode, 3, SwitchModes));
+    if (data.mode == 0) // always switch
+      Add(new cMenuEditIntItem(tr("Switch ... minutes before start"), &data.switchMinsBefore, 0, 99));
+    if (data.mode == 1) // only announce
+      Add(new cMenuEditIntItem(tr("Announce ... minutes before start"), &data.switchMinsBefore, 0, 99));
+    if (data.mode == 2) // ask for switching
+      Add(new cMenuEditIntItem(tr("Ask ... minutes before start"), &data.switchMinsBefore, 0, 99));
+
     cString info = cString::sprintf("%s:\t%s", tr("action at"), 
-				    TIMESTRING(data.event->StartTime() - 60 * data.switchMinsBefore));
+				    TIMESTRING(data.startTime - 60 * data.switchMinsBefore));
     cOsdItem* pInfoItem = new cOsdItem(info);
     pInfoItem->SetSelectable(false);
     Add(pInfoItem);
-    Add(new cMenuEditBoolItem(tr("Announce only"), &data.announceOnly, trVDR("no"), trVDR("yes")));
     Add(new cMenuEditBoolItem(tr("Unmute sound"), &data.unmute, trVDR("no"), trVDR("yes")));    
     SetCurrent(Get(current));
 }
@@ -67,12 +78,14 @@ void cMenuEditSwitchTimer::Set()
 eOSState cMenuEditSwitchTimer::ProcessKey(eKeys Key)
 {
     int iOldMinsBefore = data.switchMinsBefore;
+    int iOldMode = data.mode;
     eOSState state = cOsdMenu::ProcessKey(Key);
 
-    if (iOldMinsBefore != data.switchMinsBefore)
+    if (iOldMinsBefore != data.switchMinsBefore ||
+	iOldMode != data.mode)
     {
 	time_t now = time(NULL);
-	if (data.event->StartTime() - 60 * data.switchMinsBefore < now)
+	if (data.startTime - 60 * data.switchMinsBefore < now)
 	    data.switchMinsBefore = iOldMinsBefore;
 	Set();
 	Display();
@@ -100,19 +113,19 @@ eOSState cMenuEditSwitchTimer::ProcessKey(eKeys Key)
     return state;
 }
 
-cMenuSwitchTimerItem::cMenuSwitchTimerItem(cSwitchTimer* SwitchTimer)
+cMenuSwitchTimerItem::cMenuSwitchTimerItem(cSwitchTimer* SwitchTimer, const cEvent* Event)
 {
   switchTimer = SwitchTimer;
+  event = Event;
   Set();
 }
 
-void cMenuSwitchTimerItem::Set(void)
+void cMenuSwitchTimerItem::Set()
 {
-    if (!SwitchTimers.Exists(switchTimer) || !switchTimer || !switchTimer->event)
+    if (!SwitchTimers.Exists(switchTimer) || !switchTimer || !event)
 	return;
 
-    const cEvent* event = switchTimer->event;
-    time_t startTime = event->StartTime();
+    time_t startTime = switchTimer->startTime;
     char *buffer = NULL;
 
     char datebuf[32];
@@ -120,16 +133,16 @@ void cMenuSwitchTimerItem::Set(void)
     tm *tm = localtime_r(&startTime, &tm_r);
     strftime(datebuf, sizeof(datebuf), "%d.%m", tm);
 
-    cChannel* channel = Channels.GetByChannelID(event->ChannelID(),true,true);
+    cChannel* channel = Channels.GetByChannelID(switchTimer->channelID,true,true);
 
-    msprintf(&buffer, "%s\t%d\t%s\t%s\t%d\'\t%s~%s", switchTimer->announceOnly?"":">", channel?channel->Number():-1, datebuf, TIMESTRING(startTime), switchTimer->switchMinsBefore, event->Title()?event->Title():"", event->ShortText()?event->ShortText():"");
+    msprintf(&buffer, "%s\t%d\t%s\t%s\t%d\'\t%s~%s", switchTimer->mode==1?"":">", channel?channel->Number():-1, datebuf, TIMESTRING(startTime), switchTimer->switchMinsBefore, event->Title()?event->Title():"", event->ShortText()?event->ShortText():"");
     SetText(buffer, false);
 }
 
 int cMenuSwitchTimerItem::Compare(const cListObject &ListObject) const
 {
     cMenuSwitchTimerItem *p = (cMenuSwitchTimerItem *)&ListObject;
-    if (switchTimer->event->StartTime() > p->switchTimer->event->StartTime()) 
+    if (switchTimer->startTime > p->switchTimer->startTime) 
 	return 1; 
     else 
 	return -1;
@@ -148,9 +161,11 @@ void cMenuSwitchTimers::Set()
     Clear();
     cMutexLock SwitchTimersLock(&SwitchTimers);
     cSwitchTimer* switchTimer = SwitchTimers.First();
-    while (switchTimer) {
-	if (switchTimer->event)
-	    Add(new cMenuSwitchTimerItem(switchTimer));
+    while (switchTimer) 
+    {
+      const cEvent* event = switchTimer->Event();
+      if (event)
+	Add(new cMenuSwitchTimerItem(switchTimer, event));
 	switchTimer = SwitchTimers.Next(switchTimer);
     }
     Display();
@@ -200,8 +215,13 @@ eOSState cMenuSwitchTimers::Summary(void)
     if (HasSubMenu() || Count() == 0)
 	return osContinue;
     cSwitchTimer *curSwitchTimer = CurrentSwitchTimer();
-    if (curSwitchTimer && !isempty(curSwitchTimer->event->Description()))
-	return AddSubMenu(new cMenuText(tr("Summary"), curSwitchTimer->event->Description()));
+    
+    if (curSwitchTimer)
+    {
+      const cEvent* event = curSwitchTimer->Event();
+      if (event && !isempty(event->Description()))
+	return AddSubMenu(new cMenuText(tr("Summary"), event->Description()));
+    }
     return osContinue;
 }
 
@@ -231,7 +251,7 @@ eOSState cMenuSwitchTimers::ProcessKey(eKeys Key)
 	    if (CurrentSwitchTimer())
 	    {
 		cSwitchTimer* switchTimer = CurrentSwitchTimer();
-	        switchTimer->announceOnly = 1 - switchTimer->announceOnly;
+	        switchTimer->mode = switchTimer->mode==1?2:1;
 		cMutexLock SwitchTimersLock(&SwitchTimers);
 		SwitchTimers.Save();
 		RefreshCurrent();
