@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -102,10 +102,11 @@ bool cMenuMyScheduleItem::Update(bool Force)
 	      if (mode == showNow)
 		{
 		  int dur = event->Duration();
-		  frac = ((now - startTime) * 8 + (dur >> 1)) / dur;
+		  if (dur != 0)
+		    frac = ((now - startTime) * 8 + (dur >> 1)) / dur;
 		}
 	      if (mode == showNext)
-		frac = (  ( 30*60 - min((long int)30*60, startTime - now) ) * 8 + 15*60  ) / (30*60);
+		frac = (  ( 30*60 - min((time_t)30*60, startTime - now) ) * 8 + 15*60  ) / (30*60);
 	      
 	      frac = min(8,max(0, frac));
 	      
@@ -121,7 +122,7 @@ bool cMenuMyScheduleItem::Update(bool Force)
 		  memset(szProgressPart + 1, ICON_BAR_EMPTY, 6);
 		  szProgressPart[7] = ICON_BAR_CLOSE;
 		  szProgressPart[8] = 0;
-		  memset(szProgressPart, ICON_BAR_FULL, frac);
+		  memset(szProgressPart, ICON_BAR_FULL, frac?frac:sizeof(szProgressPart));
 		}
 	      else
 		{
@@ -179,7 +180,7 @@ bool cMenuMyScheduleItem::Update(bool Force)
       if (event && inSwitchList)
       {
          cSwitchTimer* s = SwitchTimers.InSwitchList(event);
-	 t[0] = (s && s->announceOnly)?'s':'S';
+	 t[0] = (s && s->mode==1)?'s':'S';
       }
       if (EPGSearchConfig.WarEagle && isUTF8)
 	{
@@ -200,7 +201,7 @@ bool cMenuMyScheduleItem::Update(bool Force)
       strreplace(buffer, '|', '\t');
 
       char* title = NULL; 
-      msprintf(&title, "%s", event?event->Title():tr(">>> no info! <<<"));
+      title = strdup(event?event->Title():tr(">>> no info! <<<"));
       title = strreplacei(title, ":", "%colon%"); // asume a title has the form "a?b:c", 
       // we need to replace the colon to avoid misinterpretation the expression as a condition
       buffer = strreplacei(buffer, "%title%", title);
@@ -209,7 +210,7 @@ bool cMenuMyScheduleItem::Update(bool Force)
       if (channel)
       {
          char szChannelNr[6] = "";
-	 sprintf(szChannelNr, "%d", channel->Number());
+	 snprintf(szChannelNr, 6, "%d", channel->Number());
          buffer = strreplacei(buffer, "%chnr%", szChannelNr);
          buffer = strreplacei(buffer, "%chsh%", channel->ShortName(true));
          buffer = strreplacei(buffer, "%chlng%", channel->Name());
@@ -228,8 +229,6 @@ bool cMenuMyScheduleItem::Update(bool Force)
       buffer = strreplacei(buffer, "$v_status$", v);
       buffer = strreplacei(buffer, "$r_status$", r);
 
-
-
       buffer = FixSeparators(buffer, '~');
       buffer = FixSeparators(buffer, ':');
       buffer = FixSeparators(buffer, '-');
@@ -296,7 +295,22 @@ const char* cMenuWhatsOnSearch::MenuKind()
    if (currentShowMode > showNext)  return "MenuEpgsWhatsOnElse";
    else return "MenuWhatsOnElse";
 }
-#endif
+
+void cMenuWhatsOnSearch::Display(void)
+{
+   cOsdMenu::Display();
+   
+   if (Count() > 0) 
+   {
+      int i = 0;
+
+      for (cOsdItem *item = First(); item; item = Next(item)) 
+         cStatus::MsgOsdEventItem(!item->Selectable() ? 0 : 
+                                  ((cMenuMyScheduleItem*)item)->event, 
+                                  item->Text(), i++, Count());
+   }
+}
+#endif /* GRAPHTFT */
 
 int cMenuWhatsOnSearch::GetTab(int Tab)
 {
@@ -318,6 +332,7 @@ void cMenuWhatsOnSearch::LoadSchedules()
    //   time_t SeekTime;
    cString szTitle;
    cShowMode* mode = GetShowMode(currentShowMode);
+   if (!mode) return;
 
    if (shiftTime != 0)
    {
@@ -343,17 +358,14 @@ void cMenuWhatsOnSearch::LoadSchedules()
    }
    else
    {
-      if (mode)
-      {
-         seekTime = GetTimeT(mode->GetTime());
-         if (seekTime < time(NULL) && currentShowMode != showNow && currentShowMode != showNext)
-         {
-            seekTime += HOURS2SECS(24);	
-	    szTitle = cString::sprintf("%s - %s (%s)", tr("Overview"), mode->GetDescription(), *WeekDayName(seekTime));    
-         }
-         else
-	   szTitle = cString::sprintf("%s - %s", tr("Overview"), mode->GetDescription());    
-      }
+      seekTime = GetTimeT(mode->GetTime());
+      if (seekTime < time(NULL) && currentShowMode != showNow && currentShowMode != showNext)
+	{
+	  seekTime += HOURS2SECS(24);	
+	  szTitle = cString::sprintf("%s - %s (%s)", tr("Overview"), mode->GetDescription(), *WeekDayName(seekTime));    
+	}
+      else
+	szTitle = cString::sprintf("%s - %s", tr("Overview"), mode->GetDescription());    
    }
    SetTitle(szTitle);
 
@@ -444,6 +456,8 @@ showMode cMenuWhatsOnSearch::GetNextMode()
       else
          nextShowMode = ModeNext->GetMode();
    }
+   else // no mode found? fall back to 'now'
+     nextShowMode = showNow;
    return nextShowMode;
 }
 
@@ -566,7 +580,7 @@ eOSState cMenuWhatsOnSearch::Record(void)
          }
 #ifdef USE_PINPLUGIN
          aux = "";
-         aux = UpdateAuxValue(aux, "protected", timer->FskProtection() ? "yes" : "no");
+	 aux = UpdateAuxValue(aux, "protected", timer->FskProtection() ? "yes" : "no");
          fullaux = UpdateAuxValue(fullaux, "pin-plugin", aux);
 #endif
          SetAux(timer, fullaux);
@@ -643,7 +657,7 @@ void cMenuWhatsOnSearch::SetHelpKeys(bool Force)
     { 
       showMode nextShowMode = GetNextMode();    
       cShowMode* mode = GetShowMode(nextShowMode);
-      char* szButtonGreen = NULL;
+      const char* szButtonGreen = NULL;
       if (mode) 
 	szButtonGreen = mode->GetDescription();
       if (toggleKeys==0)
