@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -42,11 +42,14 @@ cNoAnnounce::cNoAnnounce(const cEvent* e, time_t NextAnnounce)
    title = shortText = "";
    startTime = 0;
    buffer = NULL;
-   if (e->Title()) title = e->Title();
-   if (e->ShortText()) shortText = e->ShortText();
-   channelID = e->ChannelID();
-   startTime = e->StartTime();
-   nextAnnounce = NextAnnounce;
+   if (e)
+   { 
+     if (e->Title()) title = e->Title();
+     if (e->ShortText()) shortText = e->ShortText();
+     channelID = e->ChannelID();
+     startTime = e->StartTime();
+     nextAnnounce = NextAnnounce;
+   }
 }
 
 cNoAnnounce::~cNoAnnounce(void)
@@ -103,6 +106,8 @@ bool cNoAnnounce::Parse(const char *s)
 			break;
  		    case 5: nextAnnounce = atol(value);
 			break;
+		    default:
+		      break;
 		} //switch
 	    }
 	    parameter++;
