@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -68,8 +68,8 @@ class cRecsDone : public cList<cRecDone>, public cMutex {
 	    cList<cRecDone>::Clear();
 	}
     cRecsDone(void) {}
-    int GetCountRecordings(const cEvent* event, cSearchExt* search, cRecDone** first = NULL);
-    int GetCountRecordings(const cEvent*, bool compareTitle, bool compareSubtitle, bool compareSummary, unsigned long, cRecDone** first = NULL);
+    int GetCountRecordings(const cEvent* event, cSearchExt* search, cRecDone** first = NULL, int matchLimit = 90);
+    int GetCountRecordings(const cEvent*, bool compareTitle, int compareSubtitle, bool compareSummary, int compareDate, unsigned long, cRecDone** first = NULL, int matchLimit = 90);
     int GetTotalCountRecordings(cSearchExt* search, cRecDone** first);
     void RemoveSearchID(int ID);
     bool Load(const char *FileName = NULL);
