@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -26,6 +26,7 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "recdone.h"
 #include "conflictcheck_thread.h"
 #include "epgsearchcfg.h"
+#include <math.h>
 
 #define ALLOWED_BREAK_INSECS 2
 
@@ -39,7 +40,7 @@ cRecStatusMonitor::cRecStatusMonitor()
 {
 }
 
-void cRecStatusMonitor::Recording(const cDevice *Device, const char *Name, const char*, bool On) 
+void cRecStatusMonitor::Recording(const cDevice *Device, const char *Name, const char* Filename, bool On) 
 {
    time_t now = time(NULL);
    // insert new timers currently recording in TimersRecording
@@ -77,7 +78,8 @@ void cRecStatusMonitor::Recording(const cDevice *Device, const char *Name, const
             if (!search || (search->avoidRepeats == 0 && search->delMode == 0)) // ignore if not avoid repeats and no auto-delete 
                continue;
 
-            LogFile.Log(1,"recording started '%s' on device %d (search timer '%s')", Name, Device->CardIndex(), search->search);
+            bool vpsUsed = ti->HasFlags(tfVps) && ti->Event() && ti->Event()->Vps();
+            LogFile.Log(1,"recording started '%s' on device %d (search timer '%s'); VPS used: %s", Name, Device->CardIndex(), search->search, vpsUsed ? "Yes": "No");
             const cEvent* event = ti->Event();
             if (!event)
             {
@@ -91,7 +93,7 @@ void cRecStatusMonitor::Recording(const cDevice *Device, const char *Name, const
                continue;
             }
             time_t now = time(NULL);
-            if (now < ti->StartTime() + 60) // allow a delay of one minute
+            if (vpsUsed || now < ti->StartTime() + 60) // allow a delay of one minute
             {
                timerObj->recDone = new cRecDone(ti, event, search);
                return;
@@ -128,13 +130,26 @@ void cRecStatusMonitor::Recording(const cDevice *Device, const char *Name, const
                // check if recording has ended before timer end
 
                bool complete = true;
-               if (now < tiR->timer->StopTime())
+	       cRecording *pRecording = Recordings.GetByName(Filename);
+	       long timerLengthSecs = tiR->timer->StopTime()-tiR->timer->StartTime();
+	       int recFraction = 100;
+	       if (pRecording && timerLengthSecs)
+	       {
+		  int recLen = RecLengthInSecs(pRecording);
+		  recFraction = double(recLen) * 100 / timerLengthSecs;
+	       }
+	       bool vpsUsed = tiR->timer->HasFlags(tfVps) && tiR->timer->Event() && tiR->timer->Event()->Vps();
+               if ((!vpsUsed && now < tiR->timer->StopTime()) || recFraction < (vpsUsed ? 90: 98)) // assure timer has reached its end or at least 98% were recorded
                {
                   complete = false;
-                  LogFile.Log(1,"finished: '%s' (not complete!); search timer: '%s'", tiR->timer->File(), search->search);
+                  LogFile.Log(1,"finished: '%s' (not complete! - recorded only %d%%); search timer: '%s'; VPS used: %s", tiR->timer->File(), recFraction, search->search, vpsUsed ? "Yes": "No");
                }
                else
-                  LogFile.Log(1,"finished: '%s'; search timer: '%s'", tiR->timer->File(), search->search);
+	       {
+                  LogFile.Log(1,"finished: '%s'; search timer: '%s'; VPS used: %s", tiR->timer->File(), search->search, vpsUsed ? "Yes": "No");
+		  if (recFraction < 100)
+		    LogFile.Log(2,"recorded %d%%'", recFraction);
+	       }
                if (complete)
                {
                   RecsDone.Add(tiR->recDone);
@@ -148,7 +163,8 @@ void cRecStatusMonitor::Recording(const cDevice *Device, const char *Name, const
                   // check for search timers to delete automatically
                   SearchExts.CheckForAutoDelete(search);
 
-                  // trigger a search timer update
+                  // trigger a search timer update (skip running events)
+		  search->skipRunningEvents = true;
                   updateForced = 1;			
                }
                else if (tiR->lastBreak == 0) // store first break
@@ -184,3 +200,67 @@ int cRecStatusMonitor::TimerRecDevice(cTimer* timer)
       if (tiR->timer == timer && timer->Recording()) return tiR->deviceNr+1;
    return 0;
 }
+
+bool cRecStatusMonitor::IsPesRecording(cRecording *pRecording)
+{
+#if VDRVERSNUM < 10703
+  return true;
+#else
+  return pRecording && pRecording->IsPesRecording();
+#endif
+}
+
+#define LOC_INDEXFILESUFFIX     "/index"
+
+#if VDRVERSNUM < 10703
+
+int cRecStatusMonitor::RecLengthInSecs(cRecording *pRecording)
+{
+  struct stat buf;
+  cString fullname = cString::sprintf("%s%s", pRecording->FileName(), "/index.vdr");
+  if (stat(fullname, &buf) == 0) 
+  {      
+    struct tIndex { int offset; uchar type; uchar number; short reserved; };
+    int delta = buf.st_size % sizeof(tIndex);
+    if (delta) 
+    {
+      delta = sizeof(tIndex) - delta;
+      esyslog("ERROR: invalid file size (%ld) in '%s'", buf.st_size, *fullname);
+    }
+    return (buf.st_size + delta) / sizeof(tIndex) / SecondsToFrames(1);
+  }
+  else 
+    return -1;
+}
+
+#else
+
+struct tIndexTs {
+  uint64_t offset:40; // up to 1TB per file (not using off_t here - must definitely be exactly 64 bit!)
+  int reserved:7;     // reserved for future use
+  int independent:1;  // marks frames that can be displayed by themselves (for trick modes)
+  uint16_t number:16; // up to 64K files per recording
+  tIndexTs(off_t Offset, bool Independent, uint16_t Number)
+  {
+    offset = Offset;
+    reserved = 0;
+    independent = Independent;
+    number = Number;
+  }
+  };
+
+int cRecStatusMonitor::RecLengthInSecs(cRecording *pRecording)
+{
+  struct stat buf;
+  cString fullname = cString::sprintf("%s%s", pRecording->FileName(), IsPesRecording(pRecording) ? LOC_INDEXFILESUFFIX ".vdr" : LOC_INDEXFILESUFFIX);
+  if (pRecording->FileName() && *fullname && access(fullname, R_OK) == 0 && stat(fullname, &buf) == 0)
+    {
+      double frames = buf.st_size ? (buf.st_size - 1) / sizeof(tIndexTs) + 1 : 0;
+      double Seconds = 0;
+      modf((frames + 0.5) / pRecording->FramesPerSecond(), &Seconds);
+      return Seconds;
+    }
+  return -1;
+}
+#endif
+
