@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -22,6 +22,9 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 */
 
 #include <vector>
+#ifdef __FreeBSD__
+#include <netinet/in.h>
+#endif
 #include "searchtimer_thread.h"
 #include "epgsearchcfg.h"
 #include "epgsearchcats.h"
@@ -41,11 +44,10 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "menu_deftimercheckmethod.h"
 #include "timerstatus.h"
 #include "epgsearch.h"
+#include <vdr/eitscan.h>
 
 // priority for background thread
 #define SEARCHTIMER_NICE 19
-// delay before first thread activity
-#define SEARCHTIMER_WAIT 20
 
 #define DAYBUFFERSIZE 32
 
@@ -53,6 +55,7 @@ extern int updateForced;
 
 cSearchTimerThread *cSearchTimerThread::m_Instance = NULL;
 cSearchResults cSearchTimerThread::announceList;
+bool cSearchTimerThread::justRunning = false;
 
 cSearchTimerThread::cSearchTimerThread(cPluginEpgsearch* thePlugin)
    : cThread("EPGSearch: searchtimer")
@@ -91,6 +94,7 @@ void cSearchTimerThread::Exit(void) {
 
 void cSearchTimerThread::Stop(void) {
    m_Active = false;
+   Wait.Signal();
    Cancel(6);
 }
 
@@ -157,8 +161,10 @@ cTimer *cSearchTimerThread::GetTimer(cSearchExt *searchExt, const cEvent *pEvent
          if (tmStartEv->tm_mday != tmStartTi->tm_mday)
             continue;
 	    
-         bTimesMatchExactly = (tStart == eStart && tStop == eStop);
-	
+	 // some providers change EPG times only for a few seconds
+	 // ignore this to avoid search timer mails because of such changes
+         bTimesMatchExactly = (abs(tStart -eStart) < 60 && abs(tStop - eStop) < 60);
+
          if (abs(tStart - eStart) < AllowedDiff * 60 && abs(tStop - eStop) < AllowedDiff * 60) // accept a difference of max 10 min., but only if the event duration is more than 10 minutes
             return ti;
       }
@@ -178,7 +184,7 @@ bool cSearchTimerThread::TimerWasModified(cTimer* t)
    {
       time_t StartTime = time_t(atol(start));
       time_t StopTime = time_t(atol(stop));
-      if (t->StartTime() != StartTime || t->StopTime() != StopTime)
+      if (abs(t->StartTime() - StartTime) >= 60 || abs(t->StopTime() -StopTime) >= 60)
          bMod = true;
    }
    if (start) free(start);
@@ -197,8 +203,10 @@ void cSearchTimerThread::Action(void)
 
    m_Active = true;
    // let VDR do its startup
-   for(int wait = 0; wait < SEARCHTIMER_WAIT && m_Active; wait++)
-      sleepSec(1);
+   if (!cPluginEpgsearch::VDR_readyafterStartup)
+     LogFile.Log(2, "SearchTimerThread: waiting for VDR to become ready...");
+   while(m_Active && !cPluginEpgsearch::VDR_readyafterStartup)
+      Wait.Wait(1000);
 
    time_t nextUpdate = time(NULL);
    while (m_Active) 
@@ -207,9 +215,22 @@ void cSearchTimerThread::Action(void)
       bool needUpdate = NeedUpdate();
       if (now >= nextUpdate || needUpdate)
       {
+	 justRunning = true;
+
+	 if (updateForced & UPDS_WITH_EPGSCAN)
+	 {
+  	    LogFile.Log(1,"starting EPG scan before search timer update");
+	    EITScanner.ForceScan();
+	    do
+	    {
+	       Wait.Wait(1000);
+	    }
+	    while(EITScanner.Active() && m_Active);
+  	    LogFile.Log(1,"EPG scan finished");
+	 }
          if (Timers.BeingEdited()) 		
          {
-            sleepSec(1);
+            Wait.Wait(1000);
             continue;
          }
          LogFile.iSysLog("search timer update started");
@@ -263,6 +284,15 @@ void cSearchTimerThread::Action(void)
                if ((file = searchExt->BuildFile(pEvent)) != NULL)
                {
                   while(strstr(file, "!^pipe^!")) file = strreplace(file, "!^pipe^!", "|"); // revert the translation of '|' in BuildFile 
+                  if (strstr(file, "!^invalid^!") || strlen(file) == 0)
+                  {
+                     LogFile.eSysLog("Skipping timer due to invalid or empty filename");
+                     if (time(NULL) <= timer->StopTime())
+                        pOutdatedTimers->DelTimer(timer);
+                     delete timer;
+                     free(file);
+                     continue;
+                  }
                   timer->SetFile(file);
                   free(file);
                }
@@ -351,7 +381,7 @@ void cSearchTimerThread::Action(void)
                         if (timerMod & tmStartStop)
 			  LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : start/stop has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
                         if (timerMod & tmFile)
-			  LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : title and/or episdode has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
+			  LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : title and/or episdode has changed (old: %s, new: %s", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent), timer?timer->File():"", pFile);
                         if (timerMod & tmAuxEventID)
 			  LogFile.Log(3,"timer for '%s~%s' (%s - %s, channel %d) : aux info for event id has changed", pEvent->Title()?pEvent->Title():"no title", pEvent->ShortText()?pEvent->ShortText():"no subtitle", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), ChannelNrFromEvent(pEvent));
 		      }
@@ -378,33 +408,57 @@ void cSearchTimerThread::Action(void)
                   }
                }
 		
-               if (searchExt->action == searchTimerActionAnnounceOnly) 
+               if (searchExt->action == searchTimerActionAnnounceViaOSD) 
                {
                   if (t || // timer already exists or
                       NoAnnounces.InList(pEvent) || // announcement not wanted anymore or
-                      (EPGSearchConfig.noAnnounceWhileReplay && cDevice::PrimaryDevice()->Replaying() && updateForced != 2)  // no announce while replay within automatic updates
+                      (EPGSearchConfig.noAnnounceWhileReplay && 
+		       cDevice::PrimaryDevice()->Replaying() && 
+		       !(updateForced & UPDS_WITH_OSD))  // no announce while replay within automatic updates
                      ) 
                   {
                      if (Summary) free(Summary);
                      delete timer;
                      continue;
                   }
-                  announceList.Add(new cSearchResult(pEvent, searchExt));
+		  if (!announceList.Lookup(pEvent))
+		    announceList.Add(new cSearchResult(pEvent, searchExt->ID));
 
                   if (Summary) free(Summary);
                   delete timer;
                   continue;
                }
 
-               if (searchExt->action == searchTimerActionSwitchOnly) // add to switch list
+               if (searchExt->action == searchTimerActionAnnounceViaMail) 
+               {
+                  if (t || // timer already exists or
+                      NoAnnounces.InList(pEvent) ||
+		      pEvent->StartTime() < time(NULL)) // already started? 
+                  {
+                     if (Summary) free(Summary);
+                     delete timer;
+                     continue;
+                  }
+		  mailNotifier.AddAnnounceEventNotification(pEvent->EventID(), pEvent->ChannelID(), searchExt->ID); 
+
+                  if (Summary) free(Summary);
+                  delete timer;
+                  continue;
+               }
+               if (searchExt->action == searchTimerActionSwitchOnly || 
+		   searchExt->action == searchTimerActionAnnounceAndSwitch) // add to switch list
                {
                   time_t now = time(NULL);
                   if (now < pEvent->StartTime())
                   {
                      if (!SwitchTimers.InSwitchList(pEvent))
                      {			
-                        cMutexLock SwitchTimersLock(&SwitchTimers);	
-                        SwitchTimers.Add(new cSwitchTimer(pEvent, searchExt->switchMinsBefore, 0, 
+                        cMutexLock SwitchTimersLock(&SwitchTimers);
+			int mode = 0;
+			if (searchExt->action == searchTimerActionAnnounceAndSwitch)
+			  mode = 2;
+			LogFile.Log(3,"adding switch timer event for '%s~%s' (%s - %s); search timer: '%s'", pEvent->Title(), pEvent->ShortText()?pEvent->ShortText():"", GETDATESTRING(pEvent), GETTIMESTRING(pEvent), searchExt->search);
+                        SwitchTimers.Add(new cSwitchTimer(pEvent, searchExt->switchMinsBefore, mode, 
 							  searchExt->unmuteSoundOnSwitch));
                         SwitchTimers.Save();
                         cSwitchTimerThread::Init(); 
@@ -425,8 +479,10 @@ void cSearchTimerThread::Action(void)
                if (Summary) free(Summary);
                delete timer;
             }
+	    delete pSearchResults;
             searchExt = localSearchExts->Next(searchExt);
          }
+
          if (localSearchExts) delete localSearchExts;
 
          if (pOutdatedTimers)
@@ -450,6 +506,7 @@ void cSearchTimerThread::Action(void)
                   if (TimerWasModified(t)) continue;
                   if (!t->Event()) continue; // if there is no event, we keep the timer, since EPG could have been cleared
                   if (time(NULL) > t->StopTime()) continue; // if this timer has (just) finished, let VDR do the cleanup
+                  if (t->Recording()) continue; // do not remove recording timers
                   LogFile.Log(1,"delete timer for '%s' (%s, channel %s)", t->File(), DAYDATETIME(t->StartTime()), CHANNELNAME(t->Channel()));
                   RemoveTimer(t, t->Event());
                }
@@ -471,10 +528,12 @@ void cSearchTimerThread::Action(void)
 	     }
          }
 
+	 CheckEPGHours();
+
          LogFile.iSysLog("search timer update finished");
 
          // check for conflicts
-         if (EPGSearchConfig.checkTimerConflictsAfterUpdate)
+         if (EPGSearchConfig.checkTimerConflictsAfterUpdate && m_Active)
          {
             LogFile.iSysLog("check for timer conflicts");
             cConflictCheck conflictCheck;   
@@ -488,13 +547,15 @@ void cSearchTimerThread::Action(void)
                   mailNotifier.SendConflictNotifications(conflictCheck);
                }
 
+ 	       conflictCheck.EvaluateConflCheckCmd();
+
                cString msgfmt = cString::sprintf(tr("%d timer conflict(s)! First at %s. Show them?"), 
 						 conflictCheck.relevantConflicts,
 						 *DateTime(conflictCheck.nextRelevantConflictDate));
                bool doMessage = EPGSearchConfig.noConflMsgWhileReplay == 0 || 
                   !cDevice::PrimaryDevice()->Replaying() || 
                   conflictCheck.nextRelevantConflictDate - now < 2*60*60 ||
-		 updateForced == 2;
+		 (updateForced & UPDS_WITH_OSD);
                if (doMessage && SendMsg(msgfmt, true,7) == kOk)			
                {
                   m_plugin->showConflicts = true;
@@ -511,9 +572,10 @@ void cSearchTimerThread::Action(void)
          // check for updates for manual timers
          CheckManualTimers();
 
-         mailNotifier.SendUpdateNotifications();
+	 if (m_Active)
+	   mailNotifier.SendUpdateNotifications();
 
-         if (updateForced == 2)
+         if ((updateForced & UPDS_WITH_OSD) && m_Active)
             SendMsg(tr("Search timer update done!"));
 
          // reset service call flag
@@ -521,10 +583,12 @@ void cSearchTimerThread::Action(void)
 
          m_lastUpdate = time(NULL);
          nextUpdate = long(m_lastUpdate/60)*60 + (EPGSearchConfig.UpdateIntervall * 60);
+	 justRunning = false;
       }
-      sleepSec(2); // to avoid high system load if time%30==0 
+      if (m_Active)
+	Wait.Wait(2000); // to avoid high system load if time%30==0 
       while (m_Active && !NeedUpdate() && time(NULL)%30 != 0) // sync heart beat to a multiple of 5secs
-         sleepSec(1);
+         Wait.Wait(1000);
    };
    LogFile.iSysLog("Leaving search timer thread");
 }
@@ -547,6 +611,10 @@ char* cSearchTimerThread::SummaryExtended(cSearchExt* searchExt, cTimer* Timer,
       eStop = pEvent->EndTime();
    else
       eStop = pEvent->Vps() + pEvent->Duration();
+   // make sure that eStart and eStop represent a full minute
+   eStart = (eStart / 60) * 60;
+   eStop = (eStop / 60) * 60;
+
    time_t start = eStart - (UseVPS?0:(searchExt->MarginStart * 60));
    time_t stop  = eStop + (UseVPS?0:(searchExt->MarginStop * 60));
 
@@ -565,8 +633,8 @@ char* cSearchTimerThread::SummaryExtended(cSearchExt* searchExt, cTimer* Timer,
    if (!isempty(aux))
    {
       tmpaux = strdup(aux);
-      char* begin = strstr(aux, "<epgsearch>");
-      char* end = strstr(aux, "</epgsearch>");
+      const char* begin = strstr(aux, "<epgsearch>");
+      const char* end = strstr(aux, "</epgsearch>");
       if (begin && end)
       {
          if (begin == aux) strcpy(tmpaux, ""); else strn0cpy(tmpaux, aux, begin-aux+1);
@@ -653,25 +721,27 @@ bool cSearchTimerThread::AddModTimer(cTimer* Timer, int index, cSearchExt* searc
                Timer->File(),
                tmpSummary?tmpSummary:"");
 
-   SendViaSVDRP(cmdbuf);
+   if (!SendViaSVDRP(cmdbuf))
+     return false;
+   
    if (gl_timerStatusMonitor) gl_timerStatusMonitor->SetConflictCheckAdvised(); 
-
+   
    cTimerDone* timerdone = new cTimerDone(start, stop, pEvent, searchExt->ID);
    if (index==0)
-      TimersDone.Add(timerdone);
+     TimersDone.Add(timerdone);
    else
-      TimersDone.Update(start, stop, pEvent, searchExt->ID, timerdone);
-
+     TimersDone.Update(start, stop, pEvent, searchExt->ID, timerdone);
+   
    if (EPGSearchConfig.sendMailOnSearchtimers)
-   {
-      if (index==0) // new
+     {
+       if (index==0) // new
          mailNotifier.AddNewTimerNotification(pEvent->EventID(), pEvent->ChannelID()); 
-      else
-	mailNotifier.AddModTimerNotification(pEvent->EventID(), pEvent->ChannelID(), timerMod); 
-   }
+       else
+	 mailNotifier.AddModTimerNotification(pEvent->EventID(), pEvent->ChannelID(), timerMod); 
+     }
    free(cmdbuf);
    if (tmpSummary) free(tmpSummary);
-
+   
    return true;
 }
 
@@ -705,10 +775,15 @@ void cSearchTimerThread::CheckExpiredRecs()
    LogFile.Log(1, "check for expired recordings started");
    cThreadLock RecordingsLock(&Recordings);
    cList<cRecordingObj> DelRecordings;
-   for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording)) 
+   for (cRecording *recording = Recordings.First(); recording && m_Active; recording = Recordings.Next(recording)) 
    {
+#if APIVERSNUM < 10721 
       LogFile.Log(3, "check recording %s from %s for expiration", recording->Name(), DAYDATETIME(recording->start));
       if (recording->start == 0)
+#else
+      LogFile.Log(3, "check recording %s from %s for expiration", recording->Name(), DAYDATETIME(recording->Start()));
+      if (recording->Start() == 0)
+#endif
       {
          LogFile.Log(2, "oops, recording %s has no start time, skipped", recording->Name());
          continue;
@@ -734,25 +809,40 @@ void cSearchTimerThread::CheckExpiredRecs()
          if (searchName) free(searchName);
          continue;
       }
+      free(searchID);
+      free(searchName);
       LogFile.Log(3, "recording triggered from search timer %s", search->search);
       if (search->delAfterDays == 0) continue;
       time_t now = time(NULL);
+
+#if APIVERSNUM < 10721 
       int daysBetween = int(double((now - recording->start)) / (60*60*24));
+#else
+      int daysBetween = int(double((now - recording->Start())) / (60*60*24));
+#endif
       if (daysBetween  >= search->delAfterDays)
          DelRecordings.Add(new cRecordingObj(recording, search));
       else
          LogFile.Log(3, "recording will expire in %d days, search timer %s", search->delAfterDays - daysBetween, search->search);
    }    
-   for (cRecordingObj *recordingObj = DelRecordings.First(); recordingObj; recordingObj = DelRecordings.Next(recordingObj)) 
+   for (cRecordingObj *recordingObj = DelRecordings.First(); recordingObj && m_Active; recordingObj = DelRecordings.Next(recordingObj)) 
    {
       cRecording* recording = recordingObj->recording;
       cSearchExt* search = recordingObj->search;
       if (search->recordingsKeep > 0 && search->recordingsKeep >= search->GetCountRecordings())
       {
+#if APIVERSNUM < 10721 
          LogFile.Log(1, "recording '%s' from %s expired, but will be kept, search timer %s", recording->Name(), DAYDATETIME(recording->start), recordingObj->search->search);
+#else
+         LogFile.Log(1, "recording '%s' from %s expired, but will be kept, search timer %s", recording->Name(), DAYDATETIME(recording->Start()), recordingObj->search->search);
+#endif
          continue;
       }
+#if APIVERSNUM < 10721 
       LogFile.Log(1, "delete expired recording '%s' from %s, search timer %s", recording->Name(), DAYDATETIME(recording->start), recordingObj->search->search);
+#else
+      LogFile.Log(1, "delete expired recording '%s' from %s, search timer %s", recording->Name(), DAYDATETIME(recording->Start()), recordingObj->search->search);
+#endif
       cRecordControl *rc = cRecordControls::GetRecordControl(recording->FileName());
       if (!rc) 
       {
@@ -813,10 +903,16 @@ void cSearchTimerThread::CheckManualTimers()
    const cSchedules *schedules;
    schedules = cSchedules::Schedules(schedulesLock);
 
-   for (cTimer *ti = Timers.First(); ti; ti = Timers.Next(ti)) 
+   for (cTimer *ti = Timers.First(); ti && m_Active; ti = Timers.Next(ti)) 
    {
       if (TriggeredFromSearchTimerID(ti) != -1) continue; // manual timer?
-        
+
+      if (TimerWasModified(ti))
+      {
+	LogFile.Log(2,"timer for '%s' (%s, channel %s) modified by user - won't be touched", ti->File(), DAYDATETIME(ti->StartTime()), CHANNELNAME(ti->Channel()));
+	continue; // don't update timers modified by user
+      }
+
       char* szbstart = GetAuxValue(ti, "bstart");
       int bstart = szbstart? atoi(szbstart) : 0;
       free(szbstart);
@@ -885,7 +981,7 @@ void cSearchTimerThread::CheckManualTimers()
                LogFile.Log(3,"selected candidate is '%s~%s' (%s - %s)", event->Title(), event->ShortText()?event->ShortText():"", GETDATESTRING(event), GETTIMESTRING(event));
                if ((maxweight > 0 && event->StartTime() - bstart != ti->StartTime()) || (event->EndTime() + bstop != ti->StopTime()))
                   ModifyManualTimer(event, ti, bstart, bstop);		    
-	       else if (maxweight == 0)
+	       else if (maxweight <= 0)
 		 LogFile.Log(3,"selected candidate is too bad");
             }
             else
@@ -897,3 +993,71 @@ void cSearchTimerThread::CheckManualTimers()
    LogFile.Log(1, "manual timer check finished");
 }
 
+// check if EPG is present for the configured hours
+void cSearchTimerThread::CheckEPGHours()
+{
+  if (EPGSearchConfig.checkEPGHours <= 0 || 
+      (EPGSearchConfig.checkEPGWarnByOSD == 0 && EPGSearchConfig.checkEPGWarnByMail == 0) ||
+      EPGSearchConfig.checkEPGchannelGroupNr <= 0)
+    return;
+
+  LogFile.Log(2,"check if relevant EPG exists for the next %d hours", EPGSearchConfig.checkEPGHours);
+  cChannelGroup* channelGroup = ChannelGroups.Get(EPGSearchConfig.checkEPGchannelGroupNr-1); // not zero-based!
+  if (!channelGroup)
+    {
+      LogFile.Log(1,"channel group with index %d does not exist!", EPGSearchConfig.checkEPGchannelGroupNr);
+      return;
+    }
+
+   LogFile.Log(2,"checking channel group '%s'", channelGroup->name);
+
+  time_t checkTime = time(NULL) + EPGSearchConfig.checkEPGHours * 60 * 60;
+  
+  cSchedulesLock schedulesLock;
+  const cSchedules *schedules;
+  schedules = cSchedules::Schedules(schedulesLock);
+  
+  cChannelGroup channelsWithSmallEPG;
+  cChannelGroupItem* channelInGroup = channelGroup->channels.First();
+  while (channelInGroup) 
+    {
+      cChannel* channel = channelInGroup->channel;
+      // get the channels schedule
+      const cSchedule* schedule = schedules->GetSchedule(channel);
+      if (!schedule || !schedule->GetEventAround(checkTime))      
+	{
+	  LogFile.Log(3,"less than %d hours of EPG for channel %s!", EPGSearchConfig.checkEPGHours, channel->Name());
+	  cChannelGroupItem* channelitem = new cChannelGroupItem(channel);
+	  channelsWithSmallEPG.channels.Add(channelitem);
+	}
+      channelInGroup = channelGroup->channels.Next(channelInGroup);
+    }	
+
+  // create a string list of the channels found
+  if (channelsWithSmallEPG.channels.Count() > 0)
+    {
+      string sBuffer;
+      channelInGroup = channelsWithSmallEPG.channels.First();
+      while (channelInGroup) 
+	{
+	  cChannel* channel = channelInGroup->channel;
+	  if (channel)
+	    sBuffer += " " + string(channel->ShortName(true));
+	  channelInGroup = channelsWithSmallEPG.channels.Next(channelInGroup);
+	}	
+      
+      
+      if (EPGSearchConfig.checkEPGWarnByOSD)
+	{
+	  cString msgfmt = cString::sprintf(tr("small EPG content on:%s"), sBuffer.c_str());
+	  SendMsg(msgfmt);		   
+	}
+      if (EPGSearchConfig.checkEPGWarnByMail)
+	{
+	  cString msgfmt = cString::sprintf(tr("small EPG content on:%s"), sBuffer.c_str());
+	  cMailNotifier M(string(tr("VDR EPG check warning")), string(msgfmt));
+	}
+    }
+
+  LogFile.Log(2,"check for relevant EPG finished - %d channels with small EPG content", channelsWithSmallEPG.channels.Count());
+}
