@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -36,6 +36,7 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "epgsearchtools.h"
 #include "conflictcheck.h"
 #include "timerstatus.h"
+#include "uservars.h"
 
 std::list<std::string> cEpgsearchServiceHandler::SearchTimerList()
 {
@@ -296,13 +297,13 @@ std::list<std::string> cEpgsearchServiceHandler::TimerConflictList(bool relOnly)
 	       if (relOnly && (*it)->ignore) continue;
 	       std::ostringstream timerpart;
 	       int recPart = (*it)->recDuration * 100 / ((*it)->stop - (*it)->start);
-	       timerpart << (*it)->timer->Index()+1 << "|" << recPart << "|";
+	       timerpart << (*it)->origIndex+1 << "|" << recPart << "|";
 	       std::set<cConflictCheckTimerObj*,TimerObjSort>::iterator itcc;
 	       if ((*it)->concurrentTimers)
 		 {
 		   std::ostringstream cctimers;
 		   for (itcc = (*it)->concurrentTimers->begin(); itcc != (*it)->concurrentTimers->end(); itcc++) 
-		     cctimers << (cctimers.str().empty()?"":"#") << (*itcc)->timer->Index()+1;
+		     cctimers << (cctimers.str().empty()?"":"#") << (*itcc)->origIndex+1;
 		   timerpart << cctimers.str();
 		 }
 	       timerparts << (timerparts.str().empty()?"":":") << timerpart.str();
@@ -322,3 +323,16 @@ bool cEpgsearchServiceHandler::IsConflictCheckAdvised()
 {
   return gl_timerStatusMonitor?gl_timerStatusMonitor->ConflictCheckAdvised():false;  
 }    
+
+std::set<std::string> cEpgsearchServiceHandler::ShortDirectoryList()
+{
+  cMenuDirSelect::CreateDirSet(false);
+   return cMenuDirSelect::directorySet;
+}
+
+std::string cEpgsearchServiceHandler::Evaluate(const std::string& expr, const cEvent* event)
+{
+  if (!event) return expr;
+  cVarExpr varExpr(expr);
+  return varExpr.Evaluate(event);
+}
