@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -24,11 +24,11 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 #include "switchtimer_thread.h"
 #include "epgsearchcfg.h"
 #include "epgsearchtools.h"
+#include "epgsearch.h"
 #include <vdr/tools.h>
 #include <vdr/plugin.h>
 
-#define MSG_DELAY 5
-#define SWITCHTIMER_WAIT 20
+#define MSG_DELAY 7
 
 cSwitchTimerThread *cSwitchTimerThread::m_Instance = NULL;
 
@@ -60,6 +60,7 @@ void cSwitchTimerThread::Exit(void) {
 
 void cSwitchTimerThread::Stop(void) {
    m_Active = false;
+   Wait.Signal();
    Cancel(6);
 }
 
@@ -68,8 +69,10 @@ void cSwitchTimerThread::Action(void)
   m_Active = true;
   
   // let VDR do its startup
-  for(int wait = 0; wait < SWITCHTIMER_WAIT && m_Active; wait++)
-    sleepSec(1);
+  if (!cPluginEpgsearch::VDR_readyafterStartup)
+    LogFile.Log(2, "SwitchTimerThread: waiting for VDR to become ready...");
+  while(m_Active && !cPluginEpgsearch::VDR_readyafterStartup)
+    Wait.Wait(1000);
 
    time_t nextUpdate = time(NULL);
    while (m_Active) 
@@ -78,49 +81,73 @@ void cSwitchTimerThread::Action(void)
       if (now >= nextUpdate)
       {
          LogFile.Log(3,"locking switch timers");
-         cMutexLock SwitchTimersLock(&SwitchTimers);
+         SwitchTimers.Lock();
          LogFile.Log(3,"switch timer check started");
          cSwitchTimer* switchTimer = SwitchTimers.First();
          while (switchTimer && m_Active) 
          {
-            const cEvent* event = switchTimer->event;
-            if (event && event->StartTime() - now < switchTimer->switchMinsBefore*60 + MSG_DELAY + 1)
+            if (switchTimer->startTime - now < switchTimer->switchMinsBefore*60 + MSG_DELAY + 1)
             {
-               cChannel *channel = Channels.GetByChannelID(event->ChannelID(), true, true);
-               bool doswitch = (switchTimer->announceOnly == 0);
+               cChannel *channel = Channels.GetByChannelID(switchTimer->channelID, true, true);
+               bool doSwitch = (switchTimer->mode == 0);
+               bool doAsk = (switchTimer->mode == 2);
+	       bool doUnmute = switchTimer->unmute;
                SwitchTimers.Del(switchTimer);
-               if (channel && (event->EndTime() >= now))
-               {
-                  // announce and switch
-                  if (doswitch)
-                  {
-                     LogFile.Log(1,"switching to channel %d", channel->Number());
-                     cString cmd = cString::sprintf("CHAN %d", channel->Number());
-                     SendViaSVDRP(cmd);
 
-		     if (switchTimer->unmute && cDevice::PrimaryDevice()->IsMute())
-		       cDevice::PrimaryDevice()->ToggleMute();
-                  }
+  	       const cEvent* event = switchTimer->Event();     
+               if (event && channel && (event->EndTime() >= now))
+               {
                   cString Message = cString::sprintf("%s: %s - %s", event->Title(), 
 						     CHANNELNAME(channel), GETTIMESTRING(event));
-                  SendMsg(Message);
-                  sleepSec(MSG_DELAY);			
+		  cString SwitchCmd = cString::sprintf("CHAN %d", channel->Number());
+                  // switch
+                  if (doSwitch)
+                  {
+                     LogFile.Log(1,"switching to channel %d", channel->Number());
+		     if (cDevice::CurrentChannel() != channel->Number())
+		       SendViaSVDRP(SwitchCmd);
+
+		     if (doUnmute && cDevice::PrimaryDevice()->IsMute())
+		       cDevice::PrimaryDevice()->ToggleMute();
+                  }
+		  if (!doAsk)
+		    SendMsg(Message);
+
+		  if (doAsk)
+		    {
+		      cString Message = cString::sprintf(tr("Switch to (%d) '%s'?"), channel->Number(), event->Title());
+		      if(SendMsg(Message, true, MSG_DELAY) == kOk)
+			{
+			  LogFile.Log(1,"switching to channel %d", channel->Number());
+			  if (cDevice::CurrentChannel() != channel->Number())
+			    SendViaSVDRP(SwitchCmd);
+			  
+			  if (doUnmute && cDevice::PrimaryDevice()->IsMute())
+			    cDevice::PrimaryDevice()->ToggleMute();
+
+			}
+		    }			
+		
+                  if (m_Active)
+		    Wait.Wait(1000 * MSG_DELAY);
                }
                SwitchTimers.Save();
                break;
             }
             switchTimer = SwitchTimers.Next(switchTimer);
          }
+	 SwitchTimers.Unlock();
          LogFile.Log(3,"switch timer check finished");
-         sleepSec(MSG_DELAY);
+         if (m_Active)
+	    Wait.Wait(1000 * MSG_DELAY);
          m_lastUpdate = time(NULL);
          nextUpdate = long(m_lastUpdate/60)*60+ 60 - MSG_DELAY ; // check at each full minute minus 5sec
          if (SwitchTimers.Count() == 0)
             m_Active = false;
       }
       while (m_Active && time(NULL)%MSG_DELAY != 0) // sync heart beat to MSG_DELAY
-         sleepSec(1);
-      sleepSec(1);
+	Wait.Wait(1000);
+      Wait.Wait(1000);
    };
    m_Instance = NULL;
 }
