@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -107,6 +107,8 @@ bool cTimerDone::Parse(const char *s)
                   break;
                case 6:  shorttext = value;
                   break;
+	       default:
+		 break;
             } //switch
          }
          parameter++;
@@ -197,3 +199,15 @@ void cTimersDone::Update(const time_t Start, const time_t Stop, const cEvent* pE
    else
       Add(Timerdone);
 }
+
+void cTimersDone::RemoveEntriesOfSearch(const int SearchID)
+{
+   cTimerDone* timerdone = First();
+   while (timerdone) 
+   {
+      cTimerDone* timerdoneNext = Next(timerdone);
+      if (timerdone->searchID == SearchID)
+	Del(timerdone);
+      timerdone = timerdoneNext;
+   }	
+}
