@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -22,10 +22,15 @@ The project's page is at http://winni.vdr-developer.org/epgsearch
 */
 
 #include <algorithm>
+#ifdef __FreeBSD__
+#include <netinet/in.h>
+#endif
 #include "uservars.h"
 #include "epgsearchcats.h"
 #include "epgsearchtools.h"
 #include "log.h"
+#include <sys/socket.h>
+#include <netdb.h>
 
 cUserVars UserVars;
 
@@ -40,12 +45,16 @@ cUserVar::cUserVar()
 
 string cUserVar::Evaluate(const cEvent* e, bool escapeStrings)
 {
-  if (oldEvent && oldEvent == e && oldescapeStrings == escapeStrings)
+   if (oldEvent && oldEvent == e && oldescapeStrings == escapeStrings)
     return oldResult;
    usedVars.clear();
    string result;
    if (IsShellCmd())
       result = EvaluateShellCmd(e);
+   else if (IsConnectCmd())
+     result = EvaluateConnectCmd(e);
+   else if (IsLengthCmd())
+     result = EvaluateLengthCmd(e);
    else if (IsCondExpr())
       result = EvaluateCondExpr(e);
    else
@@ -81,6 +90,52 @@ string cUserVar::EvaluateShellCmd(const cEvent* e)
    return result;
 }
 
+#define MAX_LINE 1000
+string cUserVar::EvaluateConnectCmd(const cEvent* e)
+{
+   if (varparser.connectAddr == "") return "";
+
+   int       conn_s;                /*  connection socket         */
+   struct    sockaddr_in servaddr;  /*  socket address structure  */
+   char      buffer[MAX_LINE];      /*  character buffer          */
+ 
+   if ( (conn_s = socket(AF_INET, SOCK_STREAM, 0)) < 0 ) 
+   {
+     LogFile.eSysLog("Error creating listening socket");
+     return "";
+   }
+
+   memset(&servaddr, 0, sizeof(varparser.connectAddr.c_str()));
+   servaddr.sin_family      = AF_INET;
+   servaddr.sin_port        = htons(varparser.connectPort);
+
+   if (getAddrFromString(varparser.connectAddr.c_str(), &servaddr) != 0)
+   {
+     LogFile.eSysLog("Invalid remote address");
+     return "";
+   }
+
+   if ( connect(conn_s, (struct sockaddr *) &servaddr, sizeof(servaddr) ) < 0 ) 
+   {
+     LogFile.eSysLog("Error calling connect()");
+     return "";
+   }
+
+   varparser.compExpr = varparser.cmdArgs;
+   string resexp = EvaluateCompExpr(e, true);
+   sprintf(buffer, "%s\n", resexp.c_str());
+   Writeline(conn_s, buffer, strlen(buffer));
+   Readline(conn_s, buffer, MAX_LINE-1);
+
+   close(conn_s);
+   return buffer;
+}
+
+string cUserVar::EvaluateLengthCmd(const cEvent* e)
+{
+   return NumToString(EvaluateCompExpr(e, false).size());
+}
+
 string cUserVar::EvaluateCondExpr(const cEvent* e, bool escapeStrings)
 {
    string condresult = "";
@@ -194,6 +249,7 @@ string cUserVar::EvaluateInternalTimerVars(const string& Expr, const cTimer* t)
    {
       string varName = tvar->second->Name();
       int varPos = 0;
+
       while((varPos = FindIgnoreCase(expr, varName)) >= 0)
       {
          expr.replace(varPos, varName.size(), tvar->second->Evaluate(t));
