@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2004-2008 Christian Wieninger
+Copyright (C) 2004-2011 Christian Wieninger
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -40,9 +40,6 @@ using std::set;
 using std::map;
 using std::ostringstream;
 
-string NumToString(long num);
-int FindIgnoreCase(const string& expr, const string& query);
-
 class cUserVar : public cListObject {
     const cEvent* oldEvent; // cache 
     bool oldescapeStrings; 
@@ -51,6 +48,8 @@ class cUserVar : public cListObject {
     string EvaluateCondExpr(const cEvent* e, bool escapeStrings = false);
     string EvaluateCompExpr(const cEvent* e, bool escapeStrings = false);
     string EvaluateShellCmd(const cEvent* e);
+    string EvaluateConnectCmd(const cEvent* e);
+    string EvaluateLengthCmd(const cEvent* e);
 public:
     cUserVar();
     cVarParser varparser;
@@ -66,6 +65,8 @@ public:
     virtual string Name() { return varparser.varName; }
     virtual bool IsCondExpr() { return varparser.IsCondExpr(); }
     virtual bool IsShellCmd() { return varparser.IsShellCmd(); }
+    virtual bool IsConnectCmd() { return varparser.IsConnectCmd(); }
+    virtual bool IsLengthCmd() { return varparser.IsLengthCmd(); }
     bool DependsOnVar(const string& varName);
     bool DependsOnVar(cUserVar* var);
     bool AddDepVar(cUserVar* var);
@@ -168,6 +169,24 @@ public:
       else return ""; }
 };
 
+class cLiveEventIDVar : public cInternalVar {
+public:
+    cLiveEventIDVar() : cInternalVar("%liveeventid%") {}
+    string Evaluate(const cEvent* e, bool escapeStrings = false) 
+	{ 
+	    if (!e) return "";
+	    cChannel *channel = Channels.GetByChannelID(e->ChannelID(), true);
+	    if (!channel) return "";
+	    
+	    string res(channel->GetChannelID().ToString());
+	    res = "event_" + res;
+	    res = ReplaceAll(res, ".", "p");
+	    res = ReplaceAll(res, "-", "m");
+	    res += "_" + NumToString(e->EventID());
+	    if (escapeStrings) return "'" + EscapeString(res) + "'"; else return res;
+	}
+};
+
 class cTimeVar : public cInternalVar {
 public:
     cTimeVar() : cInternalVar("%time%") {}
@@ -303,6 +322,81 @@ public:
 	}
 };
 
+class cDateISOVar : public cInternalVar {
+public:
+    cDateISOVar() : cInternalVar("%date_iso%") {}
+    string Evaluate(const cEvent* e, bool escapeStrings = false) 
+	{ 
+	    if (!e) return "";
+	    char dateISO[11] = "";
+	    struct tm tm_r;
+	    const time_t t = e->StartTime();
+	    tm *tm = localtime_r(&t, &tm_r);
+	    strftime(dateISO, sizeof(dateISO), "%Y-%m-%d", tm);
+	    if (escapeStrings) return "'" + EscapeString(dateISO) + "'"; else return dateISO;
+	}
+};
+
+class cYearVar : public cInternalVar {
+public:
+    cYearVar() : cInternalVar("%year%") {}
+    string Evaluate(const cEvent* e, bool escapeStrings = false) 
+	{ 
+	    if (!e) return "";
+	    char year[5] = "";
+	    struct tm tm_r;
+	    const time_t t = e->StartTime();
+	    tm *tm = localtime_r(&t, &tm_r);
+	    strftime(year, sizeof(year), "%Y", tm);
+	    if (escapeStrings) return "'" + EscapeString(year) + "'"; else return year;
+	}
+};
+
+class cMonthVar : public cInternalVar {
+public:
+    cMonthVar() : cInternalVar("%month%") {}
+    string Evaluate(const cEvent* e, bool escapeStrings = false) 
+	{ 
+	    if (!e) return "";
+	    char month[3] = "";
+	    struct tm tm_r;
+	    const time_t t = e->StartTime();
+	    tm *tm = localtime_r(&t, &tm_r);
+	    strftime(month, sizeof(month), "%m", tm);
+	    if (escapeStrings) return "'" + EscapeString(month) + "'"; else return month;
+	}
+};
+
+class cDayVar : public cInternalVar {
+public:
+    cDayVar() : cInternalVar("%day%") {}
+    string Evaluate(const cEvent* e, bool escapeStrings = false) 
+	{ 
+	    if (!e) return "";
+	    char day[3] = "";
+	    struct tm tm_r;
+	    const time_t t = e->StartTime();
+	    tm *tm = localtime_r(&t, &tm_r);
+	    strftime(day, sizeof(day), "%d", tm);
+	    if (escapeStrings) return "'" + EscapeString(day) + "'"; else return day;
+	}
+};
+
+class cWeekVar : public cInternalVar {
+public:
+    cWeekVar() : cInternalVar("%week%") {}
+    string Evaluate(const cEvent* e, bool escapeStrings = false) 
+	{ 
+	    if (!e) return "";
+	    char day[3] = "";
+	    struct tm tm_r;
+	    const time_t t = e->StartTime();
+	    tm *tm = localtime_r(&t, &tm_r);
+	    strftime(day, sizeof(day), "%V", tm);
+	    if (escapeStrings) return "'" + EscapeString(day) + "'"; else return day;
+	}
+};
+
 class cChannelNrVar : public cInternalVar {
 public:
     cChannelNrVar() : cInternalVar("%chnr%") {}
@@ -349,6 +443,34 @@ class cChannelDataVar : public cInternalVar {
 	}
 };
 
+class cChannelGroupVar : public cInternalVar {
+public:
+    cChannelGroupVar() : cInternalVar("%chgrp%") {}
+    string Evaluate(const cEvent* e, bool escapeStrings = false) 
+	{ 
+	    if (!e) return "";
+	    ostringstream os;
+	    cChannel *channel = Channels.GetByChannelID(e->ChannelID(), true);
+	    while(channel && !channel->GroupSep())
+	      channel = Channels.Prev(channel);
+	    if (!channel || !channel->Name()) return "";
+	    string grpName = channel->Name();
+	    if (escapeStrings) return "'" + EscapeString(grpName) + "'"; else return grpName;
+	}
+};
+
+class cNEWTCmdVar : public cInternalVar {
+ public:
+    cNEWTCmdVar() : cInternalVar("%newtcmd%") {}
+    string Evaluate(const cEvent* e, bool escapeStrings = false) 
+	{ 
+	    if (!e) return "";
+	    cTimer* timer = new cTimer(e);
+	    string newtCmd =  *(timer->ToText());
+	    if (escapeStrings) return "'" + EscapeString(newtCmd) + "'"; else return newtCmd;
+	}
+};
+
 // independet variables
 class cColonVar : public cInternalVar {
  public:
@@ -387,6 +509,20 @@ public:
 	}
 };
 
+class cDateISONowVar : public cInternalVar {
+public:
+    cDateISONowVar() : cInternalVar("%date_iso_now%") {}
+    string Evaluate(const cEvent*, bool escapeStrings = false) 
+	{ 
+	    char dateISO[11] = "";
+	    struct tm tm_r;
+	    const time_t t = time(NULL);
+	    tm *tm = localtime_r(&t, &tm_r);
+	    strftime(dateISO, sizeof(dateISO), "%Y-%m-%d", tm);
+	    if (escapeStrings) return "'" + EscapeString(dateISO) + "'"; else return dateISO;
+	}
+};
+
 class cTimeNowVar : public cInternalVar {
 public:
     cTimeNowVar() : cInternalVar("%timenow%") {}
@@ -529,6 +665,24 @@ class cTimerSearchIDVar : public cTimerVar {
 	}
 };
 
+class cTimerLiveIDVar : public cTimerVar {
+public:
+    cTimerLiveIDVar() : cTimerVar("%timer.liveid%") {}
+    string Evaluate(const cTimer* t) 
+	{ 
+	    if (!t || !t->Channel()) return "";
+	    ostringstream builder;
+	    builder << *(t->Channel()->GetChannelID().ToString()) << ":" << t->WeekDays() << ":"
+				<< t->Day() << ":" << t->Start() << ":" << t->Stop();	    
+	    string res = builder.str();
+	    res = "timer_" + res;
+	    res = ReplaceAll(res, ".", "p");
+	    res = ReplaceAll(res, "-", "m");
+	    res = ReplaceAll(res, ":", "c");
+	    return res;
+	}
+};
+
 // search variables
 class cSearchVar {
     const string name;
@@ -549,6 +703,16 @@ public:
 	}
 };
 
+class cSearchSeriesVar : public cSearchVar {
+public:
+    cSearchSeriesVar() : cSearchVar("%search.series%") {}
+    string Evaluate(const cSearchExt* s) 
+	{ 
+	    if (!s) return "";
+	    return NumToString(s->useEpisode);
+	}
+};
+
 class cUserVars : public cList<cUserVar> {
  public:
     cTitleVar titleVar;
@@ -556,6 +720,7 @@ class cUserVars : public cList<cUserVar> {
     cSummaryVar summaryVar;
     cHTMLSummaryVar htmlsummaryVar;
     cEventIDVar eventIDVar;
+    cLiveEventIDVar liveeventIDVar;
     cTimeVar timeVar;
     cTimeEndVar timeEndVar;
     cTime_wVar time_wVar;
@@ -565,15 +730,24 @@ class cUserVars : public cList<cUserVar> {
     cLength_Var length_Var;
     cDateVar dateVar;
     cDateShortVar dateShortVar;
+    cDateISOVar dateISOVar;
+    cYearVar yearVar;
+    cMonthVar monthVar;
+    cDayVar dayVar;
+    cWeekVar weekVar;
     cChannelNrVar chnrVar;
     cChannelShortVar chShortVar;
     cChannelLongVar chLongVar;
     cChannelDataVar chDataVar;
+    cChannelGroupVar chGroupVar;
+    cNEWTCmdVar newtCmdVar;
     cSearchQueryVar searchQueryVar;
+    cSearchSeriesVar searchSeriesVar;
 
     cColonVar colonVar;
     cDateNowVar dateNowVar;
     cDateShortNowVar dateShortNowVar;
+    cDateISONowVar dateISONowVar;
     cTimeNowVar timeNowVar;
     cVideodirVar videodirVar;
     cPlugconfdirVar plugconfdirVar;
@@ -588,6 +762,7 @@ class cUserVars : public cList<cUserVar> {
     cTimerChannelLongVar timerChLongVar;
     cTimerSearchVar timerSearchVar; 
     cTimerSearchIDVar timerSearchIDVar; 
+    cTimerLiveIDVar timerLiveIDVar;
 
     map<string, cExtEPGVar*> extEPGVars;
     set<cUserVar*> userVars;
@@ -602,6 +777,7 @@ class cUserVars : public cList<cUserVar> {
 	    internalVars[summaryVar.Name()] = &summaryVar;
 	    internalVars[htmlsummaryVar.Name()] = &htmlsummaryVar;
 	    internalVars[eventIDVar.Name()] = &eventIDVar;
+	    internalVars[liveeventIDVar.Name()] = &liveeventIDVar;
 	    internalVars[timeVar.Name()] = &timeVar;
 	    internalVars[timeEndVar.Name()] = &timeEndVar;
 	    internalVars[time_wVar.Name()] = &time_wVar;
@@ -611,14 +787,22 @@ class cUserVars : public cList<cUserVar> {
 	    internalVars[length_Var.Name()] = &length_Var;
 	    internalVars[dateVar.Name()] = &dateVar;
 	    internalVars[dateShortVar.Name()] = &dateShortVar;
+	    internalVars[dateISOVar.Name()] = &dateISOVar;
+	    internalVars[yearVar.Name()] = &yearVar;
+	    internalVars[monthVar.Name()] = &monthVar;
+	    internalVars[dayVar.Name()] = &dayVar;
+	    internalVars[weekVar.Name()] = &weekVar;
 	    internalVars[chnrVar.Name()] = &chnrVar;
 	    internalVars[chShortVar.Name()] = &chShortVar;
 	    internalVars[chLongVar.Name()] = &chLongVar;
 	    internalVars[chDataVar.Name()] = &chDataVar;
+	    internalVars[chGroupVar.Name()] = &chGroupVar;
+	    internalVars[newtCmdVar.Name()] = &newtCmdVar;
 
 	    internalVars[colonVar.Name()] = &colonVar;
 	    internalVars[dateNowVar.Name()] = &dateNowVar;
 	    internalVars[dateShortNowVar.Name()] = &dateShortNowVar;
+	    internalVars[dateISONowVar.Name()] = &dateISONowVar;
 	    internalVars[timeNowVar.Name()] = &timeNowVar;
 	    internalVars[videodirVar.Name()] = &videodirVar;
 	    internalVars[plugconfdirVar.Name()] = &plugconfdirVar;
@@ -633,8 +817,10 @@ class cUserVars : public cList<cUserVar> {
 	    internalTimerVars[timerChLongVar.Name()] = &timerChLongVar;
 	    internalTimerVars[timerSearchVar.Name()] = &timerSearchVar;
 	    internalTimerVars[timerSearchIDVar.Name()] = &timerSearchIDVar;
+	    internalTimerVars[timerLiveIDVar.Name()] = &timerLiveIDVar;
 
 	    internalSearchVars[searchQueryVar.Name()] = &searchQueryVar;
+	    internalSearchVars[searchSeriesVar.Name()] = &searchSeriesVar;
 	}
 
     void InitExtEPGVars()
